systemheader:(PE    )#                                                  PE     1
####################################################################    PE     2
##      Copyright (C) 2000, The Xtal System, The University of        # PE     3
##            Western Australia, xtal@crystal.uwa.edu.au.             # PE     4
## Distributed under the GNU General Public Licence as published by   # PE     5
## the Free Software Foundation, in the hope that it will be useful   # PE     6
## but WITHOUT ANY WARRANTY. See the GNU General Public License       # PE     7
## for more details.                                                  # PE     8
##                                                                 #    PE     9
##                        *** PATSEE ***                           #    PE    10
##                                                                 #    PE    11
## PATSEE uses integrated Patterson and direct phasing methods to  #    PE    12
## to search for a molecular fragment of known geometry. Details   #    PE    13
## of the method are given in the paper Egert & Sheldrick (1985)   #    PE    14
## Acta Cryst. A41, 262-268. This is an adaption of the PATSEE     #    PE    15
## standalone program written by Ernst Egert for the Shelx system. #    PE    16
##                                                                 #    PE    17
####################################################################    PE    18
#                                                                       PE    19
include:(XMACRO)#                                                       PE    20
##                                                                      PE    21
macro:(raiseexception:,$(XCEPT=$1;RETURN$))# Throw an exception         PE    22
macro:(catchexception:,IF(XCEPT!=0)RETURN)#                             PE    23
##   Data lists in the order stored in the QX array.....                PE    24
##                                                                      PE    25
##        MARKA+1 to STOPA  are for atom data     STEPA=18              PE    26
##        MARKS+1 to STOPS  are for symmetry data STEPS=12              PE    27
##        MARKV+1 to STOPV  are for vector data   STEPV=4               PE    28
##        MARKB+1 to STOPB  are for best FOM data STEPB=15              PE    29
##        MARKK+1 to STOPK  are for rotn keep FOM STEPK=15              PE    30
##        MARKC+1 to STOPC  are for combined FOMs STEPC=30              PE    31
##        MARKE+1 to STOPE  are for E data        STEPE=5               PE    32
##        MARKT+1 to STOPT  are for triple invars STEPT=6               PE    33
##        MARKP+1 to STOPP  are for phased E data STEPP=9               PE    34
##        MARKM+1 to STOPM  are for Patterson map STEPM=1               PE    35
##                                                                      PE    36
macro:(atomlist:,[ifelse:($1,,IA,$1)])# Define atom list pointer        PE    37
macro:(ax:,[QX(atomlist:($1)+ 1)])#    Define atom x fract coord        PE    38
macro:(ay:,[QX(atomlist:($1)+ 2)])#    Define atom y fract coord        PE    39
macro:(az:,[QX(atomlist:($1)+ 3)])#    Define atom z fract coord        PE    40
macro:(aw:,[QX(atomlist:($1)+ 4)])#    Define atom site weight          PE    41
macro:(at:,[QX(atomlist:($1)+ 5)])#    Define atom sf type              PE    42
macro:(cx:,[QX(atomlist:($1)+ 6)])#    Define atom x fix cartn coord    PE    43
macro:(cy:,[QX(atomlist:($1)+ 7)])#    Define atom y fix cartn coord    PE    44
macro:(cz:,[QX(atomlist:($1)+ 8)])#    Define atom z fix cartn coord    PE    45
macro:(rx:,[QX(atomlist:($1)+ 9)])#    Define atom x rot cartn coord    PE    46
macro:(ry:,[QX(atomlist:($1)+10)])#    Define atom y rot cartn coord    PE    47
macro:(rz:,[QX(atomlist:($1)+11)])#    Define atom z rot cartn coord    PE    48
macro:(an:,[QX(atomlist:($1)+12)])#    Define atom name                 PE    49
#                                                                       PE    50
macro:(vectlist:,[ifelse:($1,,IV,$1)])# Define vect list pointer        PE    51
macro:(vx:,[QX(vectlist:($1)+ 1)])#    Define vect x                    PE    52
macro:(vy:,[QX(vectlist:($1)+ 2)])#    Define vect y                    PE    53
macro:(vz:,[QX(vectlist:($1)+ 3)])#    Define vect z                    PE    54
macro:(vw:,[QX(vectlist:($1)+ 4)])#    Define vect weight               PE    55
#                                                                       PE    56
macro:(bestlist:,[ifelse:($1,,IB,$1)])# Define best list pointer        PE    57
macro:(b1:,[QX(bestlist:($1)+ 1)])#    Define best ang1 / tprsum        PE    58
macro:(b2:,[QX(bestlist:($1)+ 2)])#    Define best ang2 / min d         PE    59
macro:(b3:,[QX(bestlist:($1)+ 3)])#    Define best ang3 / rfom          PE    60
macro:(bf:,[QX(bestlist:($1)+ 4)])#    Define best rfom / tfom          PE    61
macro:(bn:,[QX(bestlist:($1)+ 5)])#    Define number equiv hits         PE    62
macro:(bm:,[QX(bestlist:($1)+ 6)])#    Define rot matrix/ cfom          PE    63
#                                                                       PE    64
macro:(keeplist:,[ifelse:($1,,IK,$1)])# Define keep rotn list pointer   PE    65
macro:(k1:,[QX(keeplist:($1)+ 1)])#    Define rotn angle 1              PE    66
macro:(k2:,[QX(keeplist:($1)+ 2)])#    Define rotn angle 2              PE    67
macro:(k3:,[QX(keeplist:($1)+ 3)])#    Define rotn angle 3              PE    68
macro:(kf:,[QX(keeplist:($1)+ 4)])#    Define rotn fom                  PE    69
macro:(kt:,[QX(keeplist:($1)+ 5)])#    Define twist angle degrees       PE    70
macro:(km:,[QX(keeplist:($1)+ 6)])#    Define start rotn matrix (9)     PE    71
#                                                                       PE    72
macro:(comblist:,[ifelse:($1,,IC,$1)])# Define combined list pointer    PE    73
macro:(cc:,[QX(comblist:($1)+ 6)])#    Define combined fom              PE    74
#                                                                       PE    75
macro:(elist:,[ifelse:($1,,IE,$1)])#   Define e list pointer            PE    76
macro:(eh:,[QX(elist:($1)+ 1)])#       Define elist packed hkl          PE    77
macro:(es:,[QX(elist:($1)+ 2)])#       Define elist count : /e/sine     PE    78
macro:(er:,[QX(elist:($1)+ 3)])#       Define elist r/e : eo/ec         PE    79
macro:(ec:,[QX(elist:($1)+ 4)])#       Define elist         /e/cosine   PE    80
macro:(ee:,[QX(elist:($1)+ 5)])#       Define elist evalue              PE    81
macro:(et:,[QX(elist:($1)+ 6)])#       Define elist calc evalue         PE    82
#                                                                       PE    83
macro:(triplist:,[ifelse:($1,,IT,$1)])# Define triplet list pointer     PE    84
macro:(t1:,[QX(triplist:($1)+ 1)])#    Define pointer elist hkl1        PE    85
macro:(t2:,[QX(triplist:($1)+ 2)])#    Define pointer elist hkl2        PE    86
macro:(t3:,[QX(triplist:($1)+ 3)])#    Define pointer elist hkl3        PE    87
macro:(tc:,[QX(triplist:($1)+ 4)])#    Define cosine phase shift        PE    88
macro:(ts:,[QX(triplist:($1)+ 5)])#    Define sine   phase shift        PE    89
macro:(ti:,[QX(triplist:($1)+ 6)])#    Define sign index of phi2,3      PE    90
#                                                                       PE    91
macro:(phaslist:,[ifelse:($1,,IP,$1)])# Define phase list pointer       PE    92
macro:(ph:,[QX(phaslist:($1)+ 1)])#    Define pointer phase h           PE    93
macro:(pk:,[QX(phaslist:($1)+ 2)])#    Define pointer phase k           PE    94
macro:(pl:,[QX(phaslist:($1)+ 3)])#    Define pointer phase l           PE    95
macro:(s1:,[QX(phaslist:($1)+ 4)])#    Define pointer sine   frag1      PE    96
macro:(s2:,[QX(phaslist:($1)+ 5)])#    Define pointer sine   frag2      PE    97
macro:(s3:,[QX(phaslist:($1)+ 6)])#    Define pointer sine   frag3      PE    98
macro:(c1:,[QX(phaslist:($1)+ 7)])#    Define pointer cosine frag1      PE    99
macro:(c2:,[QX(phaslist:($1)+ 8)])#    Define pointer cosine frag2      PE   100
macro:(c3:,[QX(phaslist:($1)+ 9)])#    Define pointer cosine frag3      PE   101
#                                                                       PE   102
macro:(zmatlist:,[ifelse:($1,,IZ,$1)])# Define zmatx list pointer       PE   103
macro:(dis:,[GVAL(zmatlist:($1),1)])#  Define pointer bond distance     PE   104
macro:(ang:,[GVAL(zmatlist:($1),2)])#  Define pointer bond angle        PE   105
macro:(dih:,[GVAL(zmatlist:($1),3)])#  Define pointer dihedral angle    PE   106
macro:(ns1:,[SITE(zmatlist:($1),1)])#  Define pointer atom site 1       PE   107
macro:(ns2:,[SITE(zmatlist:($1),2)])#  Define pointer atom site 2       PE   108
macro:(ns3:,[SITE(zmatlist:($1),3)])#  Define pointer atom site 3       PE   109
macro:(ns4:,[SITE(zmatlist:($1),4)])#  Define pointer atom site 4       PE   110
#                                                                       PE   111
macro:(phaserestr:, intunpak:(QX($1+1),$2, 0, 4))# 0 -  3               PE   112
macro:(phisigncode:,intunpak:(QX($1+1),$2, 4, 3))# 4 -  6               PE   113
macro:(phiinvshift:,intunpak:(QX($1+1),$2, 7, 4))# 7 - 10               PE   114
macro:(phifragment:,intunpak:(QX($1+1),$2,11, 9))# 11 - 19              PE   115
macro:(probfactor:, intunpak:(QX($1+1),$2,20,bitlen:(20)))# 20 - 31     PE   116
#                                                                       PE   117
macro:(xvectorsum:, intunpak:(QX($1+2),$2,10,10))# 10 - 10              PE   118
macro:(probfsigma:, intunpak:(QX($1+2),$2,20,bitlen:(20)))# 20 - 31     PE   119
#                                                                       PE   120
macro:(generatornum:,         $2=NINT(QX($1+2+$3)))# Word 2+n           PE   121
#                                      Macros to compare packed data    PE   122
macro:(comppakwdone:,compakwd:(QX($1+1),$2,$3,$4))# 1st inv word        PE   123
macro:(plastbitsword:,intpak:($1,$2,20,bitlen:(20)))# Pack pfactor      PE   124
macro:(putsee:,writefmt:(iofm1:,'(A)',$1(1^$2)))# Write see file line   PE   125
#                                                                       PE   126
macro:(listp:,$1=($2-1)*STEPA+MARKA)#  Point to coord list              PE   127
macro:(ifal:,IF($1-(voidflg:)<small:))#                                 PE   128
#                                                                       PE   129
extrude:(PECOMN)#                                                       PE   130
INTEGER FADR(5)#                       Addr of last frag atom           PE   131
INTEGER FCOG#                          Signal centre of gravity appli   PE   132
INTEGER FTYP(5)#                       Fragment type 1=fix/2=r&t/3=r    PE   133
INTEGER GLIS#                          List geometry flag               PE   134
INTEGER IENV(50)#                      Address array envelope atoms     PE   135
INTEGER OFF(3)#                        Map offsets along each axis      PE   136
INTEGER IATM(3)#                       Start of atom list by search     PE   137
INTEGER ICEN#                          Centro frag (1=noncent; 0=cent   PE   138
INTEGER JATM(3)#                       Stop  of atom list by search     PE   139
INTEGER IXYZ#                          Xyz pointer 1=fract/6=cart       PE   140
INTEGER MARKA,STEPA,STOPA,IA#          Qx markers atom list             PE   141
INTEGER MARKB,STEPB,STOPB,IB,JB#       Qx markers best fom list         PE   142
INTEGER STOPBR,STOPBT#                 Qx markers best fom list         PE   143
INTEGER MARKE,STEPE,STOPE,IE#          Qx markers e list                PE   144
INTEGER MARKK,STEPK,STOPK,IK,JK#       Qx markers keep rotn fom list    PE   145
INTEGER MARKC,STEPC,STOPC,IC,JC#       Qx markers combined fom list     PE   146
INTEGER MARKM,STEPM,STOPM#             Qx markers patterson map         PE   147
INTEGER MARKP,STEPP,STOPP,IP#          Qx markers e phase list          PE   148
INTEGER MARKT,STEPT,STOPT,IT#          Qx markers triple invariant li   PE   149
INTEGER MARKS,STEPS,STOPS,STOP1,STOP2# Qx markers symmetry list         PE   150
INTEGER MARKV,STEPV,STOPV,IV,JV,KV#    Qx markers vector list           PE   151
INTEGER MAXT#                          Max triplets per generator       PE   152
INTEGER NATM(3)#                       Number of fix/r&t/t atoms        PE   153
INTEGER NENV(3)#                       Number atoms for envelope        PE   154
INTEGER NLAT#                          No. of lattice translations      PE   155
INTEGER NORA#                          Number orienting atoms           PE   156
INTEGER NFRG#                          Number of fragments              PE   157
INTEGER NSYM#                          Number of symmetry operators     PE   158
INTEGER NREF#                          Number of rot refine tries       PE   159
INTEGER NTPR#                          Triple sum application flag      PE   160
INTEGER NUME#                          Max e values in trans search     PE   161
INTEGER NUMT#                          Total number of unique triples   PE   162
INTEGER NZ#                            Number of zmatrx lines           PE   163
INTEGER SITE(100,4)#                   Zmatrx sites reference numbers   PE   164
INTEGER RTRY#                          Number of rotation searches      PE   165
INTEGER RSOL#                          Number of rotation solutions     PE   166
INTEGER RLIS#                          Signal rotation vectors listed   PE   167
INTEGER RMIR#                          Signal rotation search of mirr   PE   168
INTEGER ROTN#                          Rotation search on/off flag      PE   169
INTEGER RNUM#                          Rotation random number seed      PE   170
INTEGER TNUM#                          Translation random number seed   PE   171
INTEGER TTRY#                          Number transl'n searches         PE   172
INTEGER TSOL#                          Number transl'n solutions        PE   173
INTEGER TLIS#                          Transl'n search trip listing     PE   174
INTEGER TRAN#                          Transl'n search on/off flag      PE   175
INTEGER SKIP#                          Twist skip flag                  PE   176
INTEGER TWIS#                          First atom addr of twist frag    PE   177
INTEGER WMIN#                          Twist twist angle minimum        PE   178
INTEGER WMAX#                          Twist twist angle maximum        PE   179
INTEGER WINC#                          Twist twist angle delta          PE   180
INTEGER WTES#                          Twist twist angle test           PE   181
INTEGER XCEPT#                         Exception flag                   PE   182
#                                                                       PE   183
REAL AMIN#                             Minimum triplet a value          PE   184
REAL ARAD(10)#                         Atom bonding radii               PE   185
REAL ANUM(10)#                         Atom number z                    PE   186
REAL CELL(6)#                          Cell dimensions                  PE   187
REAL DCOG(3)#                          Centre of gravity shift          PE   188
REAL DTOR#                             Convert degrees to radians       PE   189
REAL EMIN#                             Minimum reflection e value       PE   190
REAL FTOC(9,5)#                        Fract to cart transf. matrix     PE   191
REAL CTOF(3,3)#                        Cart to fract transf matrix      PE   192
REAL G1,G2,G3,G4,G5,G6#                Metric tensor elements           PE   193
REAL GVAL(100,3)#                      Zmatrx geometry values           PE   194
REAL INC(3)#                           Map grid increments x,y,z        PE   195
REAL MAPR#                             Map resolution for search        PE   196
REAL PMAX,PMIN#                        Patterson density limits         PE   197
REAL RCUT#                             Rotation patt fom cutoff         PE   198
REAL TCUT#                             Translation patt fom cutoff      PE   199
REAL RFOM#                             Current rotation fom             PE   200
REAL RTES#                             Ratio of test rot vectors        PE   201
REAL RMUL(3)#                          Initial rotation angle multipl   PE   202
REAL RSET(3)#                          Initial rotation angle setting   PE   203
REAL RMIN#                             Rotation search d minimum        PE   204
REAL RMAX#                             Rotation search d maximum        PE   205
REAL RVEC#                             Ratio of vectors for rfom        PE   206
REAL RSCL#                             Calculated rfom scale            PE   207
REAL RRES#                             Angular resolution of rot vect   PE   208
REAL SRES#                             Rotation search d resolution     PE   209
REAL SCAL#                             Tfom scale                       PE   210
REAL SIGW#                             Weighted sigma triplet           PE   211
REAL STEP(6)#                          Translation x,y,z steps          PE   212
REAL SUME1#                            Sum of e(obs) for all e's        PE   213
REAL SUME2#                            Sum of e(obs)**2 for all e's     PE   214
REAL SUME3#                            Sum of /e1e2e3/ applied trips    PE   215
REAL SUMZ#                             Sum of n*z**2 for cell           PE   216
REAL SUMN#                             Sum of sites in the structure    PE   217
REAL TANG(3)#                          Twist angles in radians          PE   218
REAL TMUL(6)#                          Translation range multiplier     PE   219
REAL TSET(6)#                          Set translational shifts         PE   220
REAL TLIM#                             Transl'n search rejection lim    PE   221
REAL TRES#                             Transl'n search rejection lim    PE   222
REAL TMIN#                             Transl'n d minima                PE   223
REAL WRAT#                             Ratio of weights used            PE   224
REAL XLAT(4),YLAT(4),ZLAT(4)#          Lattice translations             PE   225
#                                                                       PE   226
string:(ATYP,128)#                                                      PE   227
CHARACTER*3 ATOM(100)#                 Zmatrx atom labels               PE   228
#                                                                       PE   229
COMMON/COMCPE/ATYP,ATOM#                                                PE   230
#                                                                       PE   231
COMMON/COMIPE/FADR,FCOG,FTYP,IXYZ,NZ,SITE,#                             PE   232
IA,IB,IC,IE,IK,IT,IV,IP,JB,JK,JC,JV,KV,#                                PE   233
ICEN,OFF,IENV,IATM,JATM,NENV,NORA,#                                     PE   234
NLAT,MARKA,STEPA,STOPA,MARKS,STEPS,STOPS,MARKE,STEPE,STOPE,#            PE   235
MARKB,STEPB,STOPB,MARKK,STEPK,STOPK,MARKV,STEPV,STOPV,#                 PE   236
MARKT,STEPT,STOPT,MARKP,STEPP,STOPP,MARKM,STEPM,STOPM,#                 PE   237
STOPBR,STOPBT,MARKC,STEPC,STOPC,STOP1,STOP2,#                           PE   238
RNUM,TNUM,MAXT,NATM,NUME,NUMT,NSYM,NFRG,#                               PE   239
NREF,NTPR,ROTN,RTRY,RSOL,RMIR,RLIS,SKIP,GLIS,#                          PE   240
TWIS,TTRY,TSOL,TLIS,TRAN,WMIN,WMAX,WINC,WTES,XCEPT#                     PE   241
#                                                                       PE   242
COMMON/COMFPE/AMIN,ANUM,ARAD,CELL,DCOG,DTOR,EMIN,CTOF,FTOC,MAPR,#       PE   243
INC,PMAX,PMIN,RSET,RVEC,TANG,TRES,TLIM,TMIN,TMUL,TSET,#                 PE   244
RMIN,RMAX,RRES,G1,G2,G3,G4,G5,G6,WRAT,XLAT,YLAT,ZLAT,#                  PE   245
RFOM,RSCL,RTES,SCAL,SIGW,STEP,SRES,SUME1,SUME2,SUME3,SUMZ,SUMN,#        PE   246
RCUT,TCUT,RMUL,GVAL#                                                    PE   247
endext:#                                                                PE   248
#                                                                       PE   249
systemheader:(PE00)#                   Main patsee sequence routine     PE00   1
####################################################################    PE00   2
#                                                                  #    PE00   3
#                                PE00                              #    PE00   4
#                                                                  #    PE00   5
####################################################################    PE00   6
SUBROUTINE PE00#                       S.r. hall nov 88                 PE00   7
include:(AACOMN)#                                                       PE00   8
include:(PECOMN)#                                                       PE00   9
#                                                                       PE00  10
FILEXT(IOUNIT(3))='MAP'#               Patterson map file               PE00  11
FILEXT(IOUNIT(4))='INV'#               Invariant file                   PE00  12
FILEXT(iofm1:)   ='PE0'#               Potential re-entrant fmt file    PE00  13
#                                                                       PE00  14
CALL PE10#                             Process control lines            PE00  15
catchexception:#                                                        PE00  16
CALL PE15#                             Get symm.,atoms from file a      PE00  17
IF(TRAN==yes:&NTPR==yes:) CALL PE20#   Get e-values, triplets           PE00  18
CALL PE25#                             Get patterson map & scale        PE00  19
CALL PE30#                             Define search envelope atoms     PE00  20
CALL PE35#                             Prepare for rotation search      PE00  21
IF(ROTN==yes:)#                        If rotation search applied       PE00  22
FOR(;WMIN<=WMAX;WMIN=WMIN+WINC)#       Loop over twist values           PE00  23
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  PE00  24
CALL PE40#                             Reset search parameters          PE00  25
IF(SKIP==yes:) NEXT#                   Skip this twist setting          PE00  26
CALL PE45#                             Generate search vectors          PE00  27
CALL PE50#                             Apply rotation search            PE00  28
CALL PE55#                             Access rotation solutions        PE00  29
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  PE00  30
IF(TRAN==yes:)#                        If translation requested         PE00  31
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  PE00  32
CALL PE60#                             Prepare translation search       PE00  33
FOR(IK=MARKK;IK<JK;IK=IK+STEPK)#       Loop over rotation solutions     PE00  34
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  PE00  35
CALL PE65#                             Start translation search         PE00  36
CALL PE70#                             Translation search loop          PE00  37
CALL PE75#                             Calc & store combined solution   PE00  38
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  PE00  39
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  PE00  40
CALL PE80#                             Output combined solutions        PE00  41
#                                                                       PE00  42
RETURN#                                                                 PE00  43
END#                                                                    PE00  44
systemheader:(PE10)#                   Process control lines            PE10   1
####################################################################    PE10   2
#                                                                  #    PE10   3
#                                PE10                              #    PE10   4
#                                                                  #    PE10   5
####################################################################    PE10   6
SUBROUTINE PE10#                       S.r. hall nov 88                 PE10   7
include:(AACOMN)#                                                       PE10   8
include:(PECOMN)#                                                       PE10   9
#                                                                       PE10  10
INTEGER I,J,K,L#                                                        PE10  11
INTEGER TYPE#                                                           PE10  12
INTEGER EXIST#                                                          PE10  13
#                                                                       PE10  14
REAL    XYZ(3,6,4)#                                                     PE10  15
string:(NAME,24)#                                                       PE10  16
chardata:(PE0,END TRIPLE SITE ROTATE FRAG TRANSL TWIST ZMATRX )#        PE10  17
chardata:(PE1,NTRY NSOL NRAN NREF VFOM VLIM VRES FOMT VTES SETA $#      PE10  18
SETR VLIS MIRR )#                                                       PE10  19
chardata:(PE3,NTRY NSOL NRAN VFOM VMIN VRES FOMT TPRT SETS SETR $#      PE10  20
TLIS NOTS TPRS )#                                                       PE10  21
chardata:(PE5,TMIN TMAX TINC TEST )#                                    PE10  22
chardata:(PE6,NUME NUMT EMIN AMIN )#                                    PE10  23
chardata:(PE7,CART PENT BENZ BOAT CHAI )#                               PE10  24
chardata:(PE8," GENSIN invariant file (.inv) not present -")#           PE10  25
chardata:(PE9," FOURR epat file (.map) not present -")#                 PE10  26
chardata:(PEA," spawning required calculations ...")#                   PE10  27
chardata:(NUM,123456789)#                                               PE10  28
intdata:(NN,[5,6,6,6])#                                                 PE10  29
datastuff:#                                                             PE10  30
DATA XYZ/-0.97712, 0.00000,-0.70992,$#                                  PE10  31
         -0.97712, 0.00000, 0.70992,$#                                  PE10  32
          0.37323, 0.00000, 1.14867,$#                                  PE10  33
          1.20779, 0.00000, 0.00000,$#                                  PE10  34
          0.37323, 0.00000,-1.14867,$#                                  PE10  35
          0.0    , 0.0    , 0.0    ,$#                                  PE10  36
         -1.20364, 0.00000,-0.69492,$#                                  PE10  37
         -1.20364, 0.00000, 0.69492,$#                                  PE10  38
          0.00000, 0.00000, 1.38984,$#                                  PE10  39
          1.20364, 0.00000, 0.69492,$#                                  PE10  40
          1.20364, 0.00000,-0.69492,$#                                  PE10  41
          0.00000, 0.00000,-1.38984,$#                                  PE10  42
          1.18382,-0.72478, 0.22783,$#                                  PE10  43
          1.18368, 0.72505, 0.22784,$#                                  PE10  44
         -0.00016, 1.20819,-0.45564,$#                                  PE10  45
         -1.18388, 0.72476, 0.22782,$#                                  PE10  46
         -1.18366,-0.72507, 0.22781,$#                                  PE10  47
          0.00020,-1.20814,-0.45566,$#                                  PE10  48
         -1.18377,-0.68339,-0.24165,$#                                  PE10  49
         -0.00001,-1.36689, 0.24164,$#                                  PE10  50
          1.18378,-0.68346,-0.24165,$#                                  PE10  51
          1.18378, 0.68346, 0.24165,$#                                  PE10  52
         -0.00001, 1.36689,-0.24164,$#                                  PE10  53
         -1.18377, 0.68339, 0.24165/#                                   PE10  54
#                                                                       PE10  55
#      ASSIGN FILE BUFFERS                                              PE10  56
#      -------------------                                              PE10  57
#                                                                       PE10  58
IOMARK(1)=QXSTR#                       Start of file a buffer           PE10  59
IOMARK(3)=IOMARK(1)#                   Use same buffer for file c       PE10  60
IOMARK(4)=IOMARK(1)#                   Use same buffer for file d       PE10  61
incrqx:(MARKA,IOMARK(1)+bdfbuf:,PE1005)# Get memory for buffer          PE10  62
STOPA=MARKA#                           Set start of atom list           PE10  63
STEPA=18#                              Set atom list length             PE10  64
#                                                                       PE10  65
#     SET DEFAULT VALUES                                                PE10  66
#     ------------------                                                PE10  67
#                                                                       PE10  68
DO I=1, 5; DO J=1,9; FTOC(J,I)=0.#     Set cell dimn. arrays            PE10  69
DO I=1, 3; DCOG(I)=0.#                 Set centre-of-grav. shifts       PE10  70
DO I=1,5; FTYP(I)=0#                   Zero fragments type              PE10  71
DO I=1,5; FADR(I)=0#                   Zero fragments address           PE10  72
DO I=1,10; ARAD(I)=0.#                 Zero atom radii                  PE10  73
DO I=1, 3; IATM(I)=MARKA#              Start atom list pointers         PE10  74
DO I=1, 3; JATM(I)=MARKA#              Stop  atom list pointers         PE10  75
DO I=1, 3; RSET(I)=-10.#               Default set rotations            PE10  76
DO I=1, 3; RMUL(I)=-9999.#             Default set rot multipliers      PE10  77
DO I=1, 3; TANG(I)=0.#                 Default set twist angles         PE10  78
DO I=1, 6; TSET(I)=0.#                 Default set translations         PE10  79
DO I=1, 6; TMUL(I)=-9999.#             Default set tran multipliers     PE10  80
DO I=1,4; DO J=1,100; SITE(J,I)=0#     Zero zmatrx ref sites            PE10  81
NZ=0#                                  Zero zmatrx lines input          PE10  82
ATYP=' '#                              Blank atom type list             PE10  83
FCOG=no:#                              Signal cof shift not set yet     PE10  84
ROTN=no:#                              Default no rotation search       PE10  85
TRAN=no:#                              Default no transl'n search       PE10  86
RLIS=no:#                              Default do not list vectors      PE10  87
RMIR=no:#                              Signal mirror not present        PE10  88
NTPR=-1#                               Signal triple sum application    PE10  89
TLIS=no:#                              Signal transl'n trip list        PE10  90
TWIS=no:#                              Apply twist flag & address       PE10  91
WTES=no:#                              Start twist angle test           PE10  92
GLIS=yes:#                             Set geometry list on             PE10  93
NUME=0#                                Maximum number of e's            PE10  94
MAXT=0#                                Maximum number of t3's           PE10  95
EMIN=0.#                               Set minimum e-value              PE10  96
AMIN=0.#                               Set minimum triplet a value      PE10  97
RTRY=0#                                Start number rotation tries      PE10  98
RSOL=5#                                Start number rotation solns      PE10  99
RMIN=2.#                               Start rotation d min             PE10 100
RMAX=6.#                               Start rotation d max             PE10 101
RRES=10.#                              Start ang resol of vectors       PE10 102
NREF=1000#                             Number of refinement tries       PE10 103
RNUM=5431#                             Default random number seed       PE10 104
RVEC=0.5#                              Ratio of vectors for rfom        PE10 105
RTES=0.1#                              Ratio of vectors for pretest     PE10 106
RCUT=0.4#                              Rotation patt fom threshold      PE10 107
TTRY=0#                                Start number transl'n tries      PE10 108
TSOL=5#                                Start number transl'n solns      PE10 109
TNUM=5431#                             Default random number seed       PE10 110
TCUT=0.3#                              Translation patt fom threshold   PE10 111
TLIM=0.3#                              Set tprsum rejection lim         PE10 112
TRES=0.4#                              Transl shift resoln (angstrom)   PE10 113
TMIN=3.4#                              Start transl'n d minima          PE10 114
WMIN=0#                                Start twist angle min            PE10 115
WMAX=0#                                Start twist angle max            PE10 116
WINC=5#                                Start twist angle del            PE10 117
IXYZ=1#                                Xyz pointer 1=fract/6=cart       PE10 118
TYPE=1#                                Set fragment type as fixed       PE10 119
NFRG=0#                                Set fragment count to fixed      PE10 120
FTYP(1)=1#                             Set initial fragment type        PE10 121
FADR(1)=STOPA#                         Set initial fragment address     PE10 122
DTOR=.01745329#                        Convert degrees to radians       PE10 123
XCEPT=0#                               No exceptions yet                PE10 124
inquirefile:(IOUNIT(4),EXIST,' ')#     Are the invariants there:        PE10 125
IF(EXIST==0)#                          Throw an exception               PE10 126
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  PE10 127
openfmt:(iofm1:,' ')#                  Open reentrant file stream       PE10 128
writefmt:(iofm1:,'(A)','GENEV ')#      Genev to reentrant file          PE10 129
writefmt:(iofm1:,'(A)','GENSIN ')#     Gensin to reentrant file         PE10 130
writeline:(2,PE8,NPE8,3,2)#            Output message                   PE10 131
XCEPT=1#                               Throw an exception               PE10 132
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  PE10 133
inquirefile:(IOUNIT(3),EXIST,' ')#     Is there an emap file            PE10 134
IF(EXIST==0)#                          Throw an exception               PE10 135
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  PE10 136
IF(XCEPT==0)#                          If not already open              PE10 137
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  PE10 138
openfmt:(iofm1:,' ')#                  Open reentrant file stream       PE10 139
writefmt:(iofm1:,'(A)','GENEV ')#      Genev to reentrant file          PE10 140
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  PE10 141
writefmt:(iofm1:,'(A)','FOURR epat ')# Gensin to reentrant file         PE10 142
writeline:(2,PE9,NPE9,3,2)#            Output message                   PE10 143
XCEPT=1#                               Throw an exception               PE10 144
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  PE10 145
IF(XCEPT!=0)#                                                           PE10 146
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  PE10 147
writeline:(0,PEA,NPEA,3,2)#            Output message                   PE10 148
writefmt:(iofm1:,'(A)',CHRIP(1^80))#   Redo the gentan line             PE10 149
closefmt:(iofm1:)#                     Shut down the line file          PE10 150
linestream:('PE0')#                    Insert file into line stream     PE10 151
LINFLG=1#                              Force to read another line       PE10 152
RETURN#                                Go no further                    PE10 153
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  PE10 154
#                                                                       PE10 155
IF(BUFIN(1)<voidflg:) GLIS=no:#        Switch off the geometry list     PE10 156
#                                                                       PE10 157
#     READ AND PROCESS CONTROL LINES                                    PE10 158
#     ------------------------------                                    PE10 159
#                                                                       PE10 160
REPEAT#                                Loop over input lines            PE10 161
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  PE10 162
readline:(PE0,NPE0)#                   Read an input line               PE10 163
#                                                                       PE10 164
IF(LINID==1) BREAK#                    *********** end **************   PE10 165
#                                                                       PE10 166
ELSE IF(LINID==2)#                     ********** triple ************   PE10 167
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  PE10 168
FOR(I=1;I>0;I=I+1)#                    Loop over input fields           PE10 169
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  PE10 170
ifvo:(BUFIN(I)) BREAK#                 Exit on blank field              PE10 171
stringlim:(I,J,K)#                     Get string limits                PE10 172
compchar:(CHRIN,J,PE6,1,4,5,NPE6,K)#   Find control string              PE10 173
IF(K==0) iquit:(PE1010)#               Illegal control                  PE10 174
I=I+1#                                 Incr to next field               PE10 175
IF     (K==1) NUME=NINT(BUFIN(I))#     Store number of e's              PE10 176
ELSE IF(K==2) MAXT=NINT(BUFIN(I))#     Store max number triples         PE10 177
ELSE IF(K==3) EMIN=BUFIN(I)#           Store minimum e value            PE10 178
ELSE IF(K==4) AMIN=BUFIN(I)#           Store minimum a value            PE10 179
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  PE10 180
IF(NUME==0&EMIN>0.) NUME=10000#        Emin is the generator limit      PE10 181
IF(MAXT==0&AMIN>0.) MAXT=10000#        Amin is the triplet limit        PE10 182
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  PE10 183
#                                                                       PE10 184
ELSE IF(LINID==3)#                     ************* site ***********   PE10 185
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  PE10 186
IA=STOPA#                              Reset atom list pointer          PE10 187
incrqx:(STOPA,IA+STEPA,PE1015)#        Incr atom list pointer           PE10 188
stringlim:(1,K,L)#                     Get name string limits           PE10 189
NAME=CHRIP(K^L);  label:(NAME,I,J)#    Put name in char buffer          PE10 190
movector:(NAME,1,an:,1,24,0)#          Store atom list in qx            PE10 191
movereal:(BUFIN,2,QX,IA+IXYZ,3,0)#     Store atom x,y,z                 PE10 192
ifvo:(BUFIN(5))       aw:=1.#          Store default atom weight        PE10 193
ELSE IF(BUFIN(5)<0.1) aw:=1.#          Ignore input u values            PE10 194
ELSE                  aw:=BUFIN(5)#    Get input weight                 PE10 195
at:=FLOAT(1-IXYZ)#                     Store fract/cart signal          PE10 196
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  PE10 197
#                                                                       PE10 198
ELSE IF(LINID==4)#                     *********** rotate ***********   PE10 199
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  PE10 200
IF(NZ>0) CALL PE11#                    Convert zmatrx data to coords    PE10 201
JATM(TYPE)=STOPA;  TYPE=2#             Set end of last atom list        PE10 202
IATM(TYPE)=STOPA;  ROTN=yes:#          Start of rotation atom list      PE10 203
FOR(I=1;I>0;I=I+1)#                    Loop over input fields           PE10 204
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 PE10 205
ifvo:(BUFIN(I)) BREAK#                 Exit on blank field              PE10 206
IF(BUFIN(I)>voidflg:) iquit:(PE1020)#  Illegal control                  PE10 207
stringlim:(I,J,K)#                     Get string limits                PE10 208
compchar:(CHRIN,J,PE1,1,4,5,NPE1,K)#   Find control string              PE10 209
IF(K==0) iquit:(PE1020)#               Illegal control                  PE10 210
IF(K<10) I=I+1#                        Incr to next field               PE10 211
IF     (K==1) RTRY=NINT(BUFIN(I))#     Store number rotation tries      PE10 212
ELSE IF(K==2) RSOL=NINT(BUFIN(I))#     Store number rotation solns      PE10 213
ELSE IF(K==3) RNUM=NINT(BUFIN(I))#     Store random number seed         PE10 214
ELSE IF(K==4) NREF=NINT(BUFIN(I))#     Store number refine tries        PE10 215
ELSE IF(K==5) RVEC=BUFIN(I)#           Store vector ratio for fom       PE10 216
ELSE IF(K==6)#                         Search vector limits             PE10 217
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 PE10 218
RMIN=BUFIN(I);   I=I+1#                Store rotation d min             PE10 219
RMAX=BUFIN(I)#                         Store rotation d max             PE10 220
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 PE10 221
ELSE IF(K==7) RRES=BUFIN(I)#           Store rot angular resolution     PE10 222
ELSE IF(K==8) RCUT=BUFIN(I)#           Store patt fom threshold         PE10 223
ELSE IF(K==9) RTES=BUFIN(I)#           Store vector search test ratio   PE10 224
ELSE IF(K==10)#                        Set the rotation angles          PE10 225
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 PE10 226
DO J=1,3; RSET(J)=BUFIN(J+I)#          Store rot angles in degrees      PE10 227
I=I+3#                                 Reset the field pointer          PE10 228
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 PE10 229
ELSE IF(K==11)#                        Set the rotation ranges          PE10 230
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 PE10 231
DO J=1,3; RMUL(J)=BUFIN(J+I)#          Store rot ranges in degrees      PE10 232
I=I+3#                                 Reset the field pointer          PE10 233
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 PE10 234
ELSE IF(K==12)RLIS=yes:#               Signal rotation vectors listed   PE10 235
ELSE IF(K==13)RMIR=yes:#               Signal mirror present            PE10 236
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 PE10 237
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  PE10 238
#                                                                       PE10 239
ELSE IF(LINID==5)#                     ************* frag ***********   PE10 240
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14 PE10 241
IF(NFRG>0)   FADR(NFRG)=STOPA#         Save frag atom address           PE10 242
NFRG=NFRG+1; FTYP(NFRG)=TYPE#          Count the next fragment          PE10 243
IXYZ=1#                                Assume fractional coords in      PE10 244
IF(BUFIN(1)<voidflg:)#                 Only alpha input expected        PE10 245
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 15 PE10 246
stringlim:(1,K,L)#                     Get name string limits           PE10 247
compchar:(CHRIN,K,PE7,1,4,5,NPE7,L)#   Find control string              PE10 248
IF(L==1) IXYZ=6#                       Flag as cartesian input          PE10 249
ELSE IF(L>1)#                          If preset geometry               PE10 250
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 16 PE10 251
IA=STOPA; L=L-1; IXYZ=6#               Reset atom list pointer          PE10 252
incrqx:(STOPA,IA+NN(L)*STEPA,PE1015)#  Incr atom list pointer           PE10 253
DO I=1,NN(L)#                          Loop over coords                 PE10 254
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 17 PE10 255
NAME='C'//NUM(I^I)#                    Store site label                 PE10 256
movector:(NAME,1,an:,1,24,0)#          Store atom name in qx            PE10 257
movereal:(XYZ(1,I,L),1,cx:,1,3,0)#     Store atom x,y,z                 PE10 258
aw:=1.;   at:=-5#                      Store default atom weight        PE10 259
IA=IA+STEPA#                           Incr the list pointer            PE10 260
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 17 PE10 261
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 16 PE10 262
ELSE#                                  There must be atom input         PE10 263
DO I=1,20#                             Loop over atom names             PE10 264
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 18 PE10 265
IA=STOPA#                              Reset atom list pointer          PE10 266
ifvo:(BUFIN(I)) BREAK#                 Exit after last name             PE10 267
incrqx:(STOPA,IA+STEPA,PE1015)#        Incr atom list pointer           PE10 268
stringlim:(I,K,L)#                     Get name string limits           PE10 269
NAME=CHRIN(K^L)#                       Put name in char buffer          PE10 270
movector:(NAME,1,an:,1,24,0)#          Store atom list in qx            PE10 271
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 18 PE10 272
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 15 PE10 273
IF(BUFIN(2)>voidflg:)#                 Test for numeric input           PE10 274
DO I=1,6; FTOC(I,NFRG)=BUFIN(I)#       Get cell data                    PE10 275
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14 PE10 276
#                                                                       PE10 277
ELSE IF(LINID==6)#                     ************ transl **********   PE10 278
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 19 PE10 279
IF(NZ>0) CALL PE11#                    Convert zmatrx data to coords    PE10 280
JATM(TYPE)=STOPA;   TYPE=3#            End of last atom list            PE10 281
IATM(TYPE)=STOPA;   TRAN=yes:#         Future atoms for trans only      PE10 282
FOR(I=1;I>0;I=I+1)#                    Loop over input fields           PE10 283
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 20 PE10 284
ifvo:(BUFIN(I)) BREAK#                 Exit on blank field              PE10 285
stringlim:(I,J,K)#                     Get string limits                PE10 286
compchar:(CHRIN,J,PE3,1,4,5,NPE3,K)#   Find control string              PE10 287
IF(K==0) iquit:(PE1035)#               Illegal control                  PE10 288
IF(K<9) I=I+1#                         Incr to next field               PE10 289
IF     (K==1) TTRY=NINT(BUFIN(I))#     Store number transl'n tries      PE10 290
ELSE IF(K==2) TSOL=NINT(BUFIN(I))#     Store number transl'n solns      PE10 291
ELSE IF(K==3) TNUM=NINT(BUFIN(I))#     Store random number seed         PE10 292
ELSE IF(K==5) TMIN=BUFIN(I)#           Store minimum vector length      PE10 293
ELSE IF(K==6) TRES=BUFIN(I)#           Resolution of transln shifts     PE10 294
ELSE IF(K==7) TCUT=BUFIN(I)#           Store patt fom threshold         PE10 295
ELSE IF(K==8) TLIM=BUFIN(I)#           Reduce tprsum rejection lim      PE10 296
ELSE IF(K==9)#                         Set the translation shifts       PE10 297
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 21 PE10 298
DO J=1,3; IF(BUFIN(J+I)>voidflg:) TSET(J)=BUFIN(J+I)# Shift start       PE10 299
I=I+3#                                 Reset the field pointer          PE10 300
IF(BUFIN(I+1)<=voidflg:) NEXT#         Skip if fixed transl not input   PE10 301
DO J=1,3; IF(BUFIN(J+I)>voidflg:) TSET(J+3)=BUFIN(J+I)# Shift start     PE10 302
I=I+3#                                 Reset the field pointer          PE10 303
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 21 PE10 304
ELSE IF(K==10)#                        Set the translation ranges       PE10 305
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 22 PE10 306
DO J=1,3; IF(BUFIN(J+I)>voidflg:) TMUL(J)=BUFIN(J+I)# Shift ranges      PE10 307
I=I+3#                                 Reset the field pointer          PE10 308
IF(BUFIN(I+1)<=voidflg:) NEXT#         Skip if fixed transl not input   PE10 309
DO J=1,3; IF(BUFIN(J+I)>voidflg:) TMUL(J+3)=BUFIN(J+I)# Shift ranges    PE10 310
I=I+3#                                 Reset the field pointer          PE10 311
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 22 PE10 312
ELSE IF(K==11) TLIS=yes:#              Signal triple listing requ       PE10 313
ELSE IF(K==12) NTPR=no:#               Signal triple sum not used       PE10 314
ELSE IF(K==13) NTPR=yes:#              Signal triple sum requested      PE10 315
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 20 PE10 316
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 19 PE10 317
#                                                                       PE10 318
ELSE IF(LINID==7)#                     ************ twist ***********   PE10 319
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 23 PE10 320
TWIS=STOPA;  WMAX=355#                 Save current atom address        PE10 321
FOR(I=1;I>0;I=I+1)#                    Loop over input fields           PE10 322
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 24 PE10 323
ifvo:(BUFIN(I)) BREAK#                 Exit on blank field              PE10 324
stringlim:(I,J,K)#                     Get string limits                PE10 325
compchar:(CHRIN,J,PE5,1,4,5,NPE3,K)#   Find control string              PE10 326
IF(K==0) iquit:(PE1050)#               Illegal control                  PE10 327
I=I+1#                                 Incr to next field               PE10 328
IF     (K==1) WMIN=NINT(BUFIN(I))#     Store twist angle min            PE10 329
ELSE IF(K==2) WMAX=NINT(BUFIN(I))#     Store twist angle max            PE10 330
ELSE IF(K==3) WINC=NINT(BUFIN(I))#     Store twist angle del            PE10 331
ELSE IF(K==4) WTES=NINT(BUFIN(I))#     Store twist angle test           PE10 332
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 24 PE10 333
IF(WINC==0) WINC=1#                    Min 1 degree twist needed        PE10 334
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 23 PE10 335
#                                                                       PE10 336
ELSE IF(LINID==8)#                     ************ zmatrx **********   PE10 337
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 25 PE10 338
NZ=NZ+1#                               Count the zmatrx lines           PE10 339
IF(NZ!=NINT(BUFIN(1))) iquit:(PE1071)# Line is out of order             PE10 340
ifal:(BUFIN(2)) $(stringlim:(2,J,K) $)# Define atom type string         PE10 341
ELSE            iquit:(PE1072)#        Error..blank or number           PE10 342
ATOM(NZ)=CHRIP(J^K-1)#                 Store atom type string           PE10 343
DO I=1,3#                              Loop na/dis na/ang na/tor pair   PE10 344
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 27 PE10 345
J=I*2#                                 Input field pointer              PE10 346
ifvo:(BUFIN(J+1)) BREAK#               Exit if no more                  PE10 347
ifnm:(BUFIN(J+1)) SITE(NZ,I)=NINT(BUFIN(J+1))# Store ref site           PE10 348
ELSE              iquit:(PE1073)#      Site number not supplied         PE10 349
ifnm:(BUFIN(J+2)) GVAL(NZ,I)=BUFIN(J+2)# Store geom parameter           PE10 350
ELSE              iquit:(PE1074)#      Dist/ang/tortion missing         PE10 351
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 27 PE10 352
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 25 PE10 353
#                                                                       PE10 354
ELSE iquit:(PE1099)#                   Unknown line entered             PE10 355
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  PE10 356
IF(NZ>0) CALL PE11#                    Convert zmatrx data to coords    PE10 357
IF(NFRG>0) FADR(NFRG)=STOPA;JATM(TYPE)=STOPA# Save final addresses      PE10 358
IF(NUME==0) NUME=100#                  Set default number of e's        PE10 359
IF(MAXT==0) MAXT=100#                  Set default number of triples    PE10 360
#                                                                       PE10 361
RETURN#                                                                 PE10 362
END#                                                                    PE10 363
systemheader:(PE11)#                   Convert zmatrx data to coords    PE11   1
####################################################################    PE11   2
#                                                                  #    PE11   3
#                                PE11                              #    PE11   4
#                                                                  #    PE11   5
####################################################################    PE11   6
SUBROUTINE PE11#                       S.r. hall sep 96                 PE11   7
include:(AACOMN)#                                                       PE11   8
include:(PECOMN)#                                                       PE11   9
#                                                                       PE11  10
#     ###########                                                       PE11  11
#     ########### The algorithm used in this subroutine was             PE11  12
#     ########### based on that of program MBLDR from the               PE11  13
#     ########### system GAMESS. Copied with the permission of          PE11  14
#     ########### Mike Schmidt.                                         PE11  15
#     ###########                                                       PE11  16
#                                                                       PE11  17
REAL V1(3),V2(3),V3(3),V4(3),VP(3),V3M,RAD#                             PE11  18
REAL A,B,D,GAM,OMZ,TEM,ZET#                                             PE11  19
INTEGER I,IZ,J,JA,K,KA,LA#                                              PE11  20
#                                                                       PE11  21
string:(LABL,8);string:(NAME,24)#                                       PE11  22
chardata:(NUMB,0123456789)#                                             PE11  23
datastuff:#                                                             PE11  24
#                                                                       PE11  25
RAD=3.14159/180.#                      To convert deg to rad            PE11  26
DO I=1,NZ; GVAL(I,2)=GVAL(I,2)*RAD#    Convert angles to radians        PE11  27
DO I=1,NZ; GVAL(I,3)=GVAL(I,3)*RAD#    Convert tortions to radians      PE11  28
#                                                                       PE11  29
IA=STOPA;  IXYZ=6#                     Reset atom list pointer          PE11  30
incrqx:(STOPA,IA+NZ*STEPA,PE1015)#     Incr atom list pointer           PE11  31
#                                                                       PE11  32
DO IZ=1,NZ#                            Loop over coords                 PE11  33
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  PE11  34
#                                                                       PE11  35
J=IZ/10+1; K=MOD(IZ,10)+1#             Get digits                       PE11  36
IF(IZ<10) LABL=NUMB(K^K)#              Store site label                 PE11  37
ELSE      LABL=NUMB(J^J)//NUMB(K^K)#   If number 10-99                  PE11  38
label:(ATOM(IZ),J,K)#                  Get string limits                PE11  39
NAME=ATOM(IZ)(1^K)//LABL(1^2)#         Construct atom label             PE11  40
movector:(NAME,1,an:,1,24,0)#          Store atom name in qx            PE11  41
aw:=1.;   at:=-5#                      Store default atom weight        PE11  42
#                                                                       PE11  43
#     INSERT ATOM COORDINATES                                           PE11  44
#     -----------------------                                           PE11  45
#                                                                       PE11  46
IF(IZ==1)#                             The first site                   PE11  47
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  PE11  48
cx:=0.;  cy:=0.;  cz:=0.#              Set coords to zero               PE11  49
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  PE11  50
#                                                                       PE11  51
ELSE IF(IZ==2)#                        The second site                  PE11  52
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  PE11  53
cx:=0.;  cy:=0.;  cz:=dis:#            Move coords by bond distance     PE11  54
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  PE11  55
#                                                                       PE11  56
ELSE IF(IZ==3)#                        The third site                   PE11  57
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  PE11  58
cx:=dis:*SIN(ang:)#                    X coord                          PE11  59
cy:=0.#                                Y coord                          PE11  60
cz:=dis:*COS(ang:)#                    Z coord                          PE11  61
IF(ns1:==2) cz:=cz:(JA)-cz:#           Z coord                          PE11  62
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  PE11  63
#                                                                       PE11  64
#                                                                       PE11  65
ELSE#                                  The 4th and more sites           PE11  66
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  PE11  67
listp:(KA,ns1:);  listp:(LA,ns2:)#     Conv ref numb to coord list      PE11  68
#                                                                       PE11  69
IF(ABS(cx:(JA))<small:)#               Test if last x coord zero        PE11  70
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  PE11  71
cx:=dis:*SIN(ang:)#                    Calc x coord                     PE11  72
cy:=0.#                                Calc y coord                     PE11  73
cz:=cz:(KA)-dis:*COS(ang:)*SIGN(1.,cz:(KA)-cz:(LA))# Calc z coord       PE11  74
JA=IA; IA=IA+STEPA; NEXT#              Get the next coord               PE11  75
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  PE11  76
#                                                                       PE11  77
#        DIHEDRAL ANGLE CONFIGURATION                                   PE11  78
#        ----------------------------                                   PE11  79
#                                                                       PE11  80
IF(ns4:==0)#                           Test if dihedral angle format    PE11  81
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  PE11  82
CALL PE12(V1,ns2:,ns3:)#               Get unit vect n2-n3              PE11  83
CALL PE12(V2,ns1:,ns2:)#               Get unit vect n1-n2              PE11  84
CALL PE13(VP,V3M,V1,V2)#               Get vect product v1 x v2         PE11  85
A=SQRT(1.-(V1(1)*V2(1)+V1(2)*V2(2)+V1(3)*V2(3))**2)#                    PE11  86
DO I=1,3; V3(I)=VP(I)/A#               Get unit vect of v1 x v2         PE11  87
CALL PE13(V4,V3M,V3,V2)#               Get vect product v3 x v2         PE11  88
DO I=1,3#                              Loop over coords                 PE11  89
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  PE11  90
VP(I)=dis:*(-V2(I)*COS(ang:)$#         Vect n1 to iz                    PE11  91
            +V3(I)*SIN(ang:)*SIN(dih:)$# Vect n1 to iz                  PE11  92
            +V4(I)*SIN(ang:)*COS(dih:))# Vect n1 to iz                  PE11  93
cx:(IA+I-1)=cx:(KA+I-1)+VP(I)#                                          PE11  94
IF(ABS(cx:(IA+I-1))<small:) cx:(IA+I-1)=0.# V small values to zero      PE11  95
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  PE11  96
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  PE11  97
#                                                                       PE11  98
#      BOND-PLANE CONFIGURATION                                         PE11  99
#      ------------------------                                         PE11 100
#                                                                       PE11 101
ELSE#                                  If 4th ref num present           PE11 102
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  PE11 103
CALL PE12(V1,ns1:,ns3:)#               Get unit vect n1-n3              PE11 104
CALL PE12(V2,ns2:,ns1:)#               Get unit vect n2-n1              PE11 105
CALL PE13(V3,V3M,V1,V2)#               Get vect product v1 x v2         PE11 106
ZET=-(V1(1)*V2(1)+V1(2)*V2(2)+V1(3)*V2(3))# Zeta                        PE11 107
OMZ=1.-ZET*ZET#                        Omega                            PE11 108
#                                                                       PE11 109
IF(ns4:==1)#                           Test bond in +ve direction       PE11 110
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 PE11 111
IF(A<-small:) TEM=180.*RAD#            If zeta -ve set to pi            PE11 112
ELSE          TEM=0.#                  Else set to zero                 PE11 113
ifeq:(A,0.)   GAM=90.*RAD#             Gamma = pi/2                     PE11 114
ELSE          GAM=ATAN(SQRT(OMZ)/ZET)+TEM# Else calc gamma              PE11 115
IF(ABS(GAM+ang:+dih:-3.14159)>small:)# If angle gt 2pi                  PE11 116
A=(-COS(dih:)+ZET*COS(ang:))/OMZ#                                       PE11 117
B=(+COS(ang:)-ZET*COS(dih:))/OMZ#                                       PE11 118
D=ns4:*SQRT((1.+A*COS(dih:)-B*COS(ang:))/OMZ)#                          PE11 119
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 PE11 120
#                                                                       PE11 121
ELSE#                                  Bond in other direction          PE11 122
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 PE11 123
D=(COS(dih:)/OMZ)*V3M#                                                  PE11 124
A=SQRT((1.-COS(ang:)**2-D*COS(dih:)*V3M)/OMZ)#                          PE11 125
IF(ns4:==-2) A=-A#                     Reverse the sign                 PE11 126
B=A*ZET+COS(ang:)#                                                      PE11 127
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 PE11 128
#                                                                       PE11 129
#     CALCULATE COORDINATES                                             PE11 130
#     ---------------------                                             PE11 131
#                                                                       PE11 132
DO I=1,3#                                                               PE11 133
V3(I)=A*V1(I)+B*V2(I)+D*V3(I)#                                          PE11 134
VP(I)=dis:*V3(I)#                                                       PE11 135
cx:(IA+I-1)=cx:(KA+I-1)+VP(I)#         Calc coordinates                 PE11 136
IF(ABS(cx:(IA+I-1))<small:) cx:(IA+I-1)=0.# V small values to zero      PE11 137
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  PE11 138
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  PE11 139
JA=IA;  IA=IA+STEPA#                   Incr the list pointer            PE11 140
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  PE11 141
NZ=0;  IA=JA#                          Reset number of zmatrx lines     PE11 142
#                                                                       PE11 143
RETURN#                                                                 PE11 144
END#                                                                    PE11 145
systemheader:(PE12)#                   Get unit vectors from coords     PE12   1
####################################################################    PE12   2
#                                                                  #    PE12   3
#                                PE12                              #    PE12   4
#                                                                  #    PE12   5
####################################################################    PE12   6
SUBROUTINE PE12(V,N1,N2)#              S.r. hall sep 96                 PE12   7
include:(AACOMN)#                                                       PE12   8
include:(PECOMN)#                                                       PE12   9
#                                                                       PE12  10
INTEGER N1,N2,I#                                                        PE12  11
REAL    V(3),D#                                                         PE12  12
#                                                                       PE12  13
listp:(IB,N1); listp:(JB,N2)#          Get coord list pointers          PE12  14
V(1)=cx:(IB)-cx:(JB)#                  X coord of new vector            PE12  15
V(2)=cy:(IB)-cy:(JB)#                  Y coord of new vector            PE12  16
V(3)=cz:(IB)-cz:(JB)#                  Z coord of new vector            PE12  17
D=SQRT(V(1)**2+V(2)**2+V(3)**2)#       Get length of vector             PE12  18
DO I=1,3; V(I)=V(I)/D#                 Get unit vector                  PE12  19
#                                                                       PE12  20
RETURN#                                                                 PE12  21
END#                                                                    PE12  22
systemheader:(PE13)#                   Do vector product                PE13   1
####################################################################    PE13   2
#                                                                  #    PE13   3
#                                PE13                              #    PE13   4
#                                                                  #    PE13   5
####################################################################    PE13   6
SUBROUTINE PE13(V,D,V1,V2)#            S.r. hall sep 96                 PE13   7
include:(AACOMN)#                                                       PE13   8
include:(PECOMN)#                                                       PE13   9
#                                                                       PE13  10
REAL V(3),D,V1(3),V2(3)#                                                PE13  11
#                                                                       PE13  12
V(1)=V1(2)*V2(3)-V1(3)*V2(2)#          Get x coord of product           PE13  13
V(2)=V1(3)*V2(1)-V1(1)*V2(3)#          Get y coord of product           PE13  14
V(3)=V1(1)*V2(2)-V1(2)*V2(1)#          Get z coord of product           PE13  15
D=SQRT(V(1)**2+V(2)**2+V(3)**2)#       Get length of vector             PE13  16
#                                                                       PE13  17
RETURN#                                                                 PE13  18
END#                                                                    PE13  19
systemheader:(PE15)#                   Read symm, atoms from file a     PE15   1
####################################################################    PE15   2
#                                                                  #    PE15   3
#                                PE15                              #    PE15   4
#                                                                  #    PE15   5
####################################################################    PE15   6
SUBROUTINE PE15#                       S.r. hall nov 88                 PE15   7
include:(AACOMN)#                                                       PE15   8
include:(PECOMN)#                                                       PE15   9
#                                                                       PE15  10
INTEGER I,J,K,L,N#                                                      PE15  11
INTEGER LPAK#                                                           PE15  12
INTEGER LTYP,M#                                                         PE15  13
REAL DIST#                                                              PE15  14
INTEGER IWNT(10),RELPT(11),KEY(4)#                                      PE15  15
REAL A,B,C,U,V,W,SA,SB,SG,CA,CB,CG,CGS,SGS#                             PE15  16
REAL KEEP(20)#                                                          PE15  17
#                                                                       PE15  18
string:(NAME,24);    string:(TEMP,24)#                                  PE15  19
chardata:(PE1,' Supplied molecular fragment')#                          PE15  20
chardata:(PE3,'   Fract.-to-Cart. Coord. Transf. Matrix')#              PE15  21
chardata:(PE4,"x/a     y/b     z/c   weight  type   $#                  PE15  22
xC      yC      zC")#                                                   PE15  23
chardata:(PE5,"dist.   angles")#                                        PE15  24
chardata:(PE6," Scattering fraction of all fragments             ")#    PE15  25
realdata:(CENT,[$#                                                      PE15  26
0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,#        Add for lattice type p           PE15  27
.5,.5,.5,0.,0.,0.,0.,0.,0.,1.,#        Add for lattice type i           PE15  28
.333333,.666667,.666667,.666667,.333333,.333333,0.,0.,0.,2.,# Type r    PE15  29
.0,.5,.5,.5,.0,.5,.5,.5,.0,3.,#        Add for lattice type f           PE15  30
.0,.5,.5,0.,0.,0.,0.,0.,0.,1.,#        Add for lattice type a           PE15  31
.5,.0,.5,0.,0.,0.,0.,0.,0.,1.,#        Add for lattice type b           PE15  32
.5,.5,.0,0.,0.,0.,0.,0.,0.,1.])#       Add for lattice type c           PE15  33
realdata:(FMT1,[121152.,241152.,361152.])# Matrix format                PE15  34
realdata:(FMT2,[842.,842.,842.,812.,513.,932.,832.,832.])#              PE15  35
realdata:(R,0.01745329)#               Convert degrees to radians       PE15  36
datastuff:#                                                             PE15  37
#                                                                       PE15  38
#                                                                       PE15  39
#      READ CELL CONSTANTS FROM FILE A                                  PE15  40
#      ------------------------------                                   PE15  41
DO N=1,7#                              Loop over cell packets           PE15  42
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  PE15  43
readwpkt:(1,lrcell:,LPAK,IP,0)#        Read the next packet             PE15  44
IF(IP<=0) iquit:(PE1505)#              Cell data must be present        PE15  45
IF(N==1)#                              Get the cell dimensions          PE15  46
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  PE15  47
DO I=1,3; CELL(I)=QX(IP+I)#            A,b,c                            PE15  48
DO I=4,6; CELL(I)=QX(IP+I+3)*360.#     Alf,bet,gam in degrees           PE15  49
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  PE15  50
ELSE IF(N==5)#                         Get cart to frac transf matrix   PE15  51
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  PE15  52
DO I=1,3; DO J=1,3#                    Loop over matrix elements        PE15  53
CTOF(J,I)=QX(IP+I*3+J-3)#              Get c-to-f matrix                PE15  54
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  PE15  55
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  PE15  56
G1=QX(IP+1);    G2=QX(IP+5);    G3=QX(IP+9)# Get diagonal elements      PE15  57
G4=2.*QX(IP+2); G5=2.*QX(IP+3); G6=2.*QX(IP+6)# Offdiagonal elements    PE15  58
#                                                                       PE15  59
#     READ SYMMETRY INFORMATION FROM FILE A                             PE15  60
#     -------------------------------------                             PE15  61
#                                                                       PE15  62
readwpkt:(1,lrsymm:,LPAK,IP,0)#        Read the 1st sym pkt             PE15  63
LTYP=NINT(QX(IP+1))#                   Lattice type 1-14                PE15  64
IF(LTYP>7) LTYP=LTYP-7#                Reduce to centrosymmetric        PE15  65
ICEN=NINT(QX(IP+2))#                   Centric / acentric  0/1          PE15  66
NSYM=NINT(QX(IP+3))#                   Total no of symm. rel. rot.      PE15  67
MARKS=STOPA;  STEPS=LPAK#              Set symmetry pointers            PE15  68
incrqx:(STOPS,MARKS+STEPS*NSYM,PE1510)# Memory for symop table          PE15  69
FOR(N=MARKS;N>0;N=N+LPAK)#             Loop thro. symm. rec.            PE15  70
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  PE15  71
readwpkt:(1,lrsymm:,LPAK,IP,0)#        Read sym. rel. rotations         PE15  72
IF(IP<=0) BREAK#                       Test symops finished             PE15  73
movereal:(QX,IP+1,QX,N+1,LPAK,1)#      Store symm. rel. rotations       PE15  74
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  PE15  75
STOP1=N;    NLAT=1#                    End of sym. rec. read in         PE15  76
XLAT(1)=99.5; YLAT(1)=99.5; ZLAT(1)=99.5# Store primitive lattice       PE15  77
#                                                                       PE15  78
IF(LTYP>1)#                            Add symops if non-primitive      PE15  79
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  PE15  80
L=(LTYP-1)*10; NLAT=NINT(CENT(L+10))+1# No. of lattice points           PE15  81
DO K=2,NLAT#                           Loop over lattice points         PE15  82
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  PE15  83
XLAT(K)=CENT(L+1)+99.5#                Get x lattice translation        PE15  84
YLAT(K)=CENT(L+2)+99.5#                Get y lattice translation        PE15  85
ZLAT(K)=CENT(L+3)+99.5#                Get z lattice translation        PE15  86
FOR(I=MARKS;I<STOP1;I=I+STEPS)#        Loop thro. symm. rec.            PE15  87
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  PE15  88
movereal:(QX,I+1,QX,N+1,STEPS,1)#      Rotation matrix equivalent       PE15  89
QX(N+10)=QX(N+10)+CENT(L+1)#           Add to translation in x          PE15  90
QX(N+11)=QX(N+11)+CENT(L+2)#           Add to translation in y          PE15  91
QX(N+12)=QX(N+12)+CENT(L+3)#           Add to translation in z          PE15  92
N=N+STEPS#                             Set start of new symm. rec.      PE15  93
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  PE15  94
L=L+3#                                 Incr lattice list pointer        PE15  95
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  PE15  96
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  PE15  97
STOP2=N;  M=N-MARKS+1#                 Number of elements               PE15  98
IF(ICEN==0) DO J=1,M; QX(N+J)=-QX(MARKS+J)# Set centro elements         PE15  99
#                                                                       PE15 100
#    BOND RADII AND SF NAMES FROM FILE A                                PE15 101
#    -----------------------------------                                PE15 102
#                                                                       PE15 103
IWNT(1)=3;  IWNT(2)=5#                 Request atom number z & radius   PE15 104
IWNT(3)=11; IWNT(4)=1#                 Request type name & cell conte   PE15 105
KEY(1)=4; KEY(2)=0; KEY(3)=0; KEY(4)=0# Set search keys                 PE15 106
indexpkt:(1,lrscat:,LPAK,IP,0,KEY,IWNT,RELPT)# Test for wanted items    PE15 107
IF(KEY(4)<=0) iquit:(PE1515)#          Items not found-exit             PE15 108
SUMZ=0.; SUMN=0.; A=0.#                Zero sum of n*z**2               PE15 109
DO J=1,10#                             Loop for 10 types                PE15 110
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  PE15 111
readwpkt:(1,lrscat:,LPAK,IP,0)#        Read atom type param.            PE15 112
IF(IP<=0) BREAK#                       Test for end of types            PE15 113
ANUM(J)=QX(IP+RELPT(1))#               Store atomic number              PE15 114
ARAD(J)=QX(IP+RELPT(2))+0.15#          Store bond radius                PE15 115
movertoc:(QX(IP+RELPT(3)),1,ATYP,J*8-7,8,0)# Store atom type name       PE15 116
SUMZ=SUMZ+QX(IP+RELPT(4))*ANUM(J)**2#  Sum of n*z**2 for cell           PE15 117
IF(ANUM(J)>2.) SUMN=SUMN+QX(IP+RELPT(4))# Sum of non-h atoms in cell    PE15 118
A=MAX(A,ANUM(J))#                      Get largest z value              PE15 119
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  PE15 120
SUMN=SUMN/FLOAT(NSYM)#                 Non-h sites in asymm unit        PE15 121
IF(NTPR<0)#                            Has triple sum been decided?     PE15 122
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  PE15 123
IF(A>20.) NTPR=no:#                    Switch off for heavy atoms       PE15 124
ELSE      NTPR=yes:#                   Otherwise use triplets           PE15 125
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  PE15 126
#                                                                       PE15 127
#    ATOM SITES FROM FILE A                                             PE15 128
#    ----------------------                                             PE15 129
#                                                                       PE15 130
IWNT(1)=1;   IWNT(2)=2;   IWNT(3)=3#   Request x,y,z                    PE15 131
IWNT(4)=22;  IWNT(5)=14#               Sf pointer & atom name           PE15 132
KEY(1)=5; KEY(2)=0; KEY(3)=0; KEY(4)=0# Set search keys                 PE15 133
indexpkt:(1,lratom:,LPAK,IP,0,KEY,IWNT,RELPT)# Test for wanted items    PE15 134
N=RELPT(5)#                            Store the atom name pointer      PE15 135
IF(IP>0&KEY(4)>0)#                     If atoms present check list      PE15 136
REPEAT#                                Loop over input atoms            PE15 137
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 PE15 138
readwpkt:(1,lratom:,LPAK,IP,0)#        Read atom param.                 PE15 139
IF(IP<=0) BREAK#                       Test for end of atoms            PE15 140
movertoc:(QX(IP+N),1,NAME,1,24,0)#     Store the atom name              PE15 141
FOR(IA=MARKA;IA<STOPA;IA=IA+STEPA)#    Loop thro. atoms list            PE15 142
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 PE15 143
ifne:(aw:,0.) NEXT#                    Skip if atom has been loaded     PE15 144
movertoc:(an:,1,TEMP,1,24,0)#          Store the atom name              PE15 145
IF(NAME!=TEMP) NEXT#                   Skip if no match                 PE15 146
label:(NAME,I,J)#                      Process atom label               PE15 147
ax:=QX(IP+RELPT(1))#                   X frac.                          PE15 148
ay:=QX(IP+RELPT(2))#                   Y frac.                          PE15 149
az:=QX(IP+RELPT(3))#                   Z frac.                          PE15 150
aw:=1.#                                Site weight                      PE15 151
at:=QX(IP+RELPT(4))#                   Atomic sf array address          PE15 152
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 PE15 153
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 PE15 154
readwpkt:(1,endrecord:,I,J,0)#         Finish off this record           PE15 155
##                                                                      PE15 156
##   CALCULATE FRACTIONAL-TO-CARTESIAN MATRICES (cell c along           PE15 157
##   ------------------------------------------  Cartesian z)           PE15 158
##                                                                      PE15 159
DO N=1,NFRG#                           For all fragments                PE15 160
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 PE15 161
ifeq:(FTOC(1,N),0.) DO I=1,6; FTOC(I,N)=CELL(I)# Store std cell         PE15 162
A=FTOC(1,N);  B=FTOC(2,N); C=FTOC(3,N)# Get cell lengths                PE15 163
U=FTOC(4,N);  V=FTOC(5,N); W=FTOC(6,N)# Get cell angles                 PE15 164
SA=SIN(U*R);  SB=SIN(V*R); SG=SIN(W*R)# Get sine of angles              PE15 165
CA=COS(U*R);  CB=COS(V*R); CG=COS(W*R)# Get cosine of angles            PE15 166
CGS=(CA*CB-CG)/(SA*SB); SGS=SQRT(1.-CGS**2)# Get c/s gamma*             PE15 167
DO I=1,9; FTOC(I,N)=0.#                Zero the matrix                  PE15 168
FTOC(1,N)=+A*SB*SGS#                   +a sinbeta singamma*             PE15 169
FTOC(4,N)=-A*SB*CGS#                   -a sinbeta cosgamma*             PE15 170
FTOC(7,N)=+A*CB#                       +a cosbeta                       PE15 171
FTOC(5,N)=+B*SA#                       +b sinalpha                      PE15 172
FTOC(8,N)=+B*CA#                       +b cosalpha                      PE15 173
FTOC(9,N)=+C#                          +c                               PE15 174
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 PE15 175
#                                                                       PE15 176
#    CONVERT ATOM SITES TO CARTESIAN COORDS                             PE15 177
#    --------------------------------------                             PE15 178
#                                                                       PE15 179
N=1;   M=0;    W=0.#                   Set fragment counter and sum     PE15 180
HEADOT=' '; HEADOT(LABMAX+10^)=PE4#    Store header line                PE15 181
FOR(IA=MARKA;IA<STOPA;IA=IA+STEPA)#    Loop thro. atoms list            PE15 182
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 PE15 183
WHILE(FADR(N)==IA) N=N+1#              Incr the fragment counter        PE15 184
#                                                                       PE15 185
IF(at:>-1.)#                           Test for fract-to-cartesn        PE15 186
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14 PE15 187
cx:=ax:*FTOC(1,N)#                     Cartesian x                      PE15 188
cy:=ax:*FTOC(4,N)+ay:*FTOC(5,N)#       Cartesian y                      PE15 189
cz:=ax:*FTOC(7,N)+ay:*FTOC(8,N)+az:*FTOC(9,N)# Cartesian z              PE15 190
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14 PE15 191
#                                                                       PE15 192
movertoc:(an:,1,NAME,1,24,0)#          Extract atom label               PE15 193
label:(NAME,I,J,ATYP,K)#               Identify atom type               PE15 194
IF(K==0) iquit:(PE1530)#               Atom not identified              PE15 195
at:=FLOAT(K)#                          Store atom type number           PE15 196
aw:=aw:*ANUM(K)#                       Weight = weight*z                PE15 197
W=W+aw:**2#                            Sum (w*z)**2 for fragments       PE15 198
#                                                                       PE15 199
IF(N>M)#                               Test if new fragment             PE15 200
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 15 PE15 201
M=N;   LINRM=8;    J=FTYP(N)#          Reset current frag counter       PE15 202
movechar:(PE3,10,CHROT,2,NPE1-1,1)#    Underline frag header            PE15 203
writeline:(1,PE1,NPE1,2,3)#            Print frag header                PE15 204
IF(IXYZ==1)#                           If fractional coords             PE15 205
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 16 PE15 206
writeline:(0,PE3,NPE3,3,3)#            Print matrix header              PE15 207
DO I=1,7,3#                            Loop over matrix                 PE15 208
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 17 PE15 209
ncodefld:(FTOC(I,N),1,CHROT,FMT1,3)#   Format matrix elements           PE15 210
writeline:(0,0,0,1,3)#                 Print matrix elements            PE15 211
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 17 PE15 212
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 16 PE15 213
IF(IXYZ==1)writeline:(0,0,0,1,3);  LINRM=8# Force first atom header     PE15 214
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 15 PE15 215
movertoc:(an:,1,CHROT,2,24,0)#         Insert atom name                 PE15 216
LCHROT=LABMAX+4#                       Set format pointer               PE15 217
ncodefld:(QX,IA+1,CHROT,FMT2,8)#       Format atom data                 PE15 218
writeline:(0,HEADOT,120,1,3)#          Print atom data                  PE15 219
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 PE15 220
ifeq:(W,0.0)#                          Summed weights is still zero     PE15 221
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 18 PE15 222
WRITE(6,*) " No fragment information supplied - exiting."#              PE15 223
#WRITE(6,*) " IDEALLY SHOULD FUDGE IT BASED ON HEAVY ATOMS"#            PE15 224
iquit:(PE1501)#                                                         PE15 225
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 18 PE15 226
SUMZ=W*FLOAT(NSYM)/SUMZ#               Get scattering fraction of fra   PE15 227
RVEC=MIN(1.,RVEC/SUMZ)#                Vector ratios wrt total vector   PE15 228
ncodefld:(SUMZ,1,PE6,500632.,1)#       Format scatt fraction of frag    PE15 229
writeline:(1,PE6,NPE6,3,3)#            Print scattering fraction        PE15 230
SUMZ=SQRT(SUMZ)#                       Square root for tran cfom use    PE15 231
SCAL=FLOAT((STOPA-MARKA)/STEPA+1)#     Number of sites entered          PE15 232
SUMN=(SUMN-SCAL)*1.2#                  Number of non-h sites missing    PE15 233
SCAL=W/SCAL#                           Average w*z**2 for patt scale    PE15 234
#                                                                       PE15 235
#    PRINT BOND DISTANCE & ANGLES                                       PE15 236
#    ----------------------------                                       PE15 237
#                                                                       PE15 238
IF(GLIS>0)#                            Print if requested               PE15 239
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 19 PE15 240
HEADOT=' Atoms'; HEADOT(LABMAX*2+11^)=PE5# Store header line            PE15 241
writeline:(0,0,0,1,3);  LINRM=6#       Force list header                PE15 242
FOR(IA=IATM(2);IA<JATM(2);IA=IA+STEPA)# Loop over r/t atoms             PE15 243
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 20 PE15 244
movertoc:(an:,1,CHROT,2,24,0)#         Insert atom name 1               PE15 245
J=0; DIST=ARAD(NINT(at:))#             Zero angle counter               PE15 246
FOR(I=IATM(2);I<JATM(2);I=I+STEPA)#    Loop over r/t atoms              PE15 247
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 21 PE15 248
IF(I==IA) NEXT#                        Skip identical atoms             PE15 249
U=(cx:-cx:(I))**2+(cy:-cy:(I))**2+(cz:-cz:(I))**2# Distance             PE15 250
IF(U>(DIST+ARAD(NINT(at:(I))))**2) NEXT# Skip excessive distance        PE15 251
J=J+1;   KEEP(J)=I#                    Keep the atom address            PE15 252
movertoc:(an:(I),1,CHROT,LABMAX+3,24,0)# Insert atom name 2             PE15 253
LCHROT=LABMAX*2+6#                     Store format pointer             PE15 254
ncodefld:(SQRT(U),1,CHROT,832.,1)#     Format distance                  PE15 255
#                                                                       PE15 256
DO K=1,J-1#                            Loop over stored atom addresse   PE15 257
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 22 PE15 258
N=KEEP(K)#                             Get atom address                 PE15 259
V=(cx:   -cx:(N))**2+(cy:   -cy:(N))**2+(cz:   -cz:(N))**2# D(ia-n)     PE15 260
W=(cx:(I)-cx:(N))**2+(cy:(I)-cy:(N))**2+(cz:(I)-cz:(N))**2# D(i-n)      PE15 261
W=(U+V-W)/(2.*SQRT(U*V))#              Get cosine of angle              PE15 262
W=57.29578*ATAN2(SQRT(ABS(1.-W*W)),W)# Get angle                        PE15 263
ncodefld:(W,1,CHROT,812.,1)#           Format angle                     PE15 264
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 22 PE15 265
writeline:(0,HEADOT,120,1,3)#          Print atom dist and angles       PE15 266
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 21 PE15 267
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 20 PE15 268
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 19 PE15 269
#                                                                       PE15 270
#   SET LIST MARKERS                                                    PE15 271
#   ----------------                                                    PE15 272
#                                                                       PE15 273
MARKV=STOPS;   STEPV=4#                Set vector list pointers         PE15 274
I=10*(STOPA-MARKA)/STEPA#              Get maximum test vectors         PE15 275
incrqx:(STOPV,MARKV+I*STEPV,PE1540)#   Get best fom list memory         PE15 276
MARKB=STOPV;   STEPB=15#               Set best fom list pointers       PE15 277
I=RSOL+3;   J=TSOL+3;  K=MAX(I,J)#     Allow for list expansion         PE15 278
STOPBR=MARKB+I*STEPB#                  Set best list rotation stop      PE15 279
STOPBT=MARKB+J*STEPB#                  Set best list transln. stop      PE15 280
incrqx:(STOPB,MARKB+K*STEPB,PE1540)#   Get best fom list memory         PE15 281
MARKK=STOPB;   STEPK=15;  JK=MARKK#    Set keep fom list pointers       PE15 282
incrqx:(STOPK,MARKK+3*I*STEPK,PE1540)# Get keep fom list memory         PE15 283
MARKC=STOPK;   STEPC=30;  JC=MARKC#    Set comb fom list pointers       PE15 284
incrqx:(STOPC,MARKC+3*K*STEPC,PE1540)# Get comb fom list memory         PE15 285
#                                                                       PE15 286
MARKE=STOPC;  STOPE=STOPC;  STEPE=6#   Define e list                    PE15 287
MARKT=STOPC;  STOPT=STOPC;  STEPT=6#   Define triplet list              PE15 288
MARKP=STOPC;  STOPP=STOPC;  STEPP=9#   Define phase list                PE15 289
#                                                                       PE15 290
RETURN#                                                                 PE15 291
END#                                                                    PE15 292
systemheader:(PE20)#                   Read e's and triplets file d     PE20   1
####################################################################    PE20   2
#                                                                  #    PE20   3
#                                PE20                              #    PE20   4
#                                                                  #    PE20   5
####################################################################    PE20   6
SUBROUTINE PE20#                       S.r. hall nov 88                 PE20   7
include:(AACOMN)#                                                       PE20   8
include:(PECOMN)#                                                       PE20   9
#                                                                       PE20  10
INTEGER I,J,K,EMAX,N1,N2,N3,NN,IS,ID,IAA#                               PE20  11
INTEGER N4,NP,NREC#                                                     PE20  12
REAL    ATHR,P#                                                         PE20  13
#                                                                       PE20  14
#   ENTER E-LIST PARAMETERS                                             PE20  15
#   -----------------------                                             PE20  16
#                                                                       PE20  17
readwpkt:(4,2,NP,IP,0)#                Point to record 2, file 4        PE20  18
IF(IP<=0) iquit:(PE2005)#              Triplet file missing             PE20  19
EMIN=MAX(EMIN,QX(IP+2))#               Extract /e/-min of generator     PE20  20
NUME=MIN0(NUME,NINT(QX(IP+4)))#        Extract no. nonpsi0 gens.        PE20  21
AMIN=MAX(AMIN,QX(IP+29))#              Extract min probability factor   PE20  22
#                                                                       PE20  23
FOR(J=MARKE;J>0;J=STOPE)#              Loop over generator list         PE20  24
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  PE20  25
readwpkt:(4,3,NP,IP,0)#                Point to next gen. packet        PE20  26
IF(IP<=0)         BREAK#               Exit at end of list              PE20  27
IF(QX(IP+5)<EMIN) BREAK#               Exit if below emin               PE20  28
incrqx:(STOPE,J+STEPE,PE2010)#         Expand elist                     PE20  29
movereal:(QX,IP+1,QX,J+1,NP,1)#        Store hkl,uvw,prc,sige,e         PE20  30
es:(J)=0.;     ec:(J)=0.#              Reset uvw for triplet count      PE20  31
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  PE20  32
NUME=MIN(NUME,(STOPE-MARKE)/STEPE)#    Calculate generators used        PE20  33
STOPE=MARKE+NUME*STEPE#                Make sure list end ok!           PE20  34
EMIN=ee:(STOPE-STEPE)#                 Get current minimum e            PE20  35
MARKT=STOPE;  STOPT=STOPE#             Set the start of triple list     PE20  36
#                                                                       PE20  37
#   ENTER TRIPLETS                                                      PE20  38
#   --------------                                                      PE20  39
#                                                                       PE20  40
plastbitsword:(NINT(AMIN*128.),ATHR)#  Pack a-limit for test            PE20  41
EMAX=NUME+1#                           Generator limit for test         PE20  42
NUMT=0;       AMIN=10000.#             Zero unique triplet sum          PE20  43
DO N1=1,NUME#                          Loop over all generators         PE20  44
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  PE20  45
IE=MARKE+(N1-1)*STEPE#                 Point to elist                   PE20  46
NREC=N1+3#                             Set correct file4 recnum         PE20  47
FOR(IT=STOPT;IT>0;IT=STOPT)#           Loop over all invariants         PE20  48
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  PE20  49
readwpkt:(4,NREC,NP,IP,0)#             Point to invariant packet        PE20  50
IF(IP<=0) BREAK#                       Exit if no more                  PE20  51
generatornum:(IP,NN,1)#                Max gen of triplet or quartet    PE20  52
IF(NN>EMAX) NEXT#                      Ignore si with nn>ngen   wd3     PE20  53
generatornum:(IP,N4,4)#                Generator 4 of a quartet wd6     PE20  54
IF(N4!=0) BREAK#                       Test if invariant a triplet      PE20  55
comppakwdone:(IP,LT,ATHR,NEXT)#        Ignore trip if a<amin            PE20  56
generatornum:(IP,N2,2)#                Get n2 value             wd4     PE20  57
generatornum:(IP,N3,3)#                Get n3 value             wd5     PE20  58
IF(N1>=N2\N1>=N3\N2>=N3) NEXT#         Test for unique invariant        PE20  59
IF(N3>=EMAX) NEXT#                     Test for value outside range     PE20  60
phisigncode:(IP,IS)#                   Get phi sign code (0-7)  wd1     PE20  61
phiinvshift:(IP,ID)#                   Get si shift (deg/30)    wd1     PE20  62
probfactor:(IP,IAA)#                   Get ab value*128         wd1     PE20  63
incrqx:(STOPT,IT+STEPT,PE2015)#        Get memory for triplet           PE20  64
J=MARKE+((N2-1)*STEPE)#                Convert n2 as genlist addr       PE20  65
K=MARKE+((N3-1)*STEPE)#                Convert n3 as genlist addr       PE20  66
P=FLOAT(MOD(ID*30+3600,360))*DTOR#     Total shift in radians           PE20  67
es:=es:+1; es:(J)=es:(J)+1.; es:(K)=es:(K)+1.# Count triplets           PE20  68
t1:=FLOAT(IE); t2:=FLOAT(J); t3:=FLOAT(K)# Store elist addresses        PE20  69
tc:=COS(P);    ts:=SIN(P)#             Cos and sin of phase shift       PE20  70
ti:=FLOAT(IS+1)#                       Sign index for hkl2 hkl3         PE20  71
NUMT=NUMT+1#                           Count total triplets             PE20  72
AMIN=MIN(AMIN,FLOAT(IAA)/128.)#        Get smallest a value         ^   PE20  73
ifeq:(es:,8.)  BREAK#                  Exit if 8 triplets connected     PE20  74
IF(NUMT==MAXT) BREAK#                  Exit if t3max  exceeded          PE20  75
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  PE20  76
IF(NUMT==MAXT) BREAK#                  Exit if t3max  exceeded          PE20  77
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  PE20  78
readwpkt:(4,endrecord:,I,I,0)#         Rewind and close file 4          PE20  79
MARKP=STOPT; STOPP=MARKP+NUME*NSYM*STEPP# Set length of phase list      PE20  80
#                                                                       PE20  81
RETURN#                                                                 PE20  82
END#                                                                    PE20  83
systemheader:(PE25)#                   Read patterson map file c        PE25   1
####################################################################    PE25   2
#                                                                  #    PE25   3
#                                PE25                              #    PE25   4
#                                                                  #    PE25   5
####################################################################    PE25   6
SUBROUTINE PE25#                       S.r. hall nov 88                 PE25   7
include:(AACOMN)#                                                       PE25   8
include:(PECOMN)#                                                       PE25   9
#                                                                       PE25  10
REAL P,SUMP#                                                            PE25  11
INTEGER I,IMAP,J,K,MAXP#                                                PE25  12
INTEGER MAPI(3),MAPN(3),MAPJ(3)#                                        PE25  13
INTEGER PSIZ#                                                           PE25  14
chardata:(PE1,' PATSEE Search Data=')#                                  PE25  15
chardata:(PE2,' Number of map points X, Y, Z                    -')#    PE25  16
chardata:(PE3,' QX storage needed for map (words)                ')#    PE25  17
chardata:(PE4,' Input map density limits  min/max                 ')#   PE25  18
chardata:(PE5,' Reset map density limits  min/max                 ')#   PE25  19
chardata:(PE6,' Map type used in Patterson vector search          ')#   PE25  20
chardata:(PE7,'F**2    |E|*|F| E**2-1  ')#                              PE25  21
realdata:(FMT1,[400413.,450413.,500413.])#                              PE25  22
datastuff:#                                                             PE25  23
#                                                                       PE25  24
#     GET MAP PARAMETERS FROM MAP FILE                                  PE25  25
#     --------------------------------                                  PE25  26
#                                                                       PE25  27
readwpkt:(3,21,PSIZ,IP,0)#             Read map parameter record        PE25  28
IF(IP<=0)  GOTO 100#                   This is not fourr op             PE25  29
IMAP=NINT(QX(IP+21))#                  Extract the map type             PE25  30
IF(IMAP>3)GOTO 100#                    That vector/patt map             PE25  31
DO K=1,3#                              Loop over map axes               PE25  32
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  PE25  33
J=IP+K#                                Set map pointer                  PE25  34
ifne:(QX(J  ),0.)         GOTO 100#    Origins must be at zero          PE25  35
ifne:(QX(J+3),1.)         GOTO 100#    Must be singular units           PE25  36
ifne:(QX(J+6)+1.,QX(J+9)) GOTO 100#    Must be full cell                PE25  37
MAPN(K)=NINT(QX(J+9))#                 Calculated points                PE25  38
MAPI(K)=NINT(QX(J+12))#                Cell axis to map axis pointer    PE25  39
MAPJ(K)=NINT(QX(J+15))#                Map axis to cell axis pointer    PE25  40
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  PE25  41
GOTO 200#                              The map is ok                    PE25  42
100 CONTINUE#                                                           PE25  43
readwpkt:(3,endrecord:,I,J,0)#         Finish off file c                PE25  44
#                                                                       PE25  45
FILEXT(IOUNIT(3))='MAP'#               Alternate patterson map file     PE25  46
readwpkt:(3,21,PSIZ,IP,0)#             Read map parameter record        PE25  47
IF(IP<=0)         iquit:(PE2505)#      This is not fourr op             PE25  48
IMAP=NINT(QX(IP+21))#                  Extract the map type             PE25  49
IF(IMAP>3) iquit:(PE2510)#             Test that vector/patt map        PE25  50
DO K=1,3#                              Loop over map axes               PE25  51
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  PE25  52
J=IP+K#                                Set map pointer                  PE25  53
ifne:(QX(J  ),0.)         iquit:(PE2515)# Origins must be at zero       PE25  54
ifne:(QX(J+3),1.)         iquit:(PE2515)# Must be singular units        PE25  55
ifne:(QX(J+6)+1.,QX(J+9)) iquit:(PE2515)# Must be full cell             PE25  56
MAPN(K)=NINT(QX(J+9))#                 Calculated points                PE25  57
MAPI(K)=NINT(QX(J+12))#                Cell axis to map axis pointer    PE25  58
MAPJ(K)=NINT(QX(J+15))#                Map axis to cell axis pointer    PE25  59
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  PE25  60
#                                                                       PE25  61
##   Note that MAPI indicate the direction of each map axis             PE25  62
##   MAPI(1) for X, MAPI(2) for Y, .... values 1=layer; 3=accross       PE25  63
##   Note that MAPJ indicate the direction of each cell axis            PE25  64
##   MAPJ(1) for layer, MAPJ(2) for down, .... values 1=X; 3=Z          PE25  65
##   Note that MAPN indicates grid in each map direction                PE25  66
##   MAPN(1) for layers, MAPN(2) lines/lay, MAPN(3) points/line         PE25  67
#                                                                       PE25  68
#    CALCULATE MEMORY NEEDED FOR MAP                                    PE25  69
#    -------------------------------                                    PE25  70
#                                                                       PE25  71
200 CONTINUE#                                                           PE25  72
MARKM=STOPP;   I=QXMAX-MARKM#          Get max words available          PE25  73
MAXP=MAPN(1)*MAPN(2)*MAPN(3)#          Get total points input           PE25  74
incrqx:(STOPM,MARKM+MAXP,PE2520)#      Get map memory                   PE25  75
INC(1)=FLOAT(MAPN(MAPI(1))-1)#         Set map increment x axis         PE25  76
INC(2)=FLOAT(MAPN(MAPI(2))-1)#         Set map increment y axis         PE25  77
INC(3)=FLOAT(MAPN(MAPI(3))-1)#         Set map increment z axis         PE25  78
OFF(MAPJ(3))=1#                        Set map offset for point         PE25  79
OFF(MAPJ(2))=MAPN(3)#                  Set map offset for line          PE25  80
OFF(MAPJ(1))=MAPN(2)*MAPN(3)#          Set map offset for layer         PE25  81
#                                                                       PE25  82
#   READ AND STORE PATTERSON MAP                                        PE25  83
#   ----------------------------                                        PE25  84
#                                                                       PE25  85
PMAX=0.;  PMIN=0.;  SUMP=0.#           Zero density sums                PE25  86
FOR(J=MARKM;J<STOPM;J=J+MAPN(3))#      Read & store patterson map       PE25  87
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  PE25  88
readwpkt:(3,22,PSIZ,IP,0)#             Read in line of layer            PE25  89
IF(IP<=0) iquit:(PE2525)#              Premature end to patt map        PE25  90
DO K=1,MAPN(3)#                        Loop over points in line         PE25  91
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  PE25  92
P=QX(IP+K);  QX(J+K)=P#                Store density point              PE25  93
PMAX=MAX(P,PMAX)#                      Store the maximum density        PE25  94
PMIN=MIN(P,PMIN)#                      Store the minimum density        PE25  95
SUMP=SUMP+P#                           Sum density points               PE25  96
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  PE25  97
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  PE25  98
readwpkt:(3,endrecord:,I,J,0)#         Finish off file c                PE25  99
#                                                                       PE25 100
#    PRINT MAP PARAMETERS                                               PE25 101
#    --------------------                                               PE25 102
#                                                                       PE25 103
LINRM=20#                              Ensure sufficient space          PE25 104
movechar:(PE1,NPE1,CHROT,2,NPE1-2,1)#  Insert header underline          PE25 105
writeline:(2,PE1,NPE1-1,2,3)#          Print patterson header           PE25 106
movechar:(PE7,IMAP*8-7,PE6,44,8,0)#    Insert map type                  PE25 107
writeline:(1,PE6,NPE6,3,3)#            Print map type                   PE25 108
DO I=1,3; BUFOT(I)=INC(I)+1.#          Get total map points             PE25 109
ncodefld:(BUFOT,1,PE2,FMT1,3)#         Format number of points          PE25 110
writeline:(0,PE2,NPE2,3,3)#            Print patterson maximum          PE25 111
ncodefld:(FLOAT(MAXP),1,PE3,500913.,1)# Format number of words          PE25 112
writeline:(0,PE3,NPE3,3,3)#            Put patterson stats header       PE25 113
ncodefld:(PMIN,1,PE4,400503.,1)#       Format density minimum           PE25 114
ncodefld:(PMAX,1,PE4,500503.,1)#       Format density maximum           PE25 115
writeline:(0,PE4,NPE4,3,3)#            Put patterson stats header       PE25 116
#                                                                       PE25 117
#    DEFINE MAP PARAMETERS FOR SEARCH                                   PE25 118
#    --------------------------------                                   PE25 119
#                                                                       PE25 120
MARKM=MARKM+1;    P=0.#                Point to first map point         PE25 121
SUMP=SUMP/FLOAT(MAXP)#                 Get average density              PE25 122
SCAL=SCAL/ABS(PMIN-SUMP)#              Get density scale                PE25 123
DO I=MARKM,STOPM; QX(I)=(QX(I)-SUMP)*SCAL# Adjust density               PE25 124
PMIN=(PMIN-SUMP)*SCAL#                 Adjust min density               PE25 125
PMAX=(PMAX-SUMP)*SCAL#                 Adjust max density               PE25 126
ncodefld:(PMIN,1,PE5,400503.,1)#       Format density minimum           PE25 127
ncodefld:(PMAX,1,PE5,500503.,1)#       Format density maximum           PE25 128
writeline:(0,PE5,NPE5,3,3)#            Put patterson densities          PE25 129
DO I=1,3; P=P+CELL(I)/FLOAT(MAPN(MAPI(I))+1)# Sum map resolutions       PE25 130
MAPR=MIN(MAX(P/3.,0.4),0.7)#           Set map resolution               PE25 131
PMIN=ABS(PMIN)#                        Store absolute of pmin           PE25 132
#                                                                       PE25 133
RETURN#                                                                 PE25 134
END#                                                                    PE25 135
systemheader:(PE30)#                   Prepare test atoms list          PE30   1
####################################################################    PE30   2
#                                                                  #    PE30   3
#                                PE30                              #    PE30   4
#                                                                  #    PE30   5
####################################################################    PE30   6
SUBROUTINE PE30#                       S.r. hall nov 88                 PE30   7
include:(AACOMN)#                                                       PE30   8
include:(PECOMN)#                                                       PE30   9
#                                                                       PE30  10
INTEGER I,IS,J,JA,K,M,N,NTRY,SYM#                                       PE30  11
REAL P,Q,R#                                                             PE30  12
REAL TEST(2)#                          Test limits short-dist atoms     PE30  13
REAL D,DMIN,DMAX#                                                       PE30  14
REAL X,Y,Z,U,V,W#                                                       PE30  15
#                                                                       PE30  16
chardata:(PE1,' Twist rotations applied  min<incr>max           - deg')#PE30  17
chardata:(PE2,' Rot/tran atoms used in close contact test       -')#    PE30  18
chardata:(PE3,' Transln. atoms used in close contact test       -')#    PE30  19
chardata:(PE4,' Min inter-mol. distance of fixed atoms is         A')#  PE30  20
chardata:(PE5,' Number of rotation searches attempted           -')#    PE30  21
chardata:(PE6,' Number of rotation solutions allowed            -')#    PE30  22
chardata:(PE7,' Number of transln. searches attempted           -')#    PE30  23
chardata:(PE8,' Number of transln. solutions allowed            -')#    PE30  24
chardata:(PE9,' Maximum rotation applied about X,Y,Z  xxx xxx xxx deg')#PE30  25
chardata:(PEC,' Minimum rotation applied about X,Y,Z  xxx xxx xxx deg')#PE30  26
chardata:(PEA,' Maximum fractional shift on X,Y,Z                ')#    PE30  27
chardata:(PED,' Minimum fractional shift on X,Y,Z                ')#    PE30  28
chardata:(PEB,' Number of refinement searches/solution          -')#    PE30  29
realdata:(FMT1,[420313.,460313.,500313.])#                              PE30  30
realdata:(FMT2,[400422.,450422.,500422.])#                              PE30  31
datastuff:#                                                             PE30  32
#                                                                       PE30  33
#   CALCULATE NUMBER OF ENVELOPE ATOMS                                  PE30  34
#   ----------------------------------                                  PE30  35
#                                                                       PE30  36
DO I=1,3#                              Loop thru search atom types      PE30  37
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  PE30  38
NATM(I)=(JATM(I)-IATM(I))/STEPA#       Get atom numbers in list         PE30  39
NENV(I)=MIN(MAX(NATM(I)/5,10),50,NATM(I))# Number envelope atoms        PE30  40
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  PE30  41
M=NENV(2)+NENV(3)#                     Total atoms in envelope          PE30  42
IF(M>50)#                              Reduce if limit exceeded         PE30  43
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  PE30  44
I=2;   J=3#                            Set atom type indices            PE30  45
IF(NENV(2)>NENV(3))#                   Test if less r/t atoms           PE30  46
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  PE30  47
I=3;   J=2#                            Reset atom type indices          PE30  48
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  PE30  49
NENV(I)=NENV(I)-(M-50)*NATM(I)/(NATM(I)+NATM(J))# Env atoms r/t frag    PE30  50
NENV(J)=50-NENV(I)#                    Define env atoms t frag          PE30  51
M=50#                                  Set number of envelope atoms     PE30  52
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  PE30  53
NENV(3)=M#                             Set last number to limit         PE30  54
NORA=MIN(NATM(2),3)#                   Limit of orienting atoms         PE30  55
#                                                                       PE30  56
#    FIND MINIMUM DISTANCE FOR FIXED ATOMS                              PE30  57
#    -------------------------------------                              PE30  58
#                                                                       PE30  59
IF(NATM(1)>0)#                         If fixed atoms present           PE30  60
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  PE30  61
DMIN=9999.#                            Set starting d minimum           PE30  62
FOR(JA=IATM(1);JA<JATM(1);JA=JA+STEPA)# Loop over fixed atoms           PE30  63
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  PE30  64
X=cx:(JA);  Y=cy:(JA);  Z=cz:(JA)#     Set atom 1 coords                PE30  65
FOR(IA=JA;IA<JATM(1);IA=IA+STEPA)#     Loop over fixed atoms            PE30  66
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  PE30  67
U=cx:;      V=cy:;      W=cz:#         Set atom 2 coords                PE30  68
FOR(IS=MARKS+STEPS;IS<STOPS;IS=IS+STEPS)# Loop over symops              PE30  69
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  PE30  70
P=U*QX(IS+1)+V*QX(IS+4)+W*QX(IS+7)+QX(IS+10)# Get symoped x'            PE30  71
Q=U*QX(IS+2)+V*QX(IS+5)+W*QX(IS+8)+QX(IS+11)# Get symoped y'            PE30  72
R=U*QX(IS+3)+V*QX(IS+6)+W*QX(IS+9)+QX(IS+12)# Get symoped z'            PE30  73
P=MOD(99.5+P-X,1.)-0.5#                Get smallest x distance          PE30  74
Q=MOD(99.5+Q-Y,1.)-0.5#                Get smallest y distance          PE30  75
R=MOD(99.5+R-Z,1.)-0.5#                Get smallest z distance          PE30  76
D=P*P*G1+Q*Q*G2+R*R*G3+P*Q*G4+P*R*G5+Q*R*G6# Get distance               PE30  77
DMIN=MIN(DMIN,D)#                      Save smallest distance           PE30  78
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  PE30  79
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  PE30  80
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  PE30  81
ncodefld:(SQRT(DMIN),1,PE4,500632.,1)# Format min distance              PE30  82
writeline:(1,PE4,NPE4,3,3)#            Print fixed atom min distance    PE30  83
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  PE30  84
#                                                                       PE30  85
#    SELECT ENVELOPE ROT/TRAN ATOMS                                     PE30  86
#    ------------------------------                                     PE30  87
#                                                                       PE30  88
N=0#                                   Start envelope atom count        PE30  89
TEST(1)=0.;   TEST(2)=-10.#            Tests for short-dist atoms       PE30  90
DO IT=2,3#                             For each type (2=r/t; 3=r)       PE30  91
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  PE30  92
DMAX=0.#                               Start distance search            PE30  93
DO J=1,2#                              Repeat for short-dist atoms      PE30  94
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  PE30  95
FOR(IA=IATM(IT);IA<JATM(IT);IA=IA+STEPA)# Loop over atoms list          PE30  96
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 PE30  97
IF(at:<TEST(J)) NEXT#                  Avoid short-distance atoms       PE30  98
D=cx:**2+cy:**2+cz:**2#                Get distance from origin         PE30  99
IF(D<=DMAX)     NEXT#                  Skip smaller distance            PE30 100
DMAX=D;   JA=IA#                       Keep largest distance            PE30 101
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 PE30 102
IF(DMAX>0.) BREAK#                     Avoid short distance atoms       PE30 103
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  PE30 104
#                                                                       PE30 105
DO K=1,2#                              Repeat for short-dist atoms      PE30 106
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 PE30 107
REPEAT#                                Loop over env atom list          PE30 108
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 PE30 109
IF(N==NENV(IT)) BREAK#                 Exit r/t env atom search         PE30 110
N=N+1#                                 Count envelope atoms             PE30 111
IENV(N)=JA#                            Store atom address               PE30 112
aw:(JA)=-aw:(JA)#                      Flag atom as already selected    PE30 113
DMAX=0.#                               Start distance search            PE30 114
FOR(IA=IATM(IT);IA<JATM(IT);IA=IA+STEPA)# Loop over atoms list          PE30 115
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 PE30 116
IF(at:<TEST(K)) NEXT#                  Avoid short-distance atoms       PE30 117
IF(aw:<0.)      NEXT#                  Check if already selected        PE30 118
DMIN=99999.#                           Set initial minimum              PE30 119
DO I=1,N#                              Loop over envelope list          PE30 120
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14 PE30 121
J=IENV(I)#                             Get atom list address            PE30 122
D=(cx:-cx:(J))**2+(cy:-cy:(J))**2+(cz:-cz:(J))**2# Get atom distance    PE30 123
DMIN=MIN(D,DMIN)#                      Keep closest distance            PE30 124
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14 PE30 125
IF(DMIN<=DMAX)    NEXT#                Skip if smaller distance         PE30 126
DMAX=DMIN;   JA=IA#                    Keep largest distance            PE30 127
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 PE30 128
IF(DMAX>4.) NEXT#                      Exit if < mintact dist           PE30 129
NENV(IT)=N#                            Reset number of env atoms        PE30 130
IF(IT==2) NENV(3)=N#                   Set t only limit as well         PE30 131
GOTO 100#                              Exit from this loop              PE30 132
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 PE30 133
IF(N>NENV(IT)) BREAK#                  Avoid short distance atoms       PE30 134
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 PE30 135
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  PE30 136
100 CONTINUE#                          Exit if min distance found       PE30 137
FOR(IA=MARKA;IA<STOPA;IA=IA+STEPA) aw:=ABS(aw:)# Reset flagged atoms    PE30 138
#                                                                       PE30 139
#    SORT & PRINT ENVELOPE ATOM SUMMARY                                 PE30 140
#    ----------------------------------                                 PE30 141
#                                                                       PE30 142
ncodefld:(FLOAT(WMIN),1,PE1,420413.,1)# Format min twist angle          PE30 143
ncodefld:(FLOAT(WINC),1,PE1,460413.,1)# Format min twist angle          PE30 144
ncodefld:(FLOAT(WMAX),1,PE1,500413.,1)# Format min twist angle          PE30 145
IF(TWIS>0) writeline:(1,PE1,NPE1,3,3)# Print twist angles               PE30 146
ncodefld:(FLOAT(NENV(2)),1,PE2,500313.,1)# Format rot/tran number       PE30 147
writeline:(0,PE2,NPE2,3,3)#            Print rot/tran number            PE30 148
I=NENV(3)-NENV(2)#                     Number of tran env atoms         PE30 149
ncodefld:(FLOAT(I),1,PE3,500313.,1)#   Format tran number               PE30 150
IF(I>0) writeline:(0,PE3,NPE3,3,3)#    Print tran number                PE30 151
##WRITE(6,'(10i5)') NATM,NENV,NORA#                                     PE30 152
##WRITE(6,'(10i5)') IATM,JATM,MARKA,STEPA#                              PE30 153
##WRITE(6,'(10i5)') (IENV(I),I=1,10)#                                   PE30 154
#                                                                       PE30 155
#    SET ROTATION RANGES ACCORDING TO SYMMETRY                          PE30 156
#    -----------------------------------------                          PE30 157
#                                                                       PE30 158
IF(ROTN==yes:)#                        If rotation search applied       PE30 159
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 15 PE30 160
IF(RMUL(1)<-999.)#                     Has rot ranges been input        PE30 161
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 16 PE30 162
DO I=1,3; RMUL(I)=370.#                Set range to 370 degrees         PE30 163
J=0#                                   Count non-orthogonal angles      PE30 164
DO I=1,3; ifne:(CELL(I+3),90.) J=J+1#  Exit if angles ne 90.            PE30 165
IF(J==3) RMUL(3)=200.#                 Set range for triclinic search   PE30 166
ELSE IF(J>0)#                          Some non-orthogonal angles       PE30 167
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 17 PE30 168
DO I=3,1,-1; ifeq:(CELL(I+3),90.) BREAK# Exit on orthogonal angle       PE30 169
RMUL(I)=110.#                          Restrict range to 110 degrees    PE30 170
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 17 PE30 171
ELSE        DO I=1,3; RMUL(I)=110.#    All ranges 110 if orthogonal     PE30 172
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 16 PE30 173
IF(RTRY==1) DO I=1,3; RMUL(I)=0.#      This is a set angle rotation     PE30 174
IF(RTRY==0)#                           Has rot tries been set yet?      PE30 175
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 18 PE30 176
RTRY=1#                                Initial number of tries          PE30 177
DO I=1,3; IF(RMUL(I)>0.)#              For all ranges applied           PE30 178
RTRY=RTRY*NINT(RMUL(I)/8.)#            One try per 8 degrees            PE30 179
RTRY=(RTRY/5000+1)*5000#               Round up to five thousands       PE30 180
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 18 PE30 181
DO I=1,3; RMUL(I)=RMUL(I)+RSET(I)#     Get max values                   PE30 182
ncodefld:(RSET,1,PEC,FMT1,3)#          Format the rotation min values   PE30 183
writeline:(1,PEC,NPEC,3,3)#            Print rotation ranges            PE30 184
ncodefld:(RMUL,1,PE9,FMT1,3)#          Format the rotation max values   PE30 185
writeline:(0,PE9,NPE9,3,3)#            Print rotation ranges            PE30 186
ncodefld:(FLOAT(RTRY),1,PE5,500613.,1)# Format orientations tried       PE30 187
writeline:(0,PE5,NPE5,3,3)#            Print orientations tried         PE30 188
ncodefld:(FLOAT(NREF),1,PEB,500513.,1)# Format rot refinements          PE30 189
IF(NREF>0)writeline:(0,PEB,NPEB,3,3)#  Print rot refinements            PE30 190
ncodefld:(FLOAT(RSOL),1,PE6,500613.,1)# Format rot solutions            PE30 191
writeline:(0,PE6,NPE6,3,3)#            Print rot solutions              PE30 192
DO I=1,3; RMUL(I)=RMUL(I)-RSET(I)#     Get max values                   PE30 193
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 15 PE30 194
#                                                                       PE30 195
#   CALCULATE TRANSLATION SYMMETRY FACTORS                              PE30 196
#   --------------------------------------                              PE30 197
#                                                                       PE30 198
IF(TRAN==yes:)#                        If translation search applied    PE30 199
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 19 PE30 200
IF(NATM(1)>0) SYM=1#                   If fixed atoms present           PE30 201
ELSE          SYM=8#                   Otherwise if no fixed atoms      PE30 202
FOR(IS=MARKS;IS<STOPS;IS=IS+STEPS)#    Loop over symops                 PE30 203
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 20 PE30 204
DO I=1,3; DO J=1,3#                    Loop symmetry elements           PE30 205
IF(I!=J&ABS(QX(IS+I*3+J-3))>0.5) GOTO 20# Test off-diag elements are    PE30 206
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 20 PE30 207
GOTO 40#                               Skip out of test                 PE30 208
20 IF(NLAT==4) GOTO 40#                Test for f-centred cell          PE30 209
ifeq:(CELL(1)-CELL(3)+CELL(5),90.) SYM=1# If a cubic cell               PE30 210
ELSE                               SYM=2# Other high symmetry           PE30 211
40 CONTINUE#                           F-centre branch                  PE30 212
#                                                                       PE30 213
#    SET DEFAULT TRANSLATION RANGES                                     PE30 214
#    ------------------------------                                     PE30 215
#                                                                       PE30 216
IF(TMUL(1)<-999.)#                     Has range been input?            PE30 217
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 21 PE30 218
DO I=1,6;   TMUL(I)=1.#                Set default translation          PE30 219
IF(SYM==8) DO I=1,3; TMUL(I)=0.5#      Reduce all tran shifts           PE30 220
IF(SYM==2) TMUL(3)=0.5#                Reduce shift z rot frag          PE30 221
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 21 PE30 222
#                                                                       PE30 223
#  CALCULATE TRANSLATION SEARCH STEPS                                   PE30 224
#  ----------------------------------                                   PE30 225
#                                                                       PE30 226
NTRY=10#                               Set initial trans tries          PE30 227
DO I=1,3#                              Loop over cell axes              PE30 228
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 22 PE30 229
STEP(I)=TRES/CELL(I)#                  Calc search step                 PE30 230
STEP(I+3)=STEP(I)#                     Assume axis is not polar         PE30 231
CELL(I+3)=CELL(I)#                     Reset for later conversion       PE30 232
J=I*4-3#                               Point to diagonal element        PE30 233
FOR(IS=MARKS;IS<STOPS;IS=IS+STEPS)#    Loop over symmetry               PE30 234
IF(QX(IS+J)<0.5) GOTO 50#              Exit if not +1.                  PE30 235
TSET(I)=0.;  TMUL(I)=0.#               Flag axis as polar               PE30 236
STEP(I)=10.;  NEXT#                    Flag axis as polar               PE30 237
50 NTRY=NTRY*NINT(TMUL(I)*CELL(I))#    Incr number of tries             PE30 238
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 22 PE30 239
IF(NATM(2)>0&NATM(3)>0) NTRY=NTRY**1.5# Incr tries if rot&tran atoms    PE30 240
IF(NSYM>1) NTRY=MIN(50000,(NTRY/100+1)*100)# Round off number tries     PE30 241
ELSE       NTRY=1#                     One try for p1                   PE30 242
IF(TTRY==0)TTRY=NTRY#                  Input number overrides all       PE30 243
TSOL=MIN(TSOL,TTRY)#                   Check number tran solutions      PE30 244
IF(NATM(2)==0) DO I=1,3; TMUL(I+3)=TMUL(I)# Translation only            PE30 245
#                                                                       PE30 246
ncodefld:(TSET,1,PED,FMT2,3)#          Format the translation min       PE30 247
writeline:(1,PED,NPED,3,3)#            Print translation ranges         PE30 248
ncodefld:(TMUL,1,PEA,FMT2,3)#          Format the translation max       PE30 249
writeline:(0,PEA,NPEA,3,3)#            Print translation ranges         PE30 250
ncodefld:(FLOAT(TTRY),1,PE7,500713.,1)# Format number of tries          PE30 251
writeline:(0,PE7,NPE7,3,3)#            Print number of tries            PE30 252
ncodefld:(FLOAT(TSOL),1,PE8,500713.,1)# Format number of solutions      PE30 253
writeline:(0,PE8,NPE8,3,3)#            Print number of solutions        PE30 254
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 19 PE30 255
#                                                                       PE30 256
RETURN#                                                                 PE30 257
END#                                                                    PE30 258
systemheader:(PE35)#                   Prepare for rotation search      PE35   1
####################################################################    PE35   2
#                                                                  #    PE35   3
#                                PE35                              #    PE35   4
#                                                                  #    PE35   5
####################################################################    PE35   6
SUBROUTINE PE35#                       S.r. hall nov 88                 PE35   7
include:(AACOMN)#                                                       PE35   8
include:(PECOMN)#                                                       PE35   9
#                                                                       PE35  10
INTEGER I,IS,J#                                                         PE35  11
REAL D,U,V,W#                                                           PE35  12
#                                                                       PE35  13
chardata:(PE1," WARNING  Requested translation search unnecessary $#    PE35  14
for space group P1")#                                                   PE35  15
chardata:(PE2," WARNING  Atoms ........ and ........ are too close $#   PE35  16
(     ) to allow TWIST")#                                               PE35  17
chardata:(PE3," WARNING  Flexible twist bond is longer than 2. Anstr.")#PE35  18
datastuff:#                                                             PE35  19
#                                                                       PE35  20
#   ADJUST SYMMETRY IF MIRROR PRESENT                                   PE35  21
#   ---------------------------------                                   PE35  22
#                                                                       PE35  23
IF(ROTN==yes:&RMIR==yes:)#             If mirror in input atoms         PE35  24
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  PE35  25
STOPS=STOP2#                           Remove centro symops             PE35  26
FOR(IS=MARKS;IS<STOPS;IS=IS+STEPS)#    Loop over symops                 PE35  27
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  PE35  28
U=QX(IS+5)*QX(IS+9)-QX(IS+6)*QX(IS+8)# Get 2x2 matrix                   PE35  29
V=QX(IS+6)*QX(IS+7)-QX(IS+4)*QX(IS+9)# Get 2x2 matrix                   PE35  30
W=QX(IS+4)*QX(IS+8)-QX(IS+5)*QX(IS+7)# Get 2x2 matrix                   PE35  31
U=U*QX(IS+1)+V*QX(IS+2)+W*QX(IS+3)#    Get 3x3 determinant              PE35  32
IF(U<0.) QX(IS+10)=QX(IS+10)+10.#      Flag if improper rotation        PE35  33
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  PE35  34
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  PE35  35
#                                                                       PE35  36
#   SET TRANSLATION TEST D-MINIMA                                       PE35  37
#   -----------------------------                                       PE35  38
#                                                                       PE35  39
IF(NSYM==1&TRAN==yes:)#                Test for space group p1          PE35  40
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  PE35  41
TRAN=no:#                              Switch off translation search    PE35  42
writeline:(1,PE1,NPE1,3,3)#            Print warning message            PE35  43
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  PE35  44
#                                                                       PE35  45
#   TEST TWIST-RELATED FRAGMENTS                                        PE35  46
#   ----------------------------                                        PE35  47
#                                                                       PE35  48
IF(TWIS>0)#                            Test if twist option needed      PE35  49
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  PE35  50
IA=TWIS;      J=TWIS-STEPA#            Get atoms after & before         PE35  51
U=cx:-cx:(J); V=cy:-cy:(J); W=cz:-cz:(J)# Get atomic displacements      PE35  52
D=U*U+V*V+W*W#                         Get distance squared             PE35  53
IF(D<0.01) iquit:(PE3510)#             Twist atoms too close            PE35  54
IF(D>4.) writeline:(1,PE3,NPE3,2,3)#   Warn user twist bond is long     PE35  55
IF(D-U*U<0.0001) TANG(1)=0.#           Avoid underflow                  PE35  56
ELSE#                                  Calculate twist angle 1          PE35  57
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  PE35  58
TANG(1)=ATAN2(V,W)#                    Calculate twist angle 1          PE35  59
W=V*SIN(TANG(1))+W*COS(TANG(1))#       Get new z displacement           PE35  60
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  PE35  61
TANG(2)=-ATAN2(U,W)#                   Calculate twist angle 2          PE35  62
#                                                                       PE35  63
IF(WTES==yes:)#                        Is twist test requested          PE35  64
FOR(IA=IATM(2);IA<J;IA=IA+STEPA)#      Loop over top twist atoms        PE35  65
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  PE35  66
IF(at:<1.) NEXT#                       Avoid short-distance atoms       PE35  67
FOR(I=J;I<JATM(2);I=I+STEPA)#          Loop over bot twist atoms        PE35  68
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  PE35  69
IF(at:(I)<1.) NEXT#                    Avoid short-distance atoms       PE35  70
U=(cx:-cx:(I))**2+(cy:-cy:(I))**2+(cz:-cz:(I))**2# Get distance         PE35  71
IF(U>D) NEXT#                          Skip smaller distance            PE35  72
movertoc:(an:   ,1,PE2,17,8,0)#        Insert atom name 1               PE35  73
movertoc:(an:(I),1,PE2,30,8,0)#        Insert atom name 1               PE35  74
ncodefld:(U,1,PE2,570532.,1)#          Format distance                  PE35  75
writeline:(1,PE2,NPE2,3,3)#            Print warning message            PE35  76
TWIS=no:#                              Switch off the twist             PE35  77
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  PE35  78
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  PE35  79
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  PE35  80
#                                                                       PE35  81
RETURN#                                                                 PE35  82
END#                                                                    PE35  83
systemheader:(PE40)#                   Reset rot search parameters      PE40   1
####################################################################    PE40   2
#                                                                  #    PE40   3
#                                PE40                              #    PE40   4
#                                                                  #    PE40   5
####################################################################    PE40   6
SUBROUTINE PE40#                       S.r. hall nov 88                 PE40   7
include:(AACOMN)#                                                       PE40   8
include:(PECOMN)#                                                       PE40   9
#                                                                       PE40  10
INTEGER I,J,K,N#                                                        PE40  11
REAL S,C,G(3),X,Y#                                                      PE40  12
#                                                                       PE40  13
chardata:(PE1," WARNING  Twist angle       rejected because of $#       PE40  14
   close contacts")#                                                    PE40  15
datastuff:#                                                             PE40  16
#                                                                       PE40  17
#   ROTATE FRAGMENT ABOUT TWIST BOND                                    PE40  18
#   --------------------------------                                    PE40  19
#                                                                       PE40  20
FOR(IA=IATM(2);IA<JATM(2);IA=IA+STEPA)# Loop over rotated atoms         PE40  21
DO I=1,3; QX(IA+I+8)=QX(IA+I+5)#       Reset rotation coordinates       PE40  22
#                                                                       PE40  23
IF(TWIS>0)#                            Test if twist required           PE40  24
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  PE40  25
TANG(3)=FLOAT(WMIN)*DTOR#              Start twist angle                PE40  26
N=TWIS-STEPA;       SKIP=no:#          Reset twist skip flag            PE40  27
FOR(IA=TWIS+STEPA;IA<JATM(2);IA=IA+STEPA)# Loop over bot twist atoms    PE40  28
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  PE40  29
DO I=1,3; G(I)=QX(I+IA+8)-QX(I+N+8)#   Get displ from connect atom      PE40  30
DO I=1,3#                              Loop angles about x,y,z          PE40  31
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  PE40  32
J=MOD(I,3)+1;     K=MOD(I+1,3)+1#      Cyclic axis pointers             PE40  33
S=SIN(TANG(I));   C=COS(TANG(I))#      Sin and cos twist angle          PE40  34
X=G(J);           Y=G(K)#              Get cartesian coords             PE40  35
G(J)=X*C-Y*S;     G(K)=X*S+Y*C#        Get rotated coordinates          PE40  36
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  PE40  37
DO I=2,1,-1#                           Loop angles about x,y,z          PE40  38
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  PE40  39
J=MOD(I,3)+1;     K=MOD(I+1,3)+1#      Cyclic axis pointers             PE40  40
S=SIN(TANG(I));   C=COS(TANG(I))#      Sin and cos twist angle          PE40  41
X=G(J);           Y=G(K)#              Get cartesian coords             PE40  42
G(J)=X*C+Y*S;     G(K)=-X*S+Y*C#       Get rotated coordinates          PE40  43
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  PE40  44
DO I=1,3; QX(I+IA+8)=G(I)+QX(I+N+8)#   Get twisted atom position        PE40  45
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  PE40  46
#                                                                       PE40  47
#   TEST TWISTED ATOMS FOR CLOSE CONTACTS                               PE40  48
#   -------------------------------------                               PE40  49
#                                                                       PE40  50
N=0#                                   Number of close contacts         PE40  51
FOR(IA=IATM(2);IA<TWIS;IA=IA+STEPA)#   Loop over top twist atoms        PE40  52
FOR(I=TWIS+STEPA;I<JATM(2);I=I+STEPA)# Loop over bot twist atoms        PE40  53
IF((rx:-rx:(I))**2+(ry:-ry:(I))**2+(rz:-rz:(I))**2<4.) N=N+1#           PE40  54
IF(N>0)#                               If any close contacts found      PE40  55
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  PE40  56
ncodefld:(FLOAT(WMIN),1,PE1,270502.,1)# Format current twist angle      PE40  57
ncodefld:(FLOAT(N),1,PE1,500213.,1)#   Format number of contacts        PE40  58
writeline:(1,PE1,NPE1,2,3)#            Print warning message            PE40  59
SKIP=yes:#                             Skip this twist                  PE40  60
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  PE40  61
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  PE40  62
#                                                                       PE40  63
#    APPLY CENTRE-OF-GRAVITY SHIFT                                      PE40  64
#    -----------------------------                                      PE40  65
#                                                                       PE40  66
DO I=1,3; G(I)=0.#                     Zero coordinate sums             PE40  67
FOR(I=IATM(2);I<JATM(2);I=I+STEPA)#    Loop over r/t atoms              PE40  68
DO J=1,3; G(J)=G(J)+QX(I+J+8)#         Sum atom cartn coords            PE40  69
IF(NATM(2)>0) DO I=1,3; G(I)=G(I)/NATM(2)# Get mean coordinates         PE40  70
FOR(I=IATM(2);I<JATM(2);I=I+STEPA)#    Loop over r/t atoms              PE40  71
DO J=1,3; QX(I+J+8)=QX(I+J+8)-G(J)#    Move atom cartn coords           PE40  72
##WRITE(6,'(6f10.4)') G#                                                PE40  73
##WRITE(6,'(6f10.4)') (cx:,cy:,cz:,rx:,ry:,rz:,$#                       PE40  74
##IA=IATM(2),JATM(2)-STEPA,STEPA)#                                      PE40  75
#                                                                       PE40  76
RETURN#                                                                 PE40  77
END#                                                                    PE40  78
systemheader:(PE45)#                   Prepare rotn search vectors      PE45   1
####################################################################    PE45   2
#                                                                  #    PE45   3
#                                PE45                              #    PE45   4
#                                                                  #    PE45   5
####################################################################    PE45   6
SUBROUTINE PE45#                       S.r. hall dec 88                 PE45   7
include:(AACOMN)#                                                       PE45   8
include:(PECOMN)#                                                       PE45   9
#                                                                       PE45  10
INTEGER I,J,NFOM,NTES,NV,NVEC#                                          PE45  11
REAL A,X,Y,Z,U,V,W,P,Q,D,DRES#                                          PE45  12
REAL VCNT,VTOT,VMIN,VMAX,WTMX,WTMN#                                     PE45  13
#                                                                       PE45  14
chardata:(PE0," Intra-mol. vectors used in -Rfom pretest         ")#    PE45  15
chardata:(PE1," Intra-mol. vectors within fragment(s)            ")#    PE45  16
chardata:(PE2," Intra-mol. vectors with d between x.x-x.x A       ")#   PE45  17
chardata:(PE3," Intra-mol. vectors after overlap removed         ")#    PE45  18
chardata:(PE4," Vector weight   xC      yC      zC    length")#         PE45  19
chardata:(PE5," Intra-mol. vector weight limits min/max          ")#    PE45  20
chardata:(PE6," Intra-mol. vectors min overlap separation         A")#  PE45  21
chardata:(PE7," Rotation Search for twist angle =     degrees-")#       PE45  22
chardata:(PE8," Intra-mol. vectors used to calculate Rfom        ")#    PE45  23
chardata:(PE9," Solution made equivalent if closer than           deg")#PE45  24
chardata:(PEA," Solution rejected if contact closer than     2.00 A")#  PE45  25
chardata:(PEB," Solution rejected if Rfom below cutoff            ")#   PE45  26
realdata:(FMT1,[210832.,290832.,370832.,120622.])#                      PE45  27
datastuff:#                                                             PE45  28
#                                                                       PE45  29
#    SET UP ROTATION VECTOR LIST                                        PE45  30
#    ---------------------------                                        PE45  31
#                                                                       PE45  32
DRES=MAPR*MAPR#                        Get lower resolution limit       PE45  33
VMAX=RMAX*RMAX;  VMIN=RMIN*RMIN#       Get max/min vector sqd allowed   PE45  34
VCNT=0.;         VTOT=0.#              Zero vector count                PE45  35
JV=MARKV#                              Set vector & keep list starts    PE45  36
##WRITE(6,'(10f10.4)') DRES,VMAX,VMIN#                                  PE45  37
#                                                                       PE45  38
FOR(IA=IATM(2);IA<JATM(2);IA=IA+STEPA)# Loop over r/t atoms             PE45  39
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  PE45  40
X=rx:;   Y=ry:;   Z=rz:#               Get atom coordinates             PE45  41
FOR(I=IA+STEPA;I<JATM(2);I=I+STEPA)#   Loop over r/t atoms              PE45  42
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  PE45  43
VTOT=VTOT+1.#                          Count vectors sought             PE45  44
U=rx:(I)-X; V=ry:(I)-Y; W=rz:(I)-Z#    Get displacements                PE45  45
P=aw:*aw:(I)#                          Get weighted product             PE45  46
Q=U*U+V*V+W*W#                         Get distance sqrd                PE45  47
IF(Q<VMIN\Q>VMAX) NEXT#                Skip disallowed vectors          PE45  48
VCNT=VCNT+1.#                          Count surviving vectors          PE45  49
#                                                                       PE45  50
FOR(IV=MARKV;IV<JV;IV=IV+STEPV)#       Check overlap with vector list   PE45  51
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  PE45  52
IF((vx:-U)**2+(vy:-V)**2+(vz:-W)**2<DRES)# Test if within resolution    PE45  53
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  PE45  54
##WRITE(6,'(i5,4f10.4)')IV,vx:,vy:,vz:,vw:#                             PE45  55
Q=P+vw:#                               Get total weight                 PE45  56
vx:=(vx:*vw:+U*P)/Q#                   Get new mean x                   PE45  57
vy:=(vy:*vw:+V*P)/Q#                   Get new mean y                   PE45  58
vz:=(vz:*vw:+W*P)/Q#                   Get new mean z                   PE45  59
vw:=Q#                                 Store total weight               PE45  60
##WRITE(6,'(5x,4f10.4)')vx:,vy:,vz:,vw:#                                PE45  61
BREAK#                                 Exit vector check                PE45  62
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  PE45  63
IF((vx:+U)**2+(vy:+V)**2+(vz:+W)**2<DRES)# Test if within resolution    PE45  64
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  PE45  65
##WRITE(6,'(i5,4f10.4)')IV,vx:,vy:,vz:,vw:#                             PE45  66
Q=P+vw:#                               Get total weight                 PE45  67
vx:=(vx:*vw:-U*P)/Q#                   Get new mean x                   PE45  68
vy:=(vy:*vw:-V*P)/Q#                   Get new mean y                   PE45  69
vz:=(vz:*vw:-W*P)/Q#                   Get new mean z                   PE45  70
vw:=Q#                                 Store total weight               PE45  71
##WRITE(6,'(5x,4f10.4)')vx:,vy:,vz:,vw:#                                PE45  72
BREAK#                                 Exit vector check                PE45  73
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  PE45  74
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  PE45  75
IF(IV<JV\JV==STOPV)     NEXT#          Skip if vector overlap           PE45  76
vx:(JV)=U;  vy:(JV)=V;  vz:(JV)=W#     Store new vector coords          PE45  77
vw:(JV)=P#                             Store vector weight              PE45  78
##WRITE(6,'(i5,4f10.4)')JV,vx:(JV),vy:(JV),vz:(JV),vw:(JV)#             PE45  79
JV=JV+STEPV#                           Incr the vector list pointer     PE45  80
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  PE45  81
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  PE45  82
#                                                                       PE45  83
#     CHECK NOW FOR SECONDARY OVERLAP                                   PE45  84
#     -------------------------------                                   PE45  85
#                                                                       PE45  86
WTMN=99999.;    WTMX=0.#               Set vector weight min/max        PE45  87
FOR(I=MARKV;I<JV;I=I+STEPV)#           Check overlap with vector list   PE45  88
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  PE45  89
###IF(vw:(I)<0.) NEXT#                 Skip cancelled vectors           PE45  90
X=vx:(I); Y=vy:(I); Z=vz:(I); W=vw:(I)# Get vector coords and weight    PE45  91
FOR(IV=I+STEPV;IV<JV;IV=IV+STEPV)#     Check overlap with vector list   PE45  92
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  PE45  93
###IF(vw:<0.) NEXT#                    Skip cancelled vectors           PE45  94
IF((vx:-X)**2+(vy:-Y)**2+(vz:-Z)**2<DRES)# Get displacement             PE45  95
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  PE45  96
Q=vw:+W#                               Get total weight                 PE45  97
vx:=(vx:*vw:+X*W)/Q#                   Get new mean x                   PE45  98
vy:=(vy:*vw:+Y*W)/Q#                   Get new mean y                   PE45  99
vz:=(vz:*vw:+Z*W)/Q#                   Get new mean z                   PE45 100
vw:=Q;   vw:(I)=0.#                    Cancel one of the vectors        PE45 101
BREAK#                                 Exit vector check                PE45 102
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  PE45 103
IF((vx:+X)**2+(vy:+Y)**2+(vz:+Z)**2<DRES)# Get displacement             PE45 104
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  PE45 105
Q=vw:+W#                               Get total weight                 PE45 106
vx:=(vx:*vw:-X*W)/Q#                   Get new mean x                   PE45 107
vy:=(vy:*vw:-Y*W)/Q#                   Get new mean y                   PE45 108
vz:=(vz:*vw:-Z*W)/Q#                   Get new mean z                   PE45 109
vw:=Q;   vw:(I)=0.#                    Cancel one of the vectors        PE45 110
BREAK#                                 Exit vector check                PE45 111
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  PE45 112
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  PE45 113
ifeq:(vw:(I),0.) NEXT#                 Skip if cancelled vector         PE45 114
WTMX=MAX(WTMX,vw:(I))#                 Find max vector weight           PE45 115
WTMN=MIN(WTMN,vw:(I))#                 Find min vector weight           PE45 116
D=vx:(I)**2+vy:(I)**2+vz:(I)**2#       Get vector length squared        PE45 117
vw:(I)=vw:(I)+FLOAT(10000*NINT(D*vw:(I)))# Add distance for sorting     PE45 118
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  PE45 119
#                                                                       PE45 120
#    SORT VECTORS BY DECREASING WEIGHT*DIST                             PE45 121
#    --------------------------------------                             PE45 122
#                                                                       PE45 123
FOR(I=MARKV;I<JV;I=I+STEPV)#           Check overlap with vector list   PE45 124
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 PE45 125
J=I;     W=vw:(I)#                     Set initial sort values          PE45 126
FOR(IV=I+STEPV;IV<JV;IV=IV+STEPV)#     Check overlap with vector list   PE45 127
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 PE45 128
IF(W>vw:) NEXT#                        Skip if weight smaller           PE45 129
J=IV;    W=vw:#                        Reset max weight & pointer       PE45 130
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 PE45 131
ifeq:(W,0.)  BREAK#                    Stop if zero weights             PE45 132
P=vx:(I);  vx:(I)=vx:(J);  vx:(J)=P#   Swap x coords                    PE45 133
P=vy:(I);  vy:(I)=vy:(J);  vy:(J)=P#   Swap y coords                    PE45 134
P=vz:(I);  vz:(I)=vz:(J);  vz:(J)=P#   Swap z coords                    PE45 135
vw:(J)=vw:(I);  vw:(I)=MOD(W,10000.)#  Swap weights                     PE45 136
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 PE45 137
#                                                                       PE45 138
#    SET VECTOR LIMITS FOR FOM'S                                        PE45 139
#    ---------------------------                                        PE45 140
#                                                                       PE45 141
NVEC=(I-MARKV)/STEPV#                  Vectors after overlap applied    PE45 142
NFOM=MIN(NVEC,MAX(10,NINT(RVEC*FLOAT(NVEC))))# Vectors in final fom     PE45 143
NTES=MIN(NFOM,MAX( 5,NINT(RTES*FLOAT(NFOM))))# Vectors in fom pretest   PE45 144
A=FLOAT(NFOM)/EXP(FLOAT(NFOM)/500.)#   Expected vector match number     PE45 145
RSCL=0.5*WTMN*PMIN*A/RVEC#             Calculate rfom scale             PE45 146
JV=MARKV+NFOM*STEPV#                   Set vector fom limit             PE45 147
KV=MARKV+NTES*STEPV#                   Set vector pretest limit         PE45 148
#                                                                       PE45 149
#    PRINT VECTOR LIST SUMMARY                                          PE45 150
#    -------------------------                                          PE45 151
#                                                                       PE45 152
LINRM=16#                              Ask for room on the page         PE45 153
IF(TWIS>0)#                            If twist applied                 PE45 154
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 PE45 155
ncodefld:(FLOAT(WMIN),1,PE7,380413.,1)# Format current twist angle      PE45 156
movechar:(PE7,NPE7,CHROT,2,NPE7-2,2)#  Insert underline                 PE45 157
writeline:(2,PE7,NPE7-1,2,3)#          Print rotation header            PE45 158
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 PE45 159
ELSE#                                  If twist not applied             PE45 160
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 PE45 161
movechar:(PE7,NPE7,CHROT,2,15,2)#      Insert underline                 PE45 162
writeline:(2,PE7,16,2,3)#              Print rotation header            PE45 163
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 PE45 164
ncodefld:(WTMN,1,PE5,450413.,1)#       Format maximum vector weight     PE45 165
ncodefld:(WTMX,1,PE5,500413.,1)#       Format minimum vector weight     PE45 166
writeline:(1,PE5,NPE5,3,3)#            Print total vectors in frag      PE45 167
ncodefld:(VTOT,1,PE1,500713.,1)#       Format total vectors in frag     PE45 168
writeline:(0,PE1,NPE1,3,3)#            Print total vectors in frag      PE45 169
ncodefld:(RMIN,1,PE2,380312.,1)#       Format min d vectors in frag     PE45 170
ncodefld:(RMAX,1,PE2,420312.,1)#       Format max d vectors in frag     PE45 171
ncodefld:(VCNT,1,PE2,500713.,1)#       Format reduced vectors in frag   PE45 172
writeline:(0,PE2,NPE2,3,3)#            Print total reduced in frag      PE45 173
ncodefld:(MAPR,1,PE6,500632.,1)#       Format vector resolution         PE45 174
writeline:(0,PE6,NPE6,3,3)#            Print vector resolution          PE45 175
ncodefld:(FLOAT(NVEC),1,PE3,500713.,1)# Vectors after overlap applied   PE45 176
writeline:(0,PE3,NPE3,3,3)#            Vectors after overlap applied    PE45 177
ncodefld:(FLOAT(NFOM),1,PE8,500713.,1)# Vectors used for fom            PE45 178
writeline:(0,PE8,NPE8,3,3)#            Vectors used for fom             PE45 179
ncodefld:(FLOAT(NTES),1,PE0,500713.,1)# Vectors used in fom pre-test    PE45 180
writeline:(0,PE0,NPE0,3,3)#            Vectors used in fom pre-test     PE45 181
IF(RLIS==yes:)#                        List rotation vectors            PE45 182
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14 PE45 183
writeline:(0,0,0,1,3);  LINRM=5#       Force first vector header        PE45 184
NV=MARKV+NVEC*STEPV#                   End of non-overlap vectors       PE45 185
FOR(IV=MARKV;IV<NV;IV=IV+STEPV)#       Loop over vector list            PE45 186
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 15 PE45 187
D=SQRT(vx:**2+vy:**2+vz:**2)#          Calc the vector length           PE45 188
ncodefld:(D,1,CHROT,440622.,1)#        Format vector length             PE45 189
ncodefld:(vx:,1,CHROT,FMT1,4)#         Format x,y,z,w                   PE45 190
writeline:(0,PE4,NPE4,1,3)#            Print vector data                PE45 191
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 15 PE45 192
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14 PE45 193
#                                                                       PE45 194
#   DEFINE SOLUTION ATOMIC RESOLUTION                                   PE45 195
#   ---------------------------------                                   PE45 196
#                                                                       PE45 197
D=0.#                                  Set initial displacement         PE45 198
DO I=1,NORA#                           Loop over orienting atoms        PE45 199
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 16 PE45 200
IA=IENV(I)#                            Get the atom list address        PE45 201
D=MAX(D,rx:**2+ry:**2+rz:**2)#         Get maximum displacement         PE45 202
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 16 PE45 203
SRES=D*(SIN(RRES*DTOR))**2#            D**2 resol for equiv search      PE45 204
ncodefld:(RRES,1,PE9,500613.,1)#       Format search resolution         PE45 205
writeline:(1,PE9,NPE9,3,3)#            Print search resolution          PE45 206
writeline:(0,PEA,NPEA,3,3)#            Print close contact test         PE45 207
ncodefld:(RCUT,1,PEB,500822.,1)#       Format rfom rejection limit      PE45 208
writeline:(0,PEB,NPEB,3,3)#            Print rfom rejection limit       PE45 209
#                                                                       PE45 210
RETURN#                                                                 PE45 211
END#                                                                    PE45 212
systemheader:(PE50)#                   Apply rotation search vectors    PE50   1
####################################################################    PE50   2
#                                                                  #    PE50   3
#                           PE50                                   #    PE50   4
#                                                                  #    PE50   5
####################################################################    PE50   6
SUBROUTINE PE50#                       S.r. hall jan 89                 PE50   7
include:(AACOMN)#                      S.r. hall oct 96                 PE50   8
include:(PECOMN)#                                                       PE50   9
#                                                                       PE50  10
INTEGER I,J,K,N,NCC,NPT,NFC#                                            PE50  11
INTEGER I1,I2,IR,KB,NB,SEED,NTRY,IREF#                                  PE50  12
REAL AT,X,Y,Z,U,V,W,P,Q,R,D,CUTR,SCUT,XT,YT,ZT#                         PE50  13
REAL FOM,QPHI(3),RPHI(3),RR(3),SS(3)#                                   PE50  14
REAL RS(3),RC(3)#                                                       PE50  15
REAL RM(3,3),CM(9)## BM(9)#                                             PE50  16
chardata:(PE1," Maximum Rfom found for vector set                ")#    PE50  17
chardata:(PE2," Solutions rejected due to close contact          ")#    PE50  18
chardata:(PE3," Solutions rejected when -Rfom at pretest         ")#    PE50  19
chardata:(PE4," Solutions rejected when Rfom below cutoff        ")#    PE50  20
datastuff:#                                                             PE50  21
#                                                                       PE50  22
NTRY=RTRY#                             Reset number of rotation tries   PE50  23
SEED=RNUM#                             Reset random number seed         PE50  24
CUTR=RCUT*RSCL#                        Reset rfom cutoff value          PE50  25
##WRITE(6,'(5f10.4)') CUTR,RCUT,RSCL,RRES,SRES#                         PE50  26
DO I=1,3; SS(I)=RSET(I)#               Install rotation settings        PE50  27
DO I=1,3; RR(I)=RMUL(I)/3600.#         Install rotation multipliers     PE50  28
JB=MARKB;    IREF=no:#                 Set start of the best list       PE50  29
NCC=0;  NPT=0;  NFC=0;  SCUT=0.#       Zero rejection counters          PE50  30
##WRITE(6,'(3f10.3,3i10)')INC,OFF#                                      PE50  31
#                                                                       PE50  32
#   ROTATION REFINEMENT LOOP                                            PE50  33
#   ------------------------                                            PE50  34
#                                                                       PE50  35
REPEAT#                                Loop until solutions refined     PE50  36
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  PE50  37
#                                                                       PE50  38
DO IR=1,NTRY#                          Loop over rotation tries         PE50  39
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  PE50  40
#                                                                       PE50  41
DO I=1,3#                              Loop over rotation angles        PE50  42
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  PE50  43
SEED=SEED*25173+13849#                 Generate random number seed      PE50  44
SEED=SEED-(SEED/65536)*65536#          Generate random number seed      PE50  45
RPHI(I)=SS(I)+RR(I)*FLOAT(SEED-(SEED/3600)*3600)# Phi in degrees        PE50  46
###WRITE(6,'(3i15,3f10.4)') IR,I,SEED,RPHI(I),SS(I),RR(I)##             PE50  47
RS(I)=SIN(RPHI(I)*DTOR)#               Store the sin of rotation        PE50  48
RC(I)=COS(RPHI(I)*DTOR)#               Store the cos of rotation        PE50  49
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  PE50  50
RM(1,1)=RC(2)*RC(3)#                   Matrix element 11                PE50  51
RM(2,1)=RC(2)*RS(3)#                   Matrix element 21                PE50  52
RM(3,1)=-RS(2)#                        Matrix element 31                PE50  53
RM(1,2)=RS(1)*RS(2)*RC(3)-RC(1)*RS(3)# Matrix element 12                PE50  54
RM(2,2)=RS(1)*RS(2)*RS(3)+RC(1)*RC(3)# Matrix element 22                PE50  55
RM(3,2)=RS(1)*RC(2)#                   Matrix element 32                PE50  56
RM(1,3)=RC(1)*RS(2)*RC(3)+RS(1)*RS(3)# Matrix element 13                PE50  57
RM(2,3)=RC(1)*RS(2)*RS(3)-RS(1)*RC(3)# Matrix element 23                PE50  58
RM(3,3)=RC(1)*RC(2)#                   Matrix element 33                PE50  59
#                                                                       PE50  60
N=0;                    FOM=0.#        Combined matrix pointer          PE50  61
DO I=1,3; DO J=1,3#                    Form rot/fract combined matrix   PE50  62
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  PE50  63
N=N+1;    CM(N)=0.#                    Zero combined matrix element     PE50  64
DO K=1,3; CM(N)=CM(N)+RM(K,J)*CTOF(K,I)# Store matrix product element   PE50  65
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  PE50  66
#                                                                       PE50  67
#   APPLY VECTORS TO PATTERSON MAP                                      PE50  68
#   ------------------------------                                      PE50  69
#                                                                       PE50  70
FOR(IV=MARKV;IV<JV;IV=IV+STEPV)#       Loop over vector set             PE50  71
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  PE50  72
X=CM(1)*vx:+CM(2)*vy:+CM(3)*vz:#       Get rotated fractional x         PE50  73
Y=CM(4)*vx:+CM(5)*vy:+CM(6)*vz:#       Get rotated fractional y         PE50  74
Z=CM(7)*vx:+CM(8)*vy:+CM(9)*vz:#       Get rotated fractional z         PE50  75
I=NINT(MOD(X+100.,1.)*INC(1))*OFF(1)$# Get x coord in map               PE50  76
 +NINT(MOD(Y+100.,1.)*INC(2))*OFF(2)$# Get y coord in map               PE50  77
 +NINT(MOD(Z+100.,1.)*INC(3))*OFF(3)#  Get z coord in map               PE50  78
FOM=FOM+MIN(QX(MARKM+I),vw:)*vw:#      Sum density weight product       PE50  79
##IF(IREF!=yes:)#                                                       PE50  80
##$(#                                                                   PE50  81
##L=NINT(MOD(X+100.,1.)*INC(1))# Get x coord in map                     PE50  82
##J=NINT(MOD(Y+100.,1.)*INC(2))# Get y coord in map                     PE50  83
##K=NINT(MOD(Z+100.,1.)*INC(3))# Get z coord in map                     PE50  84
##write(6,'(3i5,6f8.3,3f8.0)') l,j,k,x,y,z,rphi,qx(markm+i),vw:,fom     PE50  85
##$)#                                                                   PE50  86
IF(IV>KV&FOM<0.)    GOTO 101#          Fom pretest for -ve sum          PE50  87
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  PE50  88
##IF(IREF==yes:)WRITE(6,'(10x,6f10.2)') FOM,CUTR,RPHI                   PE50  89
IF(FOM<=CUTR)       GOTO 102#          Continue only if better soln     PE50  90
#                                                                       PE50  91
#   CHECK FOR COLLISION OF ENVELOPE ATOMS                               PE50  92
#   -------------------------------------                               PE50  93
#                                                                       PE50  94
IF(NFRG>1&IREF==no:)#                  Only in non refinement mode      PE50  95
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  PE50  96
DO I=1,NENV(2)#                        Loop over envelope atom addr     PE50  97
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  PE50  98
IA=IENV(I);    AT=at:#                 Get atom list address            PE50  99
X=CM(1)*rx:+CM(2)*ry:+CM(3)*rz:#       Get rotated fractional x         PE50 100
Y=CM(4)*rx:+CM(5)*ry:+CM(6)*rz:#       Get rotated fractional y         PE50 101
Z=CM(7)*rx:+CM(8)*ry:+CM(9)*rz:#       Get rotated fractional z         PE50 102
DO J=1,NLAT#                           Loop over lattice translations   PE50 103
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  PE50 104
XT=X+XLAT(J); YT=Y+YLAT(J); ZT=Z+ZLAT(J)# Add lattice translations      PE50 105
DO K=I+1,NENV(2)#                      Loop over envelope atom addr     PE50 106
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  PE50 107
IA=IENV(K)#                            Get atom list address            PE50 108
IF(at:<0.&AT<0.) NEXT#                 Skip if short d allowed          PE50 109
U=CM(1)*rx:+CM(2)*ry:+CM(3)*rz:#       Get rotated fractional x         PE50 110
V=CM(4)*rx:+CM(5)*ry:+CM(6)*rz:#       Get rotated fractional y         PE50 111
W=CM(7)*rx:+CM(8)*ry:+CM(9)*rz:#       Get rotated fractional z         PE50 112
P=MOD(XT-U,1.)-0.5#                    Get smallest x diff              PE50 113
Q=MOD(YT-V,1.)-0.5#                    Get smallest y diff              PE50 114
R=MOD(ZT-W,1.)-0.5#                    Get smallest z diff              PE50 115
D=P*P*G1+Q*Q*G2+R*R*G3+P*Q*G4+P*R*G5+Q*R*G6# Get distance               PE50 116
IF(D<4.)          GOTO 103#            Reject if too close              PE50 117
$)#                                    5<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  PE50 118
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  PE50 119
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  PE50 120
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  PE50 121
#                                                                       PE50 122
#    CHECK BEST SOLUTION LIST FOR EQUIVALENCE                           PE50 123
#    ----------------------------------------                           PE50 124
#                                                                       PE50 125
IF(IREF==no:)#                         In search mode only              PE50 126
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 PE50 127
NB=MARKB;      KB=0#                   Starting fom pointers            PE50 128
FOR(IB=MARKB;IB<JB;IB=IB+STEPB)#       Loop best rotation solutions     PE50 129
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 PE50 130
IF(bf:<bf:(NB)) NB=IB#                 If fom smaller than test         PE50 131
IF(bf:<0.) NEXT#                       Skip cancelled solution          PE50 132
DO I=1,3#                              Quick test of rotation angles    PE50 133
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 PE50 134
P=QX(IB+I)-RPHI(I)#                    Get diff in each angle           PE50 135
IF     (ABS(P)<RRES) QPHI(I)=RPHI(I)#  Store closest value              PE50 136
ELSE IF(360.-P<RRES) QPHI(I)=RPHI(I)+360.# Store closest value          PE50 137
ELSE IF(360.+P<RRES) QPHI(I)=RPHI(I)-360.# Store closest value          PE50 138
ELSE GOTO 50#                          No match                         PE50 139
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 PE50 140
##WRITE(6,'(I10,9f10.2)')IB,bn:,bf:,QX(IB+1),QX(IB+2),QX(IB+3),QPHI     PE50 141
bn:=bn:+1.#                            Count hit and get total fom      PE50 142
IF(bf:<FOM)#                                                            PE50 143
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 PE50 144
DO I=1,3; QX(IB+I)=QPHI(I)#            Store new angles                 PE50 145
bf:=FOM#                               Stor av form and sum fom         PE50 146
##WRITE(6,'(10x,6f10.2)') bn:, bf:,QX(IB+1),QX(IB+2),QX(IB+3)#          PE50 147
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 PE50 148
#                                                                       PE50 149
IF(KB>0)#                              Has equivalent found earlier     PE50 150
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14 PE50 151
I1=IB; I2=KB#                          Store pointers matching sets     PE50 152
IF(bf:(KB)>bf:(IB))#                   Test for largest form            PE50 153
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 15 PE50 154
I1=KB; I2=IB#                          Reverse order of pointers        PE50 155
$)#                                    5<<<<<<<<<<<<<<<<<<<<<<<<<<<< 15 PE50 156
DO I=1,3#                              Loop over angles                 PE50 157
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 16 PE50 158
P=QX(I1+I)-QX(I2+I)#                   Get diff in each angle           PE50 159
QPHI(I)=QX(I2+I)#                      Store closest value              PE50 160
IF     (360.-P<30.) QPHI(I)=QX(I2+I)+360.# Store closest value          PE50 161
ELSE IF(360.+P<30.) QPHI(I)=QX(I2+I)-360.# Store closest value          PE50 162
$)#                                    5<<<<<<<<<<<<<<<<<<<<<<<<<<<< 16 PE50 163
DO I=1,3; QX(I1+I)=QPHI(I)#            New angles                       PE50 164
bn:(I1)=bn:(KB)+bn:-1.;  bf:(I2)=-.01# Transfer to prev solution        PE50 165
KB=I1#                                 Transfer to this solution        PE50 166
##WRITE(6,'(A,I5,6f10.2)')' ****',KB,bn:(KB),bf:(KB),$#                 PE50 167
##QX(KB+1),QX(KB+2),QX(KB+3)#                                           PE50 168
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14 PE50 169
ELSE KB=IB#                            Save posn of equiv. solution     PE50 170
50 CONTINUE#                           Quick branch point               PE50 171
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 PE50 172
#                                                                       PE50 173
IF(KB>0) NEXT#                         Test if no overlap found         PE50 174
IF(JB==STOPBR)#                        Test if best list is full        PE50 175
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 17 PE50 176
IB=NB;    CUTR=MAX(CUTR,bf:)#          If list full overwrite worst     PE50 177
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 17 PE50 178
ELSE      JB=JB+STEPB#                 Incr best list stop              PE50 179
bf:=FOM;  bn:=1.#                      Store new values                 PE50 180
DO I=1,3; QX(IB+I)=RPHI(I)#            Store new angles                 PE50 181
##WRITE(6,'(A,i6,6f10.2)')' new',IB,bn:,bf:,(QX(IB+I),I=1,3)#           PE50 182
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 PE50 183
ELSE IF(FOM>bf:)#                                                       PE50 184
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 18 PE50 185
bf:=FOM#                               Save the largest form            PE50 186
DO I=1,3; QX(IB+I)=RPHI(I)#            Save the best angles             PE50 187
##WRITE(6,'(i10,6f10.2)')IB,bf:,(QX(IB+I),I=1,3)#                       PE50 188
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 18 PE50 189
NEXT#                                                                   PE50 190
#                                                                       PE50 191
101 NPT=NPT+1;                     NEXT# Reject due to rfom pretest     PE50 192
102 SCUT=MAX(SCUT,FOM); NFC=NFC+1; NEXT# Reject due to rfom cutoff      PE50 193
103 NCC=NCC+1;                     NEXT# Reject due to close contact    PE50 194
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  PE50 195
#                                                                       PE50 196
#    SUMMARISE INITAL SEARCH PARAMETERS                                 PE50 197
#    ----------------------------------                                 PE50 198
#                                                                       PE50 199
IF(IREF==no:)#                         First time only                  PE50 200
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 19 PE50 201
ncodefld:(FLOAT(NPT),1,PE3,500613.,1)# Format rejected rfom pretest     PE50 202
IF(NPT>0)writeline:(1,PE3,NPE3,3,3)#   Print rejected rfom pretest      PE50 203
ncodefld:(FLOAT(NFC),1,PE4,500613.,1)# Format rejected rfom cutoff      PE50 204
IF(NFC>0)writeline:(0,PE4,NPE4,3,3)#   Print rejected rfom cutoff       PE50 205
ncodefld:(FLOAT(NCC),1,PE2,500613.,1)# Format rejected close contact    PE50 206
IF(NCC>0)writeline:(0,PE2,NPE2,2,3)#   Print rejected close contact     PE50 207
IREF=yes:; IB=MARKB-STEPB; NTRY=NREF#  Start refinement and tries       PE50 208
DO I=1,3;    RR(I)=RRES/3600.#         Store double refinement range    PE50 209
##WRITE(6,'(9i10)')IB,JB,NB,STOPB,STOPBR#                               PE50 210
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 19 PE50 211
#                                                                       PE50 212
#    REFINEMENT OF BEST ROTATIONS                                       PE50 213
#    ----------------------------                                       PE50 214
#                                                                       PE50 215
REPEAT#                                Loop over best solutions         PE50 216
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 20 PE50 217
IB=IB+STEPB#                           Loop over the best solution      PE50 218
IF(JB==IB) GOTO 200#                                                    PE50 219
DO I=1,3;    SS(I)=QX(IB+I)-RRES*0.5#  Store best angles with offset    PE50 220
##WRITE(6,'(i10,6f10.2)')IB,bf:,(QX(IB+I),I=1,3)#                       PE50 221
IF(bf:>1.) BREAK#                      Start refinement if active       PE50 222
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 20 PE50 223
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  PE50 224
200 CONTINUE#                                                           PE50 225
IF(JB==MARKB)#                         If no solutions found            PE50 226
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 21 PE50 227
ncodefld:(SCUT/RSCL,1,PE1,500632.,1)#  Format maximum rfom found        PE50 228
writeline:(1,PE1,NPE1,3,2)#            Print maximum rfom found         PE50 229
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 21 PE50 230
#                                                                       PE50 231
#    CALC ROTATION MATRICES FOR SOLUTIONS                               PE50 232
#    ------------------------------------                               PE50 233
#                                                                       PE50 234
FOR(IB=MARKB;IB<JB;IB=IB+STEPB)#       Loop best rotation solutions     PE50 235
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 22 PE50 236
IF(bf:<0.) NEXT#                       Skip cancelled solution          PE50 237
DO I=1,3#                              Loop over rotation angles        PE50 238
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 23 PE50 239
RPHI(I)=QX(IB+I)#                      Extract the angle                PE50 240
RS(I)=SIN(RPHI(I)*DTOR)#               Store the sin of rotation        PE50 241
RC(I)=COS(RPHI(I)*DTOR)#               Store the cos of rotation        PE50 242
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 23 PE50 243
RM(1,1)=RC(2)*RC(3)#                   Matrix element 11                PE50 244
RM(2,1)=RC(2)*RS(3)#                   Matrix element 21                PE50 245
RM(3,1)=-RS(2)#                        Matrix element 31                PE50 246
RM(1,2)=RS(1)*RS(2)*RC(3)-RC(1)*RS(3)# Matrix element 12                PE50 247
RM(2,2)=RS(1)*RS(2)*RS(3)+RC(1)*RC(3)# Matrix element 22                PE50 248
RM(3,2)=RS(1)*RC(2)#                   Matrix element 32                PE50 249
RM(1,3)=RC(1)*RS(2)*RC(3)+RS(1)*RS(3)# Matrix element 13                PE50 250
RM(2,3)=RC(1)*RS(2)*RS(3)-RS(1)*RC(3)# Matrix element 23                PE50 251
RM(3,3)=RC(1)*RC(2)#                   Matrix element 33                PE50 252
N=0#                                   Combined matrix pointer          PE50 253
DO I=1,3; DO J=1,3#                    Form rot/fract combined matrix   PE50 254
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 24 PE50 255
N=N+1;    CM(N)=0.#                    Zero combined matrix element     PE50 256
DO K=1,3; CM(N)=CM(N)+RM(K,J)*CTOF(K,I)# Store matrix product element   PE50 257
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 24 PE50 258
DO I=1,9; QX(IB+I+5)=CM(I)#            Store r/f matrix                 PE50 259
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 22 PE50 260
#                                                                       PE50 261
RETURN#                                                                 PE50 262
END#                                                                    PE50 263
systemheader:(PE55)#                   Assess rotation solutions        PE55   1
####################################################################    PE55   2
#                                                                  #    PE55   3
#                                PE55                              #    PE55   4
#                                                                  #    PE55   5
####################################################################    PE55   6
SUBROUTINE PE55#                       S.r. hall nov 88                 PE55   7
include:(AACOMN)#                                                       PE55   8
include:(PECOMN)#                                                       PE55   9
#                                                                       PE55  10
INTEGER I,J,K#                                                          PE55  11
REAL F#                                                                 PE55  12
chardata:(PE1," *** No solutions found -- reduce Rfom cutoff ***")#     PE55  13
chardata:(PE4," Rsoln    Rfom     phiX     phiY     phiZ    hits ")#    PE55  14
realdata:(FMT1,[230612.,320612.,410612.,140832.,480613.])#              PE55  15
datastuff:#                                                             PE55  16
#                                                                       PE55  17
#    SORT SOLUTION LIST ACCORDING TO RFOM                               PE55  18
#    ------------------------------------                               PE55  19
#                                                                       PE55  20
writeline:(0,0,0,1,3)#                 Print spacer                     PE55  21
K=0;    LINRM=10#                      Count the solutions found        PE55  22
FOR(IB=MARKB;IB<STOPBR;IB=IB+STEPB)#   Loop best rotation solutions     PE55  23
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  PE55  24
F=bf:;      J=IB#                      Store starting sort values       PE55  25
FOR(I=IB+STEPB;I<STOPBR;I=I+STEPB)#    Loop best rotation solutions     PE55  26
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  PE55  27
IF(bf:(I)<=F) NEXT#                    Skip smaller values              PE55  28
F=bf:(I);   J=I#                       Save largest soln address        PE55  29
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  PE55  30
DO I=1,STEPB#                          Loop over list elements          PE55  31
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  PE55  32
F=QX(IB+I); QX(IB+I)=QX(J+I); QX(J+I)=F# Swap list elements             PE55  33
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  PE55  34
IF(bf:<0.001\K==RSOL) BREAK#           Exit if cancelled solution       PE55  35
bf:=bf:/RSCL;    K=K+1#                Scale fom to fragment            PE55  36
ncodefld:(FLOAT(K),1,CHROT,40213.,1)#  Format solution number           PE55  37
ncodefld:(QX,IB+1,CHROT,FMT1,3)#       Format the rotation angles       PE55  38
ncodefld:(bf:,1,CHROT,FMT1(4),2)#      Format phi1,2,3,fom,hits         PE55  39
writeline:(0,PE4,NPE4,1,3)#            Print rotation solutions         PE55  40
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  PE55  41
IF(K==0) writeline:(1,PE1,NPE1,2,3)#   Tell user to reduce fomt         PE55  42
JB=IB#                                 Reset solution list limit        PE55  43
#                                                                       PE55  44
#    ADD SOLUTIONS TO KEEP LIST & RESORT                                PE55  45
#    -----------------------------------                                PE55  46
#                                                                       PE55  47
K=JK#                                  Point add-on part of keep list   PE55  48
FOR(IB=MARKB;IB<JB;IB=IB+STEPB)#       Loop best rotation solutions     PE55  49
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  PE55  50
DO I=1,STEPK; QX(K+I)=QX(IB+I)#        Transfer fom,hits, rot matrix    PE55  51
kt:(K)=TANG(3)/DTOR;  K=K+STEPK#       Save twist & incr list pointer   PE55  52
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  PE55  53
FOR(IK=MARKK;IK<K;IK=IK+STEPB)#        Loop best combined solutions     PE55  54
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  PE55  55
F=kf:;      J=IK#                      Store starting sort values       PE55  56
FOR(I=IK+STEPK;I<K;I=I+STEPK)#         Loop best combined solutions     PE55  57
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  PE55  58
IF(kf:(I)<=F) NEXT#                    Skip smaller values              PE55  59
F=kf:(I);   J=I#                       Save largest soln address        PE55  60
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  PE55  61
ifeq:(F,0.) BREAK#                     Exit if finished                 PE55  62
DO I=1,STEPK#                          Loop over list elements          PE55  63
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  PE55  64
F=QX(IK+I); QX(IK+I)=QX(J+I); QX(J+I)=F# Swap list elements             PE55  65
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  PE55  66
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  PE55  67
IF(TWIS==no:)JK=MIN(K,MARKK+RSOL*STEPK)# Reset end of keep array        PE55  68
ELSE         JK=MIN(K,MARKK+2*RSOL*STEPK)# Reset end of keep array      PE55  69
#                                                                       PE55  70
RETURN#                                                                 PE55  71
END#                                                                    PE55  72
systemheader:(PE60)#                   Calculate transn parameters      PE60   1
####################################################################    PE60   2
#                                                                  #    PE60   3
#                                PE60                              #    PE60   4
#                                                                  #    PE60   5
####################################################################    PE60   6
SUBROUTINE PE60#                       S.r. hall jan 88                 PE60   7
include:(AACOMN)#                                                       PE60   8
include:(PECOMN)#                                                       PE60   9
#                                                                       PE60  10
INTEGER I,N#                                                            PE60  11
INTEGER NTRA,NTER#                                                      PE60  12
chardata:(PE1," Translation Search Parameters=")#                       PE60  13
chardata:(PE2," Inter-mol. vectors of fragment atoms            -")#    PE60  14
chardata:(PE3," Solution rejected if Tfom less than             -")#    PE60  15
chardata:(PE4," FAST translation search **no vectors applied** ")#      PE60  16
chardata:(PE5," Enum   h   k   l   restr.    /E/   nTrip")#             PE60  17
chardata:(PE6,' Reflections used for triples     /E/>           -')#    PE60  18
chardata:(PE7,' Triples used in transln FOM       A >           -')#    PE60  19
chardata:(PE8," Solution rejected if dmin less than             - A")#  PE60  20
chardata:(PE9," Solution rejected if t3sum less than            -")#    PE60  21
chardata:(PEA," Solution merged if shifts are within            - A")#  PE60  22
chardata:(PRC,"   none   0/180  30/210  45/225  60/240  $#              PE60  23
90/270 120/300 135/315 150/330 ")#                                      PE60  24
realdata:(FMT1,[90413.,130413.,170413.,330732.,40313.,400313.])#        PE60  25
datastuff:#                                                             PE60  26
#                                                                       PE60  27
#    SET SEARCH PARAMETERS                                              PE60  28
#    ---------------------                                              PE60  29
#                                                                       PE60  30
JC=MARKC;   N=(STOPA-MARKA)/STEPA#     Start combined fom list          PE60  31
NTRA=(NATM(2)*(NATM(2)+1)$#            Number of intra mol vectors      PE60  32
    + NATM(3)*(NATM(3)+1))/2#          Number of intra mol vectors      PE60  33
NTER=-NTRA+NSYM*(NTRA+NATM(1)*NATM(2)$# Number of inter mol vectors     PE60  34
                     +NATM(2)*NATM(3)$# Number of inter mol vectors     PE60  35
                     +NATM(3)*NATM(1))# Number of inter mol vectors     PE60  36
SIGW=0.#                               Zero sum atom weights            PE60  37
FOR(IA=MARKA;IA<STOPA;IA=IA+STEPA)#    Loop complete atom list          PE60  38
SIGW=SIGW+aw:**2#                      Sum atom weights*z squared       PE60  39
SIGW=SQRT(SIGW*FLOAT((STOP1-MARKS)/STEPS))# Sigma**1/2 for cell         PE60  40
SCAL=2.*SIGW**2*ABS(PMIN)#             Scale for tfom patt sum          PE60  41
#                                                                       PE60  42
#    PRINT SEARCH PARAMETERS                                            PE60  43
#    -----------------------                                            PE60  44
#                                                                       PE60  45
movechar:(PE1,NPE1,CHROT,2,NPE1-2,2)#  Insert header underline          PE60  46
writeline:(2,PE1,NPE1-1,2,3)#          Print transln header             PE60  47
IF(NSYM>1)#                            For non-p1 & vector search req   PE60  48
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  PE60  49
ncodefld:(FLOAT(NTER),1,PE2,500813.,1)# Format total intermol vectors   PE60  50
writeline:(1,PE2,NPE2,3,3)#            Format total inter molvectors    PE60  51
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  PE60  52
ELSE writeline:(0,PE4,NPE4,3,3)#       Print fast transln search mess   PE60  53
#                                                                       PE60  54
#    PRINT ELIST AND TRIPLET INFORMATION                                PE60  55
#    -----------------------------------                                PE60  56
#                                                                       PE60  57
IF(NTPR==yes:)#                        If triplets to be used           PE60  58
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  PE60  59
ncodefld:(EMIN,1,PE6,440532.,1)#       Format e minimum                 PE60  60
ncodefld:(FLOAT(NUME),1,PE6,500413.,1)# Format number of e's            PE60  61
writeline:(0,PE6,NPE6,3,3)#            Print e parameters               PE60  62
ncodefld:(AMIN,1,PE7,440532.,1)#       Format a minimum                 PE60  63
ncodefld:(FLOAT(NUMT),1,PE7,500413.,1)# Format number of triples        PE60  64
writeline:(0,PE7,NPE7,3,3)#            Print triple parameters          PE60  65
IF(TLIS==yes:)#                        Test if elist requested          PE60  66
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  PE60  67
writeline:(1,0,0,1,3);  LINRM=6#       Ensure sufficient room           PE60  68
FOR(IE=MARKE;IE<STOPE;IE=IE+STEPE)#    Loop over elist                  PE60  69
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  PE60  70
gethkl:(IE+1,BUFOT)#                   Get hkl from packed indices      PE60  71
BUFOT(4)=ee:#                          Store e value                    PE60  72
BUFOT(5)=FLOAT((IE-MARKE)/STEPE+1)#    Get elist number                 PE60  73
BUFOT(6)=es:#                          Number of connected triples      PE60  74
I=NINT(er:)/100#                       Get phase restriction code       PE60  75
movechar:(PRC,I*8-7,CHROT,19,8,0)#     Insert phase restr code          PE60  76
ncodefld:(BUFOT,1,CHROT,FMT1,6)#       Format elist data                PE60  77
writeline:(0,PE5,NPE5,1,3)#            Print this line                  PE60  78
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  PE60  79
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  PE60  80
SUME1=0.;     SUME2=0.#                Zero e and e**2 sums             PE60  81
FOR(IE=MARKE;IE<STOPE;IE=IE+STEPE)#    Loop over e-list                 PE60  82
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  PE60  83
ee:=ee:*SQRT(AMOD(er:,100.))#          Store e*epsilon**1/2             PE60  84
SUME1=SUME1+ee:#                       Sum e's                          PE60  85
SUME2=SUME2+ee:**2#                    Sum e**2                         PE60  86
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  PE60  87
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  PE60  88
#                                                                       PE60  89
#   PRINT REJECTION CRITERIA                                            PE60  90
#   ------------------------                                            PE60  91
#                                                                       PE60  92
ncodefld:(TRES,1,PEA,500622.,1)#       Format test dmin                 PE60  93
writeline:(0,PEA,NPEA,3,3)#            Print test dmin                  PE60  94
ncodefld:(TMIN,1,PE8,500622.,1)#       Format test dmin                 PE60  95
writeline:(0,PE8,NPE8,3,3)#            Print test dmin                  PE60  96
ncodefld:(TLIM,1,PE9,500622.,1)#       Format test t3sum limit          PE60  97
IF(NTPR==yes:)writeline:(0,PE9,NPE9,3,3)# Print test t3sum limit        PE60  98
ncodefld:(TCUT,1,PE3,500622.,1)#       Format test tfom limit           PE60  99
writeline:(0,PE3,NPE3,3,3)#            Print test tfom limit            PE60 100
TMIN=TMIN*TMIN#                        Convert to d**2 minimum          PE60 101
IF(ROTN==no:) JK=MARKK+1#              Force fixed rotation             PE60 102
#                                                                       PE60 103
RETURN#                                                                 PE60 104
END#                                                                    PE60 105
systemheader:(PE65)#                   Top of translation search        PE65   1
####################################################################    PE65   2
#                                                                  #    PE65   3
#                                PE65                              #    PE65   4
#                                                                  #    PE65   5
####################################################################    PE65   6
SUBROUTINE PE65#                       S.r. hall jan 88                 PE65   7
include:(AACOMN)#                                                       PE65   8
include:(PECOMN)#                                                       PE65   9
#                                                                       PE65  10
INTEGER I,IS,J,K,N#                                                     PE65  11
REAL H(3),SS,S1,A,B,C1,Q,T,X,Y,Z#                                       PE65  12
REAL CM(9),G(3),S,C#                                                    PE65  13
REAL SUMR,SUMT#                                                         PE65  14
chardata:(PE1," Translation Search with Rotation Rfom = x.xxx-")#       PE65  15
chardata:(PE3,' Rotation angles about X,Y,Z                     -')#    PE65  16
chardata:(PE5,' Twist torsion angle applied                     -')#    PE65  17
chardata:(PE6,' Mean Ec/Eo for /E/s applied                     -')#    PE65  18
chardata:(PE7,' Selected triplets applied                       -')#    PE65  19
realdata:(FMT2,[340612.,420612.,500612.])#                              PE65  20
datastuff:#                                                             PE65  21
#                                                                       PE65  22
#    GET FRACTIONAL COORD OF ROTATION SOLUTION                          PE65  23
#    -----------------------------------------                          PE65  24
#                                                                       PE65  25
IF(ROTN==yes:)#                        If rotation has been applied     PE65  26
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  PE65  27
FOR(IA=IATM(2);IA<JATM(2);IA=IA+STEPA)# Loop over rotated atoms         PE65  28
DO I=1,3; QX(IA+I+8)=QX(IA+I+5)#       Reset rotation coordinates       PE65  29
RFOM=kf:;     TANG(3)=kt:*DTOR#        Get the rot fom & twist angle    PE65  30
#                                                                       PE65  31
IF(TWIS>0)#                            Test if twist required           PE65  32
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  PE65  33
N=TWIS-STEPA#                          Reset twist skip flag            PE65  34
FOR(IA=TWIS+STEPA;IA<JATM(2);IA=IA+STEPA)# Loop over bot twist atoms    PE65  35
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  PE65  36
DO I=1,3; G(I)=QX(I+IA+8)-QX(I+N+8)#   Get displ from connect atom      PE65  37
DO I=1,3#                              Loop angles about x,y,z          PE65  38
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  PE65  39
J=MOD(I,3)+1;     K=MOD(I+1,3)+1#      Cyclic axis pointers             PE65  40
S=SIN(TANG(I));   C=COS(TANG(I))#      Sin and cos twist angle          PE65  41
X=G(J);           Y=G(K)#              Get cartesian coords             PE65  42
G(J)=X*C-Y*S;     G(K)=X*S+Y*C#        Get rotated coordinates          PE65  43
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  PE65  44
DO I=2,1,-1#                           Loop angles about x,y,z          PE65  45
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  PE65  46
J=MOD(I,3)+1;     K=MOD(I+1,3)+1#      Cyclic axis pointers             PE65  47
S=SIN(TANG(I));   C=COS(TANG(I))#      Sin and cos twist angle          PE65  48
X=G(J);           Y=G(K)#              Get cartesian coords             PE65  49
G(J)=X*C+Y*S;     G(K)=-X*S+Y*C#       Get rotated coordinates          PE65  50
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  PE65  51
DO I=1,3; QX(I+IA+8)=G(I)+QX(I+N+8)#   Get twisted atom position        PE65  52
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  PE65  53
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  PE65  54
#                                                                       PE65  55
DO I=1,9; CM(I)=QX(IK+I+5)#            Extract solution rot/fract ma    PE65  56
FOR(IA=IATM(2);IA<JATM(2);IA=IA+STEPA)# Loop over rot/tran atoms        PE65  57
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  PE65  58
ax:=rx:*CM(1)+ry:*CM(2)+rz:*CM(3)#     Get fractional x                 PE65  59
ay:=rx:*CM(4)+ry:*CM(5)+rz:*CM(6)#     Get fractional y                 PE65  60
az:=rx:*CM(7)+ry:*CM(8)+rz:*CM(9)#     Get fractional z                 PE65  61
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  PE65  62
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  PE65  63
#                                                                       PE65  64
#    CALC E'S FOR FRAGMENT IN THIS ORIENTATION                          PE65  65
#    -----------------------------------------                          PE65  66
#                                                                       PE65  67
IF(NTPR==yes:)#                        If triplets to be used           PE65  68
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  PE65  69
SUMR=0.;   IP=MARKP#                   Zero e(calc)/e(obs) ratio        PE65  70
A=FLOAT(ICEN);   B=2.-A#               Set centrosymmetric terms        PE65  71
DO I=IP+1,STOPP; QX(I)=0.#             Zero phase list                  PE65  72
FOR (IE=MARKE;IE<STOPE;IE=IE+STEPE)#   Loop over e list                 PE65  73
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  PE65  74
gethkl:(IE+1,H)#                       Unpack hkl of e                  PE65  75
SS=0.;   S1=0.;   C1=0.#               Zero triplet sums                PE65  76
FOR(IS=MARKS;IS<STOP1;IS=IS+STEPS)#    Loop over symops                 PE65  77
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  PE65  78
X=H(1)*QX(IS+1 )+H(2)*QX(IS+2 )+H(3)*QX(IS+3 )# Get transformed h       PE65  79
Y=H(1)*QX(IS+4 )+H(2)*QX(IS+5 )+H(3)*QX(IS+6 )# Get transformed k       PE65  80
Z=H(1)*QX(IS+7 )+H(2)*QX(IS+8 )+H(3)*QX(IS+9 )# Get transformed l       PE65  81
T=H(1)*QX(IS+10)+H(2)*QX(IS+11)+H(3)*QX(IS+12)# Get phase shift         PE65  82
I=IP+3;  N=1;  ph:=X;  pk:=Y;  pl:=Z#  Set fixed atom flag              PE65  83
FOR(IA=MARKA;IA<STOPA;IA=IA+STEPA)#    Loop over complete atom list     PE65  84
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 PE65  85
IF(IA==IATM(2)) N=2#                   Find start of rot/tran atoms     PE65  86
IF(IA==IATM(3)) N=3#                   Find start of tran only atoms    PE65  87
Q=(X*ax:+Y*ay:+Z*az:+T)*6.2831853#     Get phase contribution           PE65  88
QX(I+N  )=QX(I+N  )+aw:*SIN(Q)#        Sum e*sine phase                 PE65  89
QX(I+N+3)=QX(I+N+3)+aw:*COS(Q)#        Sum e*cosine phase               PE65  90
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 PE65  91
IP=IP+STEPP#                           Incr phase list                  PE65  92
S1=S1+QX(I+1);   C1=C1+QX(I+4)#        Sum e for frag 1                 PE65  93
SS=SS+QX(I+2)**2+QX(I+3)**2+QX(I+5)**2+QX(I+6)**2# Sum other frags      PE65  94
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  PE65  95
SS=SS+A*S1**2+B*C1**2#                 Add frag 1 contribution          PE65  96
er:=SQRT(SS)/(SIGW*ee:)#               Store e(calc)/e(obs)             PE65  97
SUMR=SUMR+er:#                         Sum e(calc)/e(obs)               PE65  98
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  PE65  99
SUMR=SUMR/FLOAT(NUME)#                 Calc mean e(calc)/e(obs)         PE65 100
#                                                                       PE65 101
#   SELECT TRIPLETS FOR USE IN TRANSLATION                              PE65 102
#   --------------------------------------                              PE65 103
#                                                                       PE65 104
SUME3=0.;          SUMT=0.#            Zero sum e and triplet count     PE65 105
S=0.1*SUMR**3;     Q=0.4*FLOAT(NUMT)#  Get e(calc)/e(obs)**3            PE65 106
FOR(IT=MARKT;IT<STOPT;IT=IT+STEPT) t1:=ABS(t1:)# Triplets reset         PE65 107
FOR(T=15.*S;T>0.;T=T-S)#               Loop over decreasing threshold   PE65 108
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 PE65 109
FOR(IT=MARKT;IT<STOPT;IT=IT+STEPT)#    Loop over triplets               PE65 110
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 PE65 111
IF(t1:<0.) NEXT#                       Triplet already selected         PE65 112
I=NINT(t1:); J=NINT(t2:); K=NINT(t3:)# Get e-list addresses             PE65 113
IF(er:(I)*er:(J)*er:(K)<T) NEXT#       Skip if less than threshold      PE65 114
t1:=-t1:;      SUMT=SUMT+1.#           Flag and count selected triple   PE65 115
SUME3=SUME3+ee:(I)*ee:(J)*ee:(K)#      Sum e1*e2*e3                     PE65 116
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 PE65 117
IF(SUMT>Q) BREAK#                      Exit if triplet limit reached    PE65 118
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 PE65 119
FOR(IT=MARKT;IT<STOPT;IT=IT+STEPT) t1:=-t1:# -ve triplets not used      PE65 120
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  PE65 121
#                                                                       PE65 122
#    PRINT CURRENT ROTATION DATA                                        PE65 123
#    ---------------------------                                        PE65 124
#                                                                       PE65 125
IF(ROTN==yes:)#                        If rotation applied              PE65 126
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 PE65 127
LINRM=15#                              Ensure sufficient room on page   PE65 128
ncodefld:(RFOM,1,PE1,460632.,1)#       Format the rot fom               PE65 129
movechar:(PE1,NPE1,CHROT,2,NPE1-2,2)#  Insert header underline          PE65 130
writeline:(2,PE1,NPE1-1,2,3)#          Print the rotation header        PE65 131
ncodefld:(k1:,1,PE3,FMT2,3)#           Format the rotation angles       PE65 132
writeline:(1,PE3,NPE3,3,3)#            Print the rotation angles        PE65 133
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 PE65 134
IF(TWIS>0)#                            If twist applied                 PE65 135
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14 PE65 136
BUFOT(1)=TANG(3)/DTOR#                 Twist angle in degrees           PE65 137
ncodefld:(BUFOT,1,PE5,500812.,1)#      Format twist angle               PE65 138
writeline:(0,PE5,NPE5,3,3)#            Print twist angle                PE65 139
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14 PE65 140
IF(NTPR==yes:)#                        If triplets to be applied        PE65 141
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 15 PE65 142
ncodefld:(SUMR,1,PE6,500832.,1)#       Format mean e(calc)/e(obs)       PE65 143
writeline:(0,PE6,NPE6,3,3)#            Print mean e(calc)/e(obs)        PE65 144
ncodefld:(SUMT,1,PE7,500613.,1)#       Number of triplets applied       PE65 145
writeline:(0,PE7,NPE7,3,3)#            Print triplets applied           PE65 146
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 15 PE65 147
#                                                                       PE65 148
RETURN#                                                                 PE65 149
END#                                                                    PE65 150
systemheader:(PE70)#                   Translation search loop          PE70   1
####################################################################    PE70   2
#                                                                  #    PE70   3
#                                PE70                              #    PE70   4
#                                                                  #    PE70   5
####################################################################    PE70   6
SUBROUTINE PE70#                       S.r. hall jan 88                 PE70   7
include:(AACOMN)#                                                       PE70   8
include:(PECOMN)#                                                       PE70   9
#                                                                       PE70  10
INTEGER I,I1,I2,II,IREF,IS,J,JT,K,KB,L,LIM,LT,M,MT,NB,N,NDM#            PE70  11
INTEGER NFC,NF1,NF2,NT,NTS,MSYM,SEED,NTRY,MARKSS#                       PE70  12
REAL TX(9),TS(6),UU(6),VV(6),ST(6),AT,SCUT,WW,X,Y,Z#                    PE70  13
REAL AW,C,D,CQ,E,ECAL,F,G,P,Q,R,S,SQ,SMIN,SUMC,SUMS,T1,T2,U,V,W#        PE70  14
REAL CFOM,FOM,TPR,RE1,RE2,QQ(6),RR(6)#                                  PE70  15
REAL SUM1,SUM2,SUM3,TP,TT,TL,TR#                                        PE70  16
realdata:(S1,[+1.,-1.,+1.,-1.])#                                        PE70  17
realdata:(S2,[+1.,+1.,-1.,-1.])#                                        PE70  18
realdata:(SS,[+1.,-1.,-1.,+1.])#                                        PE70  19
realdata:(PI,6.2831853)#                                                PE70  20
chardata:(PE2," Solutions rejected due to close contact          ")#    PE70  21
chardata:(PE3," Solutions rejected when -Tfom at pretest 1       ")#    PE70  22
chardata:(PE4," Solutions rejected when t3sum below cutoff       ")#    PE70  23
chardata:(PE5," Solutions rejected when -Tfom at pretest 2       ")#    PE70  24
chardata:(PE6," Solutions rejected when Tfom below cutoff        ")#    PE70  25
chardata:(PE7," Largest Tfom for rejected values                 ")#    PE70  26
chardata:(PE8," Cutoff  Tfom for accepted values                 ")#    PE70  27
datastuff:#                                                             PE70  28
#                                                                       PE70  29
SCUT=0.#                               Maximum tfom rejected            PE70  30
NTRY=TTRY#                             Reset number of transln tries    PE70  31
SEED=TNUM#                             Reset random number seed         PE70  32
IF(NATM(2)>0) LT=1#                    There are rotn/tran atoms        PE70  33
ELSE          LT=4#                    Only translation atoms           PE70  34
IF(NATM(3)>0) MT=6#                    There are six transln shifts     PE70  35
ELSE          MT=3#                    Three if rot/tran atoms          PE70  36
IF(ICEN==1)   TP=SIGW#                 Prob factor noncentrosymmetric   PE70  37
ELSE          TP=SIGW/1.414#           Reduce if centrosymmetric        PE70  38
IF(NSYM==1)   TT=0.#                   Threshold for p1                 PE70  39
ELSE          TT=1.0#                  Threshold for other symmetries   PE70  40
DO I=1,MT;    QQ(I)=TSET(I)#           Reset transl starting values     PE70  41
DO I=1,MT;    RR(I)=(TMUL(I)-TSET(I))/1000.# Reset transl multipliers   PE70  42
DO I=1,MT;    UU(I)=TMUL(I)/2.#        Reset transl multipliers         PE70  43
DO I=1,MT;    VV(I)=TMUL(I)/2.+10.#    Reset transl multipliers         PE70  44
JB=MARKB;     MARKSS=MARKS+STEPS#      Start fom best list end pointe   PE70  45
MSYM=(STOP1-MARKS)/STEPS#              Reduced symop set for phases     PE70  46
TPR=0.;   RE1=0.;   RE2=0.#            Set values to no tprsum          PE70  47
NDM=0; NF1=0; NF2=0; NTS=0; NFC=0#     Zero rejected solution counts    PE70  48
DO I=1,6; TS(I)=0.#                    Zero initial translations        PE70  49
IF(ROTN==no:) RFOM=1.#                 Set rfom as unity if no rotn     PE70  50
LIM=1;  IREF=no:#                      Set for search mode              PE70  51
##WRITE(6,'(3f10.2,10i5)') TRES,TCUT,SCAL,JATM#                         PE70  52
##WRITE(6,'(9f10.4)') STEP#                                             PE70  53
#                                                                       PE70  54
#                                                                       PE70  55
#    TOP OF TRANSLATION SEARCH                                          PE70  56
#    =========================                                          PE70  57
#                                                                       PE70  58
REPEAT#                                Loop possible refinements        PE70  59
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  PE70  60
#                                                                       PE70  61
DO NT=1,NTRY#                          Loop over translation searches   PE70  62
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  PE70  63
#                                                                       PE70  64
IF(NT>LIM)#                            Except for the first try         PE70  65
DO I=LT,MT#                            Loop axes of translation frags   PE70  66
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  PE70  67
SEED=SEED*25173+13849#                 Generate random number seed      PE70  68
SEED=SEED-(SEED/65536)*65536#          Generate random number seed      PE70  69
TS(I)=QQ(I)+RR(I)*FLOAT(SEED-(SEED/1000)*1000)# Shift in 1000*cell      PE70  70
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  PE70  71
#                                                                       PE70  72
#   TEST DISTANCES AND PATTERSON WITH ENVELOPE ATOMS                    PE70  73
#   ------------------------------------------------                    PE70  74
#                                                                       PE70  75
IF(IREF==no:)#                         In search mode only              PE70  76
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  PE70  77
FOM=0.#                                Set for min distance calc        PE70  78
DO N=1,NENV(3)#                        Loop over envelope atoms         PE70  79
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  PE70  80
IA=IENV(N);    AT=at:;   WW=aw:#       Extract atom list pointer        PE70  81
IF(IA<IATM(3)) J=0#                    Set for rot/tran atom            PE70  82
ELSE           J=3#                    Set for tran only atom           PE70  83
X=ax:+TS(J+1); Y=ay:+TS(J+2); Z=az:+TS(J+3)# Apply shifts x,y,z         PE70  84
DO M=N,NENV(3)#                        Loop over envelope atoms         PE70  85
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  PE70  86
IA=IENV(M)#                            Extract atom list pointer        PE70  87
IF(AT<0.&at:<0.) NEXT#                 Skip short distance allowed      PE70  88
IF(IA<IATM(3)) K=0#                    Set for rot/tran atom            PE70  89
ELSE           K=3#                    Set for tran only atom           PE70  90
U=ax:+TS(K+1); V=ay:+TS(K+2); W=az:+TS(K+3)# Apply shifts x,y,z         PE70  91
IF(J==K) IS=MARKSS#                    Skip identity if same frag       PE70  92
ELSE     IS=MARKS#                     All symops if diff frag          PE70  93
AW=aw:*WW#                             Set vector weight                PE70  94
FOR(;IS<STOPS;IS=IS+STEPS)#            Loop over symmetry operators     PE70  95
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  PE70  96
E=U*QX(IS+1)+V*QX(IS+4)+W*QX(IS+7)+QX(IS+10)# Get symoped x             PE70  97
F=U*QX(IS+2)+V*QX(IS+5)+W*QX(IS+8)+QX(IS+11)# Get symoped y             PE70  98
G=U*QX(IS+3)+V*QX(IS+6)+W*QX(IS+9)+QX(IS+12)# Get symoped z             PE70  99
P=MOD(99.5+E-X,1.)-0.5#                Get reduced x                    PE70 100
Q=MOD(99.5+F-Y,1.)-0.5#                Get reduced y                    PE70 101
R=MOD(99.5+G-Z,1.)-0.5#                Get reduced z                    PE70 102
D=P*P*G1+Q*Q*G2+R*R*G3+P*Q*G4+P*R*G5+Q*R*G6# Get distance               PE70 103
IF(D<TMIN) GOTO 101#                   Exit if distance too small       PE70 104
I=NINT(MOD(P+100.,1.)*INC(1))*OFF(1)$# Get x coord in patterson         PE70 105
 +NINT(MOD(Q+100.,1.)*INC(2))*OFF(2)$# Get y coord in patterson         PE70 106
 +NINT(MOD(R+100.,1.)*INC(3))*OFF(3)#  Get z coord in patterson         PE70 107
FOM=FOM+MIN(QX(MARKM+I),AW)*AW#        Sum weighted patt density        PE70 108
$)#                                    5<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  PE70 109
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  PE70 110
IF(FOM<0.) GOTO 102#                   Primary fom test                 PE70 111
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  PE70 112
#                                                                       PE70 113
#   REFINE EACH COORDINATE SHIFT USING TRIPLET SUM                      PE70 114
#   ----------------------------------------------                      PE70 115
#                                                                       PE70 116
IF(NTPR==yes:)#                        Is triple sum refinement used    PE70 117
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  PE70 118
TL=TLIM;    TR=TT#                     Set triple sum limit & incr      PE70 119
DO JT=1,3#                             Loop over tprsum iterations      PE70 120
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  PE70 121
DO II=1,MT#                            Refine each coord x,y,z          PE70 122
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 PE70 123
IF(STEP(II)>9.)  NEXT#                 Skip polar axes                  PE70 124
T1=-999999.;   T2=T1#                  Set initial values               PE70 125
REPEAT#                                Refinement loop of each coord    PE70 126
$(#                                    6>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 PE70 127
IP=MARKP-9#                            Start phase list pointer         PE70 128
FOR(IE=MARKE;IE<STOPE;IE=IE+STEPE)#    Loop over e list                 PE70 129
$(#                                    7>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 PE70 130
SUMS=0.;    SUMC=0.#                   Zero sine/cos sums               PE70 131
DO I=1,MSYM#                           Loop over symmetry               PE70 132
$(#                                    8>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 PE70 133
IP=IP+9#                               Incr e/symm list                 PE70 134
Q=PI*(ph:*TS(1)+pk:*TS(2)+pl:*TS(3))#  Calculate delta phase            PE70 135
SQ=SIN(Q);   CQ=COS(Q)#                Calc sin/cos delta phase         PE70 136
SUMC=SUMC+c1:+CQ*c2:-SQ*s2:#           Sum cosines frags 1 & 2          PE70 137
IF(ICEN==1)SUMS=SUMS+s1:+SQ*c2:+CQ*s2:# Sum sines frags 1 & 2 noncen    PE70 138
IF(MT==3) NEXT#                        For tran only atoms              PE70 139
Q=PI*(ph:*TS(4)+pk:*TS(5)+pl:*TS(6))#  Calculate delta phase            PE70 140
SQ=SIN(Q);   CQ=COS(Q)#                Calc sin/cos delta phase frag    PE70 141
SUMC=SUMC+CQ*c3:-SQ*s3:#               Sum cosines frag 3               PE70 142
IF(ICEN==1)SUMS=SUMS+SQ*c3:+CQ*s3:#    Sum sines frag 3 noncentro       PE70 143
$)#                                    8<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 PE70 144
ECAL=SQRT(SUMS**2+SUMC**2)+.001#       Estimate e(calc)                 PE70 145
et:=ECAL#                              Store /e(calc)/ summation        PE70 146
ec:=ee:*SUMC/ECAL#                     Store /e/ cos phase              PE70 147
es:=ee:*SUMS/ECAL#                     Store /e/ sin phase              PE70 148
$)#                                    7<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 PE70 149
#                                                                       PE70 150
TPR=0.#                                Zero triplet cosine sum          PE70 151
FOR(IT=MARKT;IT<STOPT;IT=IT+STEPT)#    Loop over triplet list           PE70 152
$(#                                    7>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14 PE70 153
IF(t1:<0.) NEXT#                       Skip unused triplets             PE70 154
I=NINT(t1:); J=NINT(t2:); K=NINT(t3:)# Get e-list addresses             PE70 155
L=NINT(ti:)#                           Get sign index                   PE70 156
IF(ICEN==0)#                           If centrosymmetric               PE70 157
$(#                                    8>>>>>>>>>>>>>>>>>>>>>>>>>>>> 15 PE70 158
TPR=TPR+ec:(I)*ec:(J)*ec:(K)*tc:#      Sum centrosymmetric phases       PE70 159
NEXT#                                  Skip to next triplet             PE70 160
$)#                                    8<<<<<<<<<<<<<<<<<<<<<<<<<<<< 15 PE70 161
C=ec:(J)*ec:(K)      -es:(J)*es:(K)*SS(L)# E2e3 cos(p2+p3) noncent      PE70 162
S=es:(J)*ec:(K)*S1(L)+ec:(J)*es:(K)*S2(L)# E2e3 sin(p2+p3) noncent      PE70 163
TPR=TPR+ec:(I)*(C*tc:-S*ts:)$#         Sum of e1e2e3 cos(p1+p2+p3+d)    PE70 164
       -es:(I)*(S*tc:+C*ts:)#          Sum of e1e2e3 cos(p1+p2+p3+d)    PE70 165
$)#                                    7<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14 PE70 166
TPR=TPR/SUME3#                         Normalize trpsum                 PE70 167
IF(TPR>T1)#                            Test for largest tprsum          PE70 168
$(#                                    7>>>>>>>>>>>>>>>>>>>>>>>>>>>> 16 PE70 169
T2=T1;     T1=TPR#                     Save best tprsum so far          PE70 170
TS(II)=TS(II)+TR*STEP(II)#             Get new shift                    PE70 171
$)#                                    7<<<<<<<<<<<<<<<<<<<<<<<<<<<< 16 PE70 172
ELSE IF(T2<-999998.)#                  Has second tpr threshold been    PE70 173
$(#                                    7>>>>>>>>>>>>>>>>>>>>>>>>>>>> 17 PE70 174
T2=TPR;    TR=-TR#                     Save tprsum and reverse shift    PE70 175
TS(II)=TS(II)+2.*TR*STEP(II)#          Reverse the shift                PE70 176
$)#                                    7<<<<<<<<<<<<<<<<<<<<<<<<<<<< 17 PE70 177
ELSE BREAK#                            Exit refinement loop             PE70 178
$)#                                    6<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 PE70 179
TS(II)=TS(II)+TR*STEP(II)*((TPR-T2)$#  Calculate shift                  PE70 180
          /(2.*(2.001*T1-TPR-T2))-1.)# Calculate shift                  PE70 181
$)#                                    5<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 PE70 182
IF(T1<TL) GOTO 103#                    Reject if max tpr too small      PE70 183
TR=TR*0.333#                           Reduce refinement shift factor   PE70 184
TL=TL+0.2#                             Increase tprsum threshold        PE70 185
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  PE70 186
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  PE70 187
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  PE70 188
#                                                                       PE70 189
#   CHECK PATTERSON PEAK OF EACH INTERATOMIC VECTOR                     PE70 190
#   -----------------------------------------------                     PE70 191
#                                                                       PE70 192
TX(1)=0.;    TX(4)=0.;    TX(7)=0.#    Reset x,y,z for frag 1           PE70 193
TX(2)=TS(1); TX(5)=TS(2); TX(8)=TS(3)# Reset x,y,z for frag 2           PE70 194
TX(3)=TS(4); TX(6)=TS(5); TX(9)=TS(6)# Reset x,y,z for frag 3           PE70 195
N=1;    SMIN=999999.;    FOM=0.#       Start the frag type pointer      PE70 196
FOR(IA=MARKA;IA<STOPA;IA=IA+STEPA)#    Loop over complete atom list     PE70 197
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 18 PE70 198
IF     (IA<JATM(1)) N=1#               Fixed atoms                      PE70 199
ELSE IF(IA<JATM(2)) N=2#               Rotn/tran atoms                  PE70 200
ELSE                N=3#               Tran only atoms                  PE70 201
X=ax:+TX(N); Y=ay:+TX(N+3); Z=az:+TX(N+6)# Add the tran shifts          PE70 202
FOR(J=IA;J<STOPA;J=J+STEPA)#           Loop over complete atom list     PE70 203
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 19 PE70 204
IF     (J<JATM(1)) M=1#                Fixed atoms                      PE70 205
ELSE IF(J<JATM(2)) M=2#                Rotn/tran atoms                  PE70 206
ELSE               M=3#                Tran only atoms                  PE70 207
U=ax:(J)+TX(M); V=ay:(J)+TX(M+3); W=az:(J)+TX(M+6)# Add tran shifts     PE70 208
AW=aw:*aw:(J)#                         Set vector weight                PE70 209
IF(M==N) IS=MARKS+STEPS#               Skip identity if same frag       PE70 210
ELSE     IS=MARKS#                     All symops if diff frag          PE70 211
FOR(;IS<STOPS;IS=IS+STEPS)#            Loop over symmetry operators     PE70 212
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 20 PE70 213
E=U*QX(IS+1)+V*QX(IS+4)+W*QX(IS+7)+QX(IS+10)# Get symoped x             PE70 214
F=U*QX(IS+2)+V*QX(IS+5)+W*QX(IS+8)+QX(IS+11)# Get symoped y             PE70 215
G=U*QX(IS+3)+V*QX(IS+6)+W*QX(IS+9)+QX(IS+12)# Get symoped z             PE70 216
P=MOD(99.5+E-X,1.)-0.5#                Get reduced x                    PE70 217
Q=MOD(99.5+F-Y,1.)-0.5#                Get reduced y                    PE70 218
R=MOD(99.5+G-Z,1.)-0.5#                Get reduced z                    PE70 219
D=P*P*G1+Q*Q*G2+R*R*G3+P*Q*G4+P*R*G5+Q*R*G6# Get distance               PE70 220
IF(at:>0.&at:(J)>0.&D<TMIN) GOTO 101#  Exit if distance too small       PE70 221
SMIN=MIN(SMIN,D)#                      Save smallest distance           PE70 222
#                                                                       PE70 223
I=NINT(MOD(P+100.,1.)*INC(1))*OFF(1)$# Get x coord in patterson         PE70 224
 +NINT(MOD(Q+100.,1.)*INC(2))*OFF(2)$# Get y coord in patterson         PE70 225
 +NINT(MOD(R+100.,1.)*INC(3))*OFF(3)#  Get z coord in patterson         PE70 226
FOM=FOM+MIN(QX(MARKM+I),AW)*AW#        Sum weighted patt density        PE70 227
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 20 PE70 228
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 19 PE70 229
IF(FOM<0.)    GOTO 104#                Primary fom test                 PE70 230
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 18 PE70 231
FOM=FOM/SCAL#                          Normalize the fom value          PE70 232
IF(FOM<=TCUT) GOTO 105#                Reject if below cutoff           PE70 233
#                                                                       PE70 234
#    CHECK BEST LIST FOR EQUIVALENCE                                    PE70 235
#    -------------------------------                                    PE70 236
#                                                                       PE70 237
IF(IREF==no:)#                         In search mode only              PE70 238
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 21 PE70 239
NB=MARKB;   KB=0#                      Starting fom pointers            PE70 240
FOR(IB=MARKB;IB<JB;IB=IB+STEPB)#       Loop best transln solutions      PE70 241
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 22 PE70 242
IF(bf:<bf:(NB)) NB=IB#                 If fom smaller than test         PE70 243
DO I=1,MT#                             Loop over each coordinate        PE70 244
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 23 PE70 245
ifeq:(TMUL(I),0.) NEXT#                Test for polar axis              PE70 246
X=MOD(TS(I)-QX(IB+I+8)+VV(I),TMUL(I))# Get difference in shift unit     PE70 247
IF(ABS(UU(I)-X)>STEP(I)) GOTO 60#      Exit if coord not equivalent     PE70 248
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 23 PE70 249
#                                                                       PE70 250
bn:=bn:+1.#                            Count equivalent solution        PE70 251
##WRITE(6,'(A,2I5,9f8.4)')'     ',NT,IB,bn:,bf:,FOM,$#                  PE70 252
##QX(IB+9),QX(IB+10),QX(IB+11),TS(1),TS(2),TS(3)#                       PE70 253
IF(bf:<FOM)#                           If new fom is better             PE70 254
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 24 PE70 255
bf:=FOM#                               Store better fom                 PE70 256
DO I=1,6; QX(IB+I+8)=TS(I)#            Store new x,y,z shifts           PE70 257
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 24 PE70 258
IF(KB>0)#                              Already matched another shift    PE70 259
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 25 PE70 260
I1=IB; I2=KB#                          Store pointers matching sets     PE70 261
IF(bf:(KB)>bf:(IB))#                   Test for largest fom             PE70 262
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 26 PE70 263
I1=KB; I2=IB#                          Reverse order of pointers        PE70 264
$)#                                    5<<<<<<<<<<<<<<<<<<<<<<<<<<<< 26 PE70 265
bn:(I1)=bn:(KB)+bn:-1.;  bf:(I2)=-.01# Transfer to prev solution        PE70 266
KB=I1#                                 Transfer to this solution        PE70 267
##WRITE(6,'(A,2I5,6f8.4)')' ****',I1,I2,bn:(KB),bf:(KB),$#              PE70 268
##QX(KB+9),QX(KB+10),QX(KB+11)#                                         PE70 269
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 25 PE70 270
ELSE KB=IB#                            Save current position            PE70 271
60 CONTINUE#                           Escape for shift mismatch        PE70 272
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 22 PE70 273
#                                                                       PE70 274
IF(KB>0) NEXT#                         Skip if already matched          PE70 275
IF(JB==STOPBT)#                        Test for list limit              PE70 276
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 27 PE70 277
IB=NB; TCUT=bf:; bn:=1.#               If list full overwrite worst     PE70 278
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 27 PE70 279
ELSE#                                  Expand the best list             PE70 280
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 28 PE70 281
JB=JB+STEPB;    bn:=1.#                Incr best list stop              PE70 282
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 28 PE70 283
#                                                                       PE70 284
CFOM=1.#                               Combined figure of merit         PE70 285
IF(NTPR==yes:)#                        If triple sum applied            PE70 286
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 29 PE70 287
SUM1=0.;    SUM2=0.;    SUM3=0.#       Zero cfom sums                   PE70 288
FOR(IE=MARKE;IE<STOPE;IE=IE+STEPE)#    Loop over all e's                PE70 289
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 30 PE70 290
S=et:/TP#                              Get e(calc)/prob factor          PE70 291
Q=(ee:-S/SUMZ)#                        Get relative e difference        PE70 292
SUM1=SUM1+S**2#                        Sum e(calc)**2                   PE70 293
SUM2=SUM2+S*ee:#                       Sum e(obs)*e(calc)               PE70 294
SUM3=SUM3+MAX(Q,ABS(Q)*.2)#            Sum [e(obs)-e(calc)]             PE70 295
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 30 PE70 296
RE1=SUM3/SUME1#                        Calc fom e rfactor 1             PE70 297
RE2=SQRT(1.-SUM2**2/(SUM1*SUME2))#     Calc fom e rfactor 2             PE70 298
CFOM=0.1*SQRT(TPR)/(RE1*RE2)#          Combined figure of merit         PE70 299
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 29 PE70 300
#                                                                       PE70 301
bf:=FOM#                               Scale the transln fom            PE70 302
bm:=CFOM;        b1:=TPR#              Store new values                 PE70 303
b2:=SQRT(SMIN);  b3:=RFOM#             Store new values                 PE70 304
QX(IB+7)=RE1;    QX(IB+8)=RE2#         Store e rfactors                 PE70 305
DO I=1,6; QX(IB+I+8)=TS(I)#            Store new x,y,z shifts           PE70 306
##WRITE(6,'(A,2I5,8f8.4)')' new ',NT,IB,bn:(IB),bf:(IB),TS#             PE70 307
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 21 PE70 308
ELSE IF(FOM>bf:)#                      Not in refinement mode           PE70 309
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 31 PE70 310
bf:=FOM; b2:=SQRT(SMIN); bn:=bn:+1.#   Save the largest form            PE70 311
DO I=1,6; QX(IB+I+8)=TS(I)#            Save the best shifts             PE70 312
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 31 PE70 313
NEXT#                                                                   PE70 314
#                                                                       PE70 315
101 NDM=NDM+1;   NEXT#                 Count rejected d min             PE70 316
102 NF1=NF1+1;   NEXT#                 Count rejected fom pretest 1     PE70 317
103 NTS=NTS+1;   NEXT#                 Count rejected t3sum cutoff      PE70 318
104 NF2=NF2+1;   NEXT#                 Count rejected fom pretest 2     PE70 319
105 SCUT=MAX(SCUT,FOM); NFC=NFC+1; NEXT# Count rejected fom cutoff      PE70 320
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  PE70 321
#                                                                       PE70 322
#   PRINT REJECTION STATISTICS                                          PE70 323
#   --------------------------                                          PE70 324
#                                                                       PE70 325
IF(IREF==no:)#                         If close of search loop          PE70 326
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 32 PE70 327
ncodefld:(FLOAT(NDM),1,PE2,500613.,1)# Format rejections dmin           PE70 328
IF(NDM>0)writeline:(1,PE2,NPE2,3,3)#   Print rejections dmin            PE70 329
ncodefld:(FLOAT(NF1),1,PE3,500613.,1)# Format rejections fom pretest    PE70 330
IF(NF1>0)writeline:(0,PE3,NPE3,3,3)#   Print rejections fom pretest 1   PE70 331
ncodefld:(FLOAT(NTS),1,PE4,500613.,1)# Format rejections t3sum cutoff   PE70 332
IF(NTS>0)writeline:(0,PE4,NPE4,3,3)#   Print rejections t3sum cutoff    PE70 333
ncodefld:(FLOAT(NF2),1,PE5,500613.,1)# Format rejections fom pretest    PE70 334
IF(NF2>0)writeline:(0,PE5,NPE5,3,3)#   Print rejections fom pretest 2   PE70 335
ncodefld:(FLOAT(NFC),1,PE6,500613.,1)# Format rejections tfom  cutoff   PE70 336
IF(NFC>0)writeline:(0,PE6,NPE6,3,3)#   Print rejections tfom  cutoff    PE70 337
ncodefld:(TCUT,1,PE8,501552.,1)#       Format tfom threshold value      PE70 338
writeline:(1,PE8,NPE8,3,3)#            Print rejections tfom  cutoff    PE70 339
ncodefld:(SCUT,1,PE7,501552.,1)#       Format maximum form detected     PE70 340
writeline:(0,PE7,NPE7,3,3)#            Print rejections tfom  cutoff    PE70 341
IREF=yes:;   IB=MARKB-STEPB#           Start refinement and tries       PE70 342
DO I=1,6; ST(I)=MOD(STEP(I),1.)#       Store resolution range           PE70 343
DO I=1,6; RR(I)=ST(I)/1000.#           Store resolution range           PE70 344
LIM=0;       NTRY=NREF/4#              Cancel zero translation          PE70 345
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 32 PE70 346
#                                                                       PE70 347
#    REFINEMENT OF BEST TRANSLATIONS                                    PE70 348
#    -------------------------------                                    PE70 349
#                                                                       PE70 350
REPEAT#                                Loop over best solutions         PE70 351
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 33 PE70 352
IB=IB+STEPB#                           Loop over the best solution      PE70 353
IF(JB==IB) GOTO 200#                                                    PE70 354
DO I=1,6;  QQ(I)=QX(IB+I+8)-ST(I)*0.5# Best shifts with offset          PE70 355
##WRITE(6,'(i10,7f10.2)')IB,bf:,(QX(IB+I+8),I=1,6)#                     PE70 356
bn:=0.#                                Reset the hits count             PE70 357
IF(bf:>TCUT) BREAK#                    Start refinement if active       PE70 358
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 33 PE70 359
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  PE70 360
200 CONTINUE#                                                           PE70 361
FOR(IB=MARKB;IB<JB;IB=IB+STEPB)#       Loop best transln solutions      PE70 362
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 34 PE70 363
bm:=bm:*bf:*b3:#                       Calc combined fom                PE70 364
DO I=1,6#                              Loop over shifts                 PE70 365
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 35 PE70 366
X=QX(IB+I+8)-TMUL(I)#                  Get diff with cell modulus       PE70 367
IF(ABS(X)<STEP(I)) QX(IB+I+8)=X#       Reduce to lowest values          PE70 368
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 35 PE70 369
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 34 PE70 370
#                                                                       PE70 371
RETURN#                                                                 PE70 372
END#                                                                    PE70 373
systemheader:(PE75)#                   Calc & store combined fom's      PE75   1
####################################################################    PE75   2
#                                                                  #    PE75   3
#                                PE75                              #    PE75   4
#                                                                  #    PE75   5
####################################################################    PE75   6
SUBROUTINE PE75#                       S.r. hall mar 89                 PE75   7
include:(AACOMN)#                                                       PE75   8
include:(PECOMN)#                                                       PE75   9
#                                                                       PE75  10
REAL F#                                                                 PE75  11
INTEGER I,J,K#                                                          PE75  12
#                                                                       PE75  13
chardata:(PE1," ***** No translation solutions found *****")#           PE75  14
chardata:(PE2,"   Cfom    Tfom   t3sum   t3Re1   t3Re2   hits   $#      PE75  15
dmin    x/a    y/b    z/c     x/a    y/b    z/b")#                      PE75  16
chardata:(PE3,"                                                 $#      PE75  17
         R/T atom shifts        T atom shifts")#                        PE75  18
realdata:(FMT1,[220632.,530622.,450632.,150632.,450613.,#               PE75  19
70632.,310532.,390532.,#                                                PE75  20
610742.,680742.,750742.,830742.,900742.,970742.])#                      PE75  21
datastuff:#                                                             PE75  22
#                                                                       PE75  23
#  SORT BEST TRANSLATIONS ACCORDING TO CFOM                             PE75  24
#  ----------------------------------------                             PE75  25
#                                                                       PE75  26
IF(JB>MARKB) writeline:(0,PE3,NPE3,3,3)# Print shift header             PE75  27
K=0;  LINRM=10#                        Count transln solutions          PE75  28
FOR(IB=MARKB;IB<JB;IB=IB+STEPB)#       Loop best transln. solutions     PE75  29
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  PE75  30
F=bm:;      J=IB#                      Store starting sort values       PE75  31
FOR(I=IB+STEPB;I<JB;I=I+STEPB)#        Loop best rotation solutions     PE75  32
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  PE75  33
IF(bm:(I)<=F) NEXT#                    Skip smaller values              PE75  34
F=bm:(I);   J=I#                       Save largest soln address        PE75  35
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  PE75  36
IF(F<.001\K==TSOL) BREAK#              Exit if cancelled solution       PE75  37
K=K+1#                                 Count this solution              PE75  38
DO I=1,STEPB#                          Loop over list elements          PE75  39
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  PE75  40
F=QX(IB+I); QX(IB+I)=QX(J+I); QX(J+I)=F# Swap list elements             PE75  41
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  PE75  42
ncodefld:(b1:,1,CHROT,FMT1,14)#        Format the translation data      PE75  43
writeline:(0,PE2,NPE2,1,3)#            Print the translation data       PE75  44
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  PE75  45
IF(K==0) writeline:(1,PE1,NPE1,3,3)#   Tell user out of luck            PE75  46
JB=IB#                                 Reset solution list limit        PE75  47
#                                                                       PE75  48
#    ADD SOLUTIONS TO COMBINED LIST & RESORT                            PE75  49
#    ---------------------------------------                            PE75  50
#                                                                       PE75  51
IC=JC#                                 Point add-on part of comb list   PE75  52
FOR(IB=MARKB;IB<JB;IB=IB+STEPB)#       Loop best transln. solutions     PE75  53
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  PE75  54
DO I=1,STEPK; QX(IC+I)=QX(IB+I)#       Transfer contents tran list      PE75  55
IC=IC+STEPK#                           Incr the combined list           PE75  56
DO I=1,STEPK; QX(IC+I)=QX(IK+I)#       Transfer contents rotn list      PE75  57
IC=IC+STEPK#                           Incr the combined list           PE75  58
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  PE75  59
JC=IC#                                 Reset current list limit         PE75  60
FOR(IC=MARKC;IC<JC;IC=IC+STEPC)#       Loop best combined solutions     PE75  61
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  PE75  62
F=cc:;      J=IC#                      Store starting sort values       PE75  63
FOR(I=IC+STEPC;I<JC;I=I+STEPC)#        Loop best combined solutions     PE75  64
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  PE75  65
IF(cc:(I)<=F) NEXT#                    Skip smaller values              PE75  66
F=cc:(I);   J=I#                       Save largest soln address        PE75  67
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  PE75  68
ifeq:(F,0.) BREAK#                     Exit when fom is zero            PE75  69
DO I=1,STEPC#                          Loop over list elements          PE75  70
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  PE75  71
F=QX(IC+I); QX(IC+I)=QX(J+I); QX(J+I)=F# Swap list elements             PE75  72
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  PE75  73
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  PE75  74
IF(TWIS==no:) JC=MIN(IC,MARKC+STEPC*TSOL)# Limit solutions to list      PE75  75
ELSE          JC=MIN(IC,MARKC+2*STEPC*TSOL)# Limit solutions to half    PE75  76
#                                                                       PE75  77
RETURN#                                                                 PE75  78
END#                                                                    PE75  79
systemheader:(PE80)#                   Report final merged fom's        PE80   1
####################################################################    PE80   2
#                                                                  #    PE80   3
#                                PE80                              #    PE80   4
#                                                                  #    PE80   5
####################################################################    PE80   6
SUBROUTINE PE80#                       S.r. hall mar 89                 PE80   7
include:(AACOMN)#                                                       PE80   8
include:(PECOMN)#                                                       PE80   9
#                                                                       PE80  10
INTEGER I,IS,J,K,N#                                                     PE80  11
REAL CM(9),G(3),C,DEN,P,Q,R,S,SDEN,X,Y,Z#                               PE80  12
string:(PUNCH,80)#                                                      PE80  13
chardata:(PE1,"  ** Summary of best PATSEE solutions **")#              PE80  14
chardata:(PE2,"   Cfom   Tfom   Rfom  twist   phi1   phi2   phi3$#      PE80  15
    x/a    y/b    z/c     x/a    y/b    z/b")#                          PE80  16
chardata:(PE3,'  Coordinates of solutions output to file .see')#        PE80  17
chardata:(PE4,':             Solution 1  Cfom =                   ')#   PE80  18
chardata:(PE5,': atom fom')#                                            PE80  19
chardata:(PE6,"                                                 $#      PE80  20
    R/T atom shifts        T atom shifts")#                             PE80  21
chardata:(PE7,"peaks numb    ")#                                        PE80  22
chardata:(PE8,':compid       ')#                                        PE80  23
realdata:(FMT1,[210632.,140632.,70613.,70632.,630112.,#                 PE80  24
630112.,570742.,640742.,710742.,790742.,860742.,930742.])#              PE80  25
realdata:(FMT2,[350612.,420612.,490612.,280612.,280612.])#              PE80  26
realdata:(FMT3,[952.,952.,952.])#                                       PE80  27
datastuff:#                                                             PE80  28
#                                                                       PE80  29
LINRM=10#                              Ensure sufficient space          PE80  30
movechar:(PE1,4,CHROT,3,NPE1-2,2)#     Insert header boundary           PE80  31
writeline:(2,0,0,1,3)#                 Print header boundary            PE80  32
writeline:(0,PE1,NPE1,3,3)#            Tell user out of luck            PE80  33
movechar:(PE1,4,CHROT,3,NPE1-2,2)#     Insert header boundary           PE80  34
writeline:(0,0,0,1,3)#                 Print header boundary            PE80  35
writeline:(1,PE3,NPE3,3,3)#            Insert punch message             PE80  36
#                                                                       PE80  37
#   UPDATE COMBINED LIST IF ROTATION ONLY                               PE80  38
#   -------------------------------------                               PE80  39
#                                                                       PE80  40
IF(TRAN==no:)#                         Is there a translation search    PE80  41
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  PE80  42
JC=MARKC#                              Start the combined list          PE80  43
FOR(IK=MARKK;IK<JK;IK=IK+STEPK)#       Loop over rotation keep list     PE80  44
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  PE80  45
QX(JC+6)=kf:;   QX(JC+3)=kf:#          Store rotation fom as cfom       PE80  46
JC=JC+STEPK#                           Incr over transln items          PE80  47
DO I=1,STEPK; QX(JC+I)=QX(IK+I)#       Store keep values in combined    PE80  48
JC=JC+STEPK#                           Incr for rotation items          PE80  49
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  PE80  50
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  PE80  51
#                                                                       PE80  52
#  PRINT COMBINED SOLUTIONS IN ORDER OF CFOM                            PE80  53
#  -----------------------------------------                            PE80  54
#                                                                       PE80  55
FILEXT(iofm1:)='see'#                  Define fmt as atom line file     PE80  56
openfmt:(iofm1:,' ')#                  Open atom line file              PE80  57
PUNCH=':      PATSEE Solutions.                 '#                      PE80  58
movechar:(TITLE,18,PUNCH,30,8,0)#      Insert the date of solution      PE80  59
putsee:(PUNCH,53); putsee:(PE3,1)#     Space header                     PE80  60
movechar:(COMPID,1,PE8,9,6,0)#         Insert compound id               PE80  61
putsee:(PE8,14); putsee:(PE3,1)#       Space header                     PE80  62
ncodefld:(SUMN,1,PE7,140413.,1)#       Insert number missing sites      PE80  63
#                                                                       PE80  64
writeline:(0,PE6,NPE6,3,3);  LINRM=2#  Insert shift header              PE80  65
FOR(IC=MARKC;IC<JC;IC=IC+STEPC)#       Loop best transln. solutions     PE80  66
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  PE80  67
ncodefld:(QX,IC+3,CHROT,FMT1,12)#      Format the combined foms         PE80  68
ncodefld:(QX,IC+16,CHROT,FMT2,5)#      Format the rotation angles       PE80  69
writeline:(0,PE2,NPE2,1,3)#            Print the combined foms          PE80  70
#                                                                       PE80  71
#    GET FRACTIONAL COORD OF ROTATION SOLUTION                          PE80  72
#    -----------------------------------------                          PE80  73
#                                                                       PE80  74
IF(ROTN==yes:)#                        If rotation applied              PE80  75
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  PE80  76
FOR(IA=IATM(2);IA<JATM(2);IA=IA+STEPA)# Loop over rotated atoms         PE80  77
DO I=1,3; QX(IA+I+8)=QX(IA+I+5)#       Reset rotation coordinates       PE80  78
#                                                                       PE80  79
IF(TWIS>0)#                            Test if twist required           PE80  80
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  PE80  81
N=TWIS-STEPA;   TANG(3)=kt:(IC)*DTOR#  Reset twist skip flag            PE80  82
FOR(IA=TWIS+STEPA;IA<JATM(2);IA=IA+STEPA)# Loop over bot twist atoms    PE80  83
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  PE80  84
DO I=1,3; G(I)=QX(I+IA+8)-QX(I+N+8)#   Get displ from connect atom      PE80  85
DO I=1,3#                              Loop angles about x,y,z          PE80  86
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  PE80  87
J=MOD(I,3)+1;     K=MOD(I+1,3)+1#      Cyclic axis pointers             PE80  88
S=SIN(TANG(I));   C=COS(TANG(I))#      Sin and cos twist angle          PE80  89
X=G(J);           Y=G(K)#              Get cartesian coords             PE80  90
G(J)=X*C-Y*S;     G(K)=X*S+Y*C#        Get rotated coordinates          PE80  91
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  PE80  92
DO I=2,1,-1#                           Loop angles about x,y,z          PE80  93
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  PE80  94
J=MOD(I,3)+1;     K=MOD(I+1,3)+1#      Cyclic axis pointers             PE80  95
S=SIN(TANG(I));   C=COS(TANG(I))#      Sin and cos twist angle          PE80  96
X=G(J);           Y=G(K)#              Get cartesian coords             PE80  97
G(J)=X*C+Y*S;     G(K)=-X*S+Y*C#       Get rotated coordinates          PE80  98
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  PE80  99
DO I=1,3; QX(I+IA+8)=G(I)+QX(I+N+8)#   Get twisted atom position        PE80 100
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  PE80 101
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  PE80 102
#                                                                       PE80 103
DO I=1,9; CM(I)=QX(IC+I+20)#           Get rot/fract tranf matrix       PE80 104
FOR(IA=IATM(2);IA<JATM(2);IA=IA+STEPA)# Loop over rotated atoms         PE80 105
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  PE80 106
cx:=rx:*CM(1)+ry:*CM(2)+rz:*CM(3)+QX(IC+ 9)# Get new fractional x       PE80 107
cy:=rx:*CM(4)+ry:*CM(5)+rz:*CM(6)+QX(IC+10)# Get new fractional y       PE80 108
cz:=rx:*CM(7)+ry:*CM(8)+rz:*CM(9)+QX(IC+11)# Get new fractional z       PE80 109
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  PE80 110
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  PE80 111
#                                                                       PE80 112
#    SHIFT TRANSLATED COORDINATES                                       PE80 113
#    ----------------------------                                       PE80 114
#                                                                       PE80 115
FOR(IA=IATM(3);IA<JATM(3);IA=IA+STEPA)# Loop over shifted atoms         PE80 116
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 PE80 117
cx:=ax:+QX(IC+12)#                     Get shifted fractional x         PE80 118
cy:=ay:+QX(IC+13)#                     Get shifted fractional y         PE80 119
cz:=az:+QX(IC+14)#                     Get shifted fractional z         PE80 120
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 PE80 121
#                                                                       PE80 122
#   OUTPUT COORDINATES WITH WEIGHTS                                     PE80 123
#   -------------------------------                                     PE80 124
#                                                                       PE80 125
I=(IC-MARKC)/STEPC+1;   SDEN=0.#       Number of solution               PE80 126
ncodefld:(FLOAT(I),1,PE4,240213.,1)#   Format solution number           PE80 127
ncodefld:(cc:,1,PE4,400632.,1)#        Format combined fom              PE80 128
putsee:(PE4,51)#                       Output to punch file             PE80 129
PUNCH='ADDATM'; putsee:(PUNCH,6)#      Insert addatm header             PE80 130
#                                                                       PE80 131
FOR(IA=MARKA;IA<STOPA;IA=IA+STEPA)#    Loop over atom list              PE80 132
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 PE80 133
DEN=0.; X=cx:; Y=cy:; Z=cz:#           Store parent positions           PE80 134
FOR(IS=MARKS;IS<STOP1;IS=IS+STEPS)#    Loop symmetry operators          PE80 135
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 PE80 136
P=X*QX(IS+1)+Y*QX(IS+4)+Z*QX(IS+7)+QX(IS+10)# Get symoped x             PE80 137
Q=X*QX(IS+2)+Y*QX(IS+5)+Z*QX(IS+8)+QX(IS+11)# Get symoped y             PE80 138
R=X*QX(IS+3)+Y*QX(IS+6)+Z*QX(IS+9)+QX(IS+12)# Get symoped z             PE80 139
FOR(J=MARKA;J<STOPA;J=J+STEPA)#        Loop over atom list              PE80 140
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 PE80 141
IF(IA==J&IS==MARKS) NEXT#              Skip trivial vector              PE80 142
I=NINT(MOD(P-cx:(J)+100.,1.)*INC(1))*OFF(1)$# Get x coord patterson     PE80 143
 +NINT(MOD(Q-cy:(J)+100.,1.)*INC(2))*OFF(2)$# Get y coord patterson     PE80 144
 +NINT(MOD(R-cz:(J)+100.,1.)*INC(3))*OFF(3)# Get z coord patterson      PE80 145
DEN=DEN+MIN(QX(MARKM+I),aw:*aw:(J))#   Sum patt density                 PE80 146
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 PE80 147
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 PE80 148
PUNCH='atom'#                          Insert atom & blank line         PE80 149
movechar:(PE5,1,PUNCH,52,NPE5,0)#      Insert weight clause             PE80 150
ncodefld:(DEN,1,PUNCH,700902.,1)#      Format patterson density         PE80 151
movertoc:(an:,1,PUNCH,6,24,0)#         Insert atom name                 PE80 152
LCHROT=LABMAX+10#                      Set output pointer               PE80 153
ncodefld:(cx:,1,PUNCH,FMT3,3)#         Format x,y,z                     PE80 154
putsee:(PUNCH,80)#                     Output atom line to punch        PE80 155
SDEN=SDEN+DEN#                         Total atom fom's                 PE80 156
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 PE80 157
PUNCH=PE4(1^6)#                        Insert remark header             PE80 158
ncodefld:(SDEN,1,PUNCH,701002.,1)#     Format patterson density         PE80 159
putsee:(PUNCH,70)#                     Output atom line to punch        PE80 160
PUNCH='FC egt 1.2'; putsee:(PUNCH,10)# Insert fc header                 PE80 161
PUNCH='copybdf map ept'; putsee:(PUNCH,15)# Space header                PE80 162
PUNCH='FOURR emap'; putsee:(PUNCH,10)# Insert fourr header              PE80 163
PUNCH='coeff 1600 -1 -1 0 -1 -1 0 0 1700'# Special e phased fourr       PE80 164
putsee:(PUNCH,33)#                     Put special coef line?pe3        PE80 165
PUNCH='PEKPIK'; putsee:(PUNCH,6)#      Insert pekpik header             PE80 166
PUNCH='end'; putsee:(PUNCH,3)#         Insert edn line                  PE80 167
PUNCH='copybdf ept map'; putsee:(PUNCH,15)# Space header                PE80 168
PUNCH='PIG'; putsee:(PUNCH,3)#         Insert pig header                PE80 169
IF(SUMN>0.0)#                                                           PE80 170
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14 PE80 171
putsee:(PE7,14)#                       Insert extra peaks expected      PE80 172
putsee:(PE3,1)#                        Insert blank lines               PE80 173
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14 PE80 174
PUNCH='finish : remove to test all solutions'# Buffer finish line       PE80 175
IF(IC==MARKC) putsee:(PUNCH,37)#       Punch finish line once           PE80 176
putsee:(PE3,1)#                        Insert blank lines               PE80 177
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  PE80 178
#                                                                       PE80 179
closefmt:(iofm1:)#                     Close see file                   PE80 180
linestream:('see')#                    Spawn map/peak/pig stream        PE80 181
RETURN#                                                                 PE80 182
END#                                                                    PE80 183
pmacro:([no:])#                                                         PE80 184
